@@ -27,26 +27,29 @@ try:
     import MDAnalysis
     from MDAnalysis.topology.tpr import utils as tpr_utils, setting as tpr_setting
 except Exception:
-    logging.warn('Required module MDAnalysis not found.')
+    logging.warning('Required module MDAnalysis not found.')
     MDAnalysis = False
 
 from nomad.units import ureg
 from nomad.parsing.file_parser import TextParser, Quantity, FileParser
 from nomad.datamodel.metainfo.simulation.run import Run, Program, TimeRun
 from nomad.datamodel.metainfo.simulation.method import (
-    Method, ForceField, Model, Interaction, AtomParameters
+    NeighborSearching, ForceCalculations, Method, ForceField, Model, Interaction, AtomParameters
 )
 from nomad.datamodel.metainfo.simulation.system import (
     System, Atoms, AtomsGroup
 )
 from nomad.datamodel.metainfo.simulation.calculation import (
-    Calculation, Energy, EnergyEntry, Forces, ForcesEntry, Thermodynamics
+    Calculation, Energy, EnergyEntry, Forces, ForcesEntry,
+    # RadiusOfGyration, RadiusOfGyrationValues
 )
 from nomad.datamodel.metainfo.workflow import (
-    DiffusionConstantValues, MeanSquaredDisplacement, MeanSquaredDisplacementValues,
+    BarostatParameters, GeometryOptimization, ThermostatParameters, IntegrationParameters,
+    DiffusionConstantValues, MeanSquaredDisplacement, MeanSquaredDisplacementValues, MolecularDynamicsResults,
     RadialDistributionFunction, RadialDistributionFunctionValues,
     Workflow, MolecularDynamics
 )
+from nomad.datamodel.metainfo.simulation import workflow as workflow2
 from .metainfo.gromacs import x_gromacs_section_control_parameters, x_gromacs_section_input_output_files
 from atomisticparsers.utils import MDAnalysisParser
 
@@ -65,7 +68,7 @@ class GromacsLogParser(TextParser):
 
         def str_to_input_parameters(val_in):
             re_array = re.compile(r'\s*([\w\-]+)\[[\d ]+\]\s*=\s*\{*(.+)')
-            re_scalar = re.compile(r'\s*([\w\-]+)\s*=\s*(.+)')
+            re_scalar = re.compile(r'\s*([\w\-]+)\s*[=:]\s*(.+)')
             parameters = dict()
             val = val_in.strip().split('\n')
             for val_n in val:
@@ -128,12 +131,18 @@ class GromacsLogParser(TextParser):
             Quantity('execution_path', r'Executable:\s*(.+)'),
             Quantity('working_path', r'Data prefix:\s*(.+)'),
             # TODO cannot understand treatment of the command line in the old parser
+            # Quantity(
+            #     'header',
+            #     r'(?:GROMACS|Gromacs) (2019[\s\S]+?)\n\n', str_operation=str_to_header),
             Quantity(
                 'header',
                 r'(?:GROMACS|Gromacs) (version:[\s\S]+?)\n\n', str_operation=str_to_header),
             Quantity(
                 'input_parameters',
                 r'Input Parameters:\s*([\s\S]+?)\n\n', str_operation=str_to_input_parameters),
+            Quantity(
+                'maximum_force',
+                r'Norm of force\s*([\s\S]+?)\n\n', flatten=False),
             Quantity(
                 'step',
                 r'(Step\s*Time[\s\S]+?Energies[\s\S]+?\n\n)',
@@ -172,7 +181,7 @@ class GromacsLogParser(TextParser):
 
     def get_tpstat_settings(self):
         input_parameters = self.get('input_parameters', {})
-        target_T = input_parameters.get('ref-t', 0) * ureg.kelvin
+        target_t = input_parameters.get('ref-t', 0) * ureg.kelvin
 
         thermostat_type = None
         tcoupl = input_parameters.get('tcoupl', 'no').lower()
@@ -185,10 +194,10 @@ class GromacsLogParser(TextParser):
         # bd_fric = self.get('bd-fric', 0, unit='amu/ps')
         langevin_gamma = None
 
-        target_P = input_parameters.get('ref-p', 0) * ureg.bar
+        target_p = input_parameters.get('ref-p', 0) * ureg.bar
         # if P is array e.g. for non-isotropic pressures, get average since metainfo is float
-        if hasattr(target_P, 'shape'):
-            target_P = np.average(target_P)
+        if hasattr(target_p, 'shape'):
+            target_p = np.average(target_p)
 
         barostat_type = None
         pcoupl = input_parameters.get('pcoupl', 'no').lower()
@@ -198,8 +207,8 @@ class GromacsLogParser(TextParser):
         barostat_tau = input_parameters.get('tau-p', 0) * ureg.ps
 
         return dict(
-            target_T=target_T, thermostat_type=thermostat_type, thermostat_tau=thermostat_tau,
-            target_P=target_P, barostat_type=barostat_type, barostat_tau=barostat_tau,
+            target_t=target_t, thermostat_type=thermostat_type, thermostat_tau=thermostat_tau,
+            target_p=target_p, barostat_type=barostat_type, barostat_tau=barostat_tau,
             langevin_gamma=langevin_gamma)
 
 
@@ -277,7 +286,7 @@ class GromacsMDAnalysisParser(MDAnalysisParser):
         # copied from MDAnalysis.topology.tpr.utils
         # TODO maybe a better implementation exists
         if MDAnalysis.__version__ != '2.0.0':
-            self.logger.warn('Incompatible version of MDAnalysis.')
+            self.logger.warning('Incompatible version of MDAnalysis.')
 
         with open(self.mainfile, 'rb') as f:
             data = tpr_utils.TPXUnpacker(f.read())
@@ -572,6 +581,9 @@ class GromacsParser:
     def get_gromacs_file(self, ext):
         files = [d for d in self._gromacs_files if d.endswith(ext)]
 
+        if len(files) == 0:
+            return ''
+
         if len(files) == 1:
             return os.path.join(self._maindir, files[0])
 
@@ -608,11 +620,10 @@ class GromacsParser:
         # forces = self.traj_parser.get('forces')
         n_frames = self.traj_parser.get('n_frames')
         for n in range(n_frames):
-            if (n % self.frame_rate) > 0:
-                continue
             sec_scc = sec_run.m_create(Calculation)
-            sec_scc.forces = Forces(total=ForcesEntry(value=self.traj_parser.get_forces(n)))
-            sec_scc.system_ref = sec_run.system[n // self.frame_rate]
+            if (n % self.frame_rate) == 0:
+                sec_scc.forces = Forces(total=ForcesEntry(value=self.traj_parser.get_forces(n)))
+                sec_scc.system_ref = sec_run.system[n // self.frame_rate]
             sec_scc.method_ref = sec_run.method[-1]
 
         # get it from edr file
@@ -643,27 +654,23 @@ class GromacsParser:
 
         create_scc = False
         if n_frames != n_evaluations:
-            self.logger.warn(
+            self.logger.warning(
                 'Mismatch in number of calculations and number of thermodynamic '
                 'evaluations, will create new sections')
             create_scc = True
 
-        timestep = self.traj_parser.get('timestep')
-        if timestep is None:
-            timestep = self.log_parser.get('input_parameters', {}).get('dt', 1.0) * ureg.ps
+        time_step = self.traj_parser.get('timestep')
+        if time_step is None:
+            time_step = self.log_parser.get('input_parameters', {}).get('dt', 1.0) * ureg.ps
 
         # TODO add other energy contributions, properties
         energy_keys = ['LJ (SR)', 'Coulomb (SR)', 'Potential', 'Kinetic En.']
 
         for n in range(n_evaluations):
-            if (n % self.frame_rate) > 0:
-                continue
-
             if create_scc:
                 sec_scc = sec_run.m_create(Calculation)
             else:
-                sec_scc = sec_run.calculation[n // self._frame_rate]
-            sec_thermo = sec_scc.m_create(Thermodynamics)
+                sec_scc = sec_run.calculation[n]
             sec_energy = sec_scc.m_create(Energy)
             for key in thermo_data.keys():
                 val = thermo_data.get(key)[n]
@@ -673,17 +680,18 @@ class GromacsParser:
                 if key == 'Total Energy':
                     sec_energy.total = EnergyEntry(value=val)
                 elif key == 'Potential':
-                    sec_thermo.potential_energy = val
+                    sec_energy.potential = EnergyEntry(value=val)
                 elif key == 'Kinetic En.':
-                    sec_thermo.kinetic_energy = val
+                    sec_energy.kinetic = EnergyEntry(value=val)
                 elif key == 'Coulomb (SR)':
                     sec_energy.coulomb = EnergyEntry(value=val)
                 elif key == 'Pressure':
-                    sec_thermo.pressure = val
+                    sec_scc.pressure = val
                 elif key == 'Temperature':
-                    sec_thermo.temperature = val
+                    sec_scc.temperature = val
                 elif key == 'Time':
-                    sec_thermo.time_step = int((val / timestep).magnitude)
+                    sec_scc.time = val
+                    sec_scc.step = int((val / time_step).magnitude)
                 if key in energy_keys:
                     sec_energy.contributions.append(
                         EnergyEntry(kind=self._metainfo_mapping[key], value=val))
@@ -719,7 +727,7 @@ class GromacsParser:
                 sec_atoms.velocities = velocities
 
         # parse atomsgroup (segments --> molecules --> residues)
-        atoms_info = self.traj_parser._results['atom_info']
+        atoms_info = self.traj_parser._results['atoms_info']
         atoms_moltypes = np.array(atoms_info['moltypes'])
         atoms_molnums = np.array(atoms_info['molnums'])
         atoms_resids = np.array(atoms_info['resids'])
@@ -728,7 +736,6 @@ class GromacsParser:
         for segment in self.traj_parser.universe.segments:
             # we only create atomsgroup in the initial system
             sec_segment = sec_run.system[0].m_create(AtomsGroup)
-            sec_segment.label = segment.segid
             sec_segment.type = 'molecule_group'
             sec_segment.index = int(segment.segindex)
             sec_segment.atom_indices = segment.atoms.ix
@@ -744,6 +751,7 @@ class GromacsParser:
                 moltypes_count[moltype] = np.unique(mol_nums).shape[0]
             formula = ''.join([f'{moltype}({moltypes_count[moltype]})' for moltype in moltypes_count])
             sec_segment.composition_formula = formula
+            sec_segment.label = f'group_{moltypes[0]}'
 
             for i_molecule, molecule in enumerate(np.unique(atoms_molnums[sec_segment.atom_indices])):
                 sec_molecule = sec_segment.m_create(AtomsGroup)
@@ -762,18 +770,32 @@ class GromacsParser:
                     elements = atoms_elements[sec_molecule.atom_indices]
                     sec_molecule.composition_formula = get_composition(elements)
                 else:
-                    for i_res, res_id in enumerate(mol_resids):
-                        sec_residue = sec_molecule.m_create(AtomsGroup)
-                        sec_residue.index = i_res
-                        atom_indices = np.where(atoms_resids == res_id)[0]
-                        sec_residue.atom_indices = np.intersect1d(atom_indices, sec_molecule.atom_indices)
-                        sec_residue.n_atoms = len(sec_residue.atom_indices)
-                        res_names = atoms_resnames[sec_residue.atom_indices]
-                        sec_residue.label = str(res_names[0])
-                        sec_residue.type = 'monomer'
-                        sec_residue.is_molecule = False
-                        elements = atoms_elements[sec_residue.atom_indices]
-                        sec_residue.composition_formula = get_composition(elements)
+                    mol_resnames = atoms_resnames[sec_molecule.atom_indices]
+                    restypes = np.unique(mol_resnames)
+                    for i_restype, restype in enumerate(restypes):
+                        sec_monomer_group = sec_molecule.m_create(AtomsGroup)
+                        restype_indices = np.where(atoms_resnames == restype)[0]
+                        sec_monomer_group.label = f'group_{restype}'
+                        sec_monomer_group.type = 'monomer_group'
+                        sec_monomer_group.index = i_restype
+                        sec_monomer_group.atom_indices = np.intersect1d(restype_indices, sec_molecule.atom_indices)
+                        sec_monomer_group.n_atoms = len(sec_monomer_group.atom_indices)
+                        sec_monomer_group.is_molecule = False
+
+                        restype_resids = np.unique(atoms_resids[sec_monomer_group.atom_indices])
+                        restype_count = restype_resids.shape[0]
+                        sec_monomer_group.composition_formula = f'{restype}({restype_count})'
+                        for i_res, res_id in enumerate(restype_resids):
+                            sec_residue = sec_monomer_group.m_create(AtomsGroup)
+                            sec_residue.index = i_res
+                            atom_indices = np.where(atoms_resids == res_id)[0]
+                            sec_residue.atom_indices = np.intersect1d(atom_indices, sec_monomer_group.atom_indices)
+                            sec_residue.n_atoms = len(sec_residue.atom_indices)
+                            sec_residue.label = str(restype)
+                            sec_residue.type = 'monomer'
+                            sec_residue.is_molecule = False
+                            elements = atoms_elements[sec_residue.atom_indices]
+                            sec_residue.composition_formula = get_composition(elements)
 
                     names = atoms_resnames[sec_molecule.atom_indices]
                     ids = atoms_resids[sec_molecule.atom_indices]
@@ -786,68 +808,28 @@ class GromacsParser:
                     names_firstatom = names[ids_firstatom]
                     sec_molecule.composition_formula = get_composition(names_firstatom)
 
-        # calculate molecular radial distribution functions
-        sec_molecular_dynamics = self.archive.workflow[-1].molecular_dynamics
-        rdf_results = self.traj_parser.calc_molecular_rdf()
-        if rdf_results is not None:
-            sec_rdfs = sec_molecular_dynamics.m_create(RadialDistributionFunction)
-            sec_rdfs.type = 'Molecular'
-            sec_rdfs.n_smooth = rdf_results.get('n_smooth')
-            sec_rdfs.n_variables = 1
-            sec_rdfs.variables_name = np.array(['distance'])
-            for i_pair, pair_type in enumerate(rdf_results.get('types')):
-                sec_rdf_values = sec_rdfs.m_create(RadialDistributionFunctionValues)
-                sec_rdf_values.label = str(pair_type)
-                sec_rdf_values.n_bins = len(rdf_results['bins'][i_pair]) if rdf_results.get(
-                    'bins') is not None else []
-                sec_rdf_values.bins = rdf_results['bins'][i_pair] if rdf_results.get(
-                    'bins') is not None else []
-                sec_rdf_values.value = rdf_results['value'][i_pair] if rdf_results.get(
-                    'value') is not None else []
-
-        # calculate the molecular mean squared displacements
-        msd_results = self.traj_parser.calc_molecular_mean_squard_displacements()
-        if msd_results is not None:
-            sec_msds = sec_molecular_dynamics.m_create(MeanSquaredDisplacement)
-            sec_msds.type = 'Molecular'
-            for i_type, moltype in enumerate(msd_results.get('types')):
-                sec_msd_values = sec_msds.m_create(MeanSquaredDisplacementValues)
-                sec_msd_values.label = str(moltype)
-                sec_msd_values.n_times = len(msd_results['times'][i_type]) if msd_results.get(
-                    'times') is not None else []
-                sec_msd_values.times = msd_results['times'][i_type] if msd_results.get(
-                    'times') is not None else []
-                sec_msd_values.value = msd_results['value'][i_type] if msd_results.get(
-                    'value') is not None else []
-                sec_diffusion = sec_msd_values.m_create(DiffusionConstantValues)
-                sec_diffusion.value = msd_results['diffusion_constant'][i_type] if msd_results.get(
-                    'diffusion_constant') is not None else []
-                sec_diffusion.error_type = 'Pearson correlation coefficient'
-                sec_diffusion.errors = msd_results['error_diffusion_constant'][i_type] if msd_results.get(
-                    'error_diffusion_constant') is not None else []
-
     def parse_method(self):
         sec_method = self.archive.run[-1].m_create(Method)
         sec_force_field = sec_method.m_create(ForceField)
         sec_model = sec_force_field.m_create(Model)
         try:
-            n_atoms = self.traj_parser.get('n_atoms')
+            n_atoms = self.traj_parser.get('n_atoms', 0)
         except Exception:
             gro_file = self.get_gromacs_file('gro')
             self.traj_parser.mainfile = gro_file
             n_atoms = self.traj_parser.get('n_atoms', 0)
 
-        atom_info = self.traj_parser.get('atom_info', {})
+        atoms_info = self.traj_parser.get('atoms_info', {})
         for n in range(n_atoms):
             sec_atom = sec_method.m_create(AtomParameters)
-            sec_atom.charge = atom_info.get('charges', [None] * (n + 1))[n]
-            sec_atom.mass = atom_info.get('masses', [None] * (n + 1))[n]
-            sec_atom.label = atom_info.get('names', [None] * (n + 1))[n]
-            sec_atom.x_gromacs_atom_name = atom_info.get('atom_names', [None] * (n + 1))[n]
-            sec_atom.x_gromacs_atom_resid = atom_info.get('resids', [None] * (n + 1))[n]
-            sec_atom.x_gromacs_atom_resname = atom_info.get('resnames', [None] * (n + 1))[n]
-            sec_atom.x_gromacs_atom_molnum = atom_info.get('molnums', [None] * (n + 1))[n]
-            sec_atom.x_gromacs_atom_moltype = atom_info.get('moltypes', [None] * (n + 1))[n]
+            sec_atom.charge = atoms_info.get('charges', [None] * (n + 1))[n]
+            sec_atom.mass = atoms_info.get('masses', [None] * (n + 1))[n]
+            sec_atom.label = atoms_info.get('names', [None] * (n + 1))[n]
+            sec_atom.x_gromacs_atom_name = atoms_info.get('atom_names', [None] * (n + 1))[n]
+            sec_atom.x_gromacs_atom_resid = atoms_info.get('resids', [None] * (n + 1))[n]
+            sec_atom.x_gromacs_atom_resname = atoms_info.get('resnames', [None] * (n + 1))[n]
+            sec_atom.x_gromacs_atom_molnum = atoms_info.get('molnums', [None] * (n + 1))[n]
+            sec_atom.x_gromacs_atom_moltype = atoms_info.get('moltypes', [None] * (n + 1))[n]
 
         if n_atoms == 0:
             self.logger.error('Error parsing interactions.')
@@ -858,41 +840,234 @@ class GromacsParser:
             for key, val in interaction.items():
                 setattr(sec_interaction, key, val)
 
-    def parse_workflow(self):
-        sec_workflow = self.archive.m_create(Workflow)
-
-        sampling_settings = self.log_parser.get_sampling_settings()
-
-        sec_workflow.type = sampling_settings['sampling_method']
-        sec_md = sec_workflow.m_create(MolecularDynamics)
-        sec_md.ensemble_type = sampling_settings['ensemble_type']
-        sec_md.x_gromacs_integrator_type = sampling_settings['integrator_type']
-
         input_parameters = self.log_parser.get('input_parameters', {})
-        timestep = input_parameters.get('dt', 0)
-        sec_md.x_gromacs_integrator_dt = timestep
-        sec_md.timestep = timestep * 1e-12
+        sec_force_calculations = sec_force_field.m_create(ForceCalculations)
+        sec_neighbor_searching = sec_force_calculations.m_create(NeighborSearching)
 
-        nsteps = input_parameters.get('nsteps', 0)
-        sec_md.x_gromacs_number_of_steps_requested = nsteps
+        nstlist = input_parameters.get('nstlist', None)
+        sec_neighbor_searching.neighbor_update_frequency = int(nstlist) if nstlist else None
+        rlist = input_parameters.get('rlist', None)
+        sec_neighbor_searching.neighbor_update_cutoff = float(rlist) * ureg.nanometer if rlist else None
+        rvdw = input_parameters.get('rvdw', None)
+        sec_force_calculations.vdw_cutoff = float(rvdw) * ureg.nanometer if rvdw else None
+        coulombtype = input_parameters.get('coulombtype', 'no').lower()
+        coulombtype_map = {'cut-off': 'cutoff', 'ewald': 'ewald', 'pme': 'particle_mesh_ewald',
+                           'p3m-ad': 'particle_particle_particle_mesh', 'reaction-field': 'reaction_field',
+                           'shift': 'cutoff', 'switch': 'cutoff', 'user': 'cutoff'}
+        value = coulombtype_map.get(coulombtype, [val for key, val in coulombtype_map.items() if key in coulombtype])
+        value = value if not isinstance(value, list) else value[0] if len(value) != 0 else None
+        sec_force_calculations.coulomb_type = value
+        rcoulomb = input_parameters.get('rcoulomb', None)
+        sec_force_calculations.coulomb_cutoff = float(rcoulomb) if rcoulomb else None
 
-        tp_settings = self.log_parser.get_tpstat_settings()
+    def parse_workflow(self):
 
-        target_T = tp_settings.get('target_T', None)
-        if target_T is not None:
-            sec_md.x_gromacs_thermostat_target_temperature = target_T
-        thermostat_tau = tp_settings.get('thermostat_tau', None)
-        if thermostat_tau is not None:
-            sec_md.x_gromacs_thermostat_tau = thermostat_tau
-        target_P = tp_settings.get('target_P', None)
-        if target_P is not None:
-            sec_md.x_gromacs_barostat_target_pressure = target_P
-        barostat_tau = tp_settings.get('barostat_P', None)
-        if barostat_tau is not None:
-            sec_md.x_gromacs_barostat_tau = barostat_tau
-        langevin_gamma = tp_settings.get('langevin_gamma', None)
-        if langevin_gamma is not None:
-            sec_md.x_gromacs_langevin_gamma = langevin_gamma
+        sec_run = self.archive.run[-1]
+        sec_calc = sec_run.get('calculation')
+        sec_workflow = self.archive.m_create(Workflow)
+        input_parameters = self.log_parser.get('input_parameters', {})
+
+        workflow = None
+        integrator = input_parameters.get('integrator', 'md').lower()
+        if integrator in ['l-bfgs', 'cg', 'steep']:
+            sec_workflow.type = 'geometry_optimization'
+            sec_go = sec_workflow.m_create(GeometryOptimization)
+            workflow = workflow2.GeometryOptimization(
+                method=workflow2.GeometryOptimizationMethod(), results=workflow2.GeometryOptimizationResults())
+            sec_go.type = 'atomic'
+            workflow.method.type = 'atomic'
+            integrator_map = {'steep': 'steepest_descent', 'cg': 'conjugant_gradient',
+                              'l-bfgs': 'low_memory_broyden_fletcher_goldfarb_shanno'}
+            value = integrator_map.get(integrator, [val for key, val in integrator_map.items() if key in integrator])
+            value = value if not isinstance(value, list) else value[0] if len(value) != 0 else None
+            sec_go.method = value
+            workflow.method.method = value
+            nsteps = input_parameters.get('nsteps', None)
+            sec_go.optimization_steps_maximum = int(nsteps) if nsteps else None
+            workflow.method.optimization_steps_maximum = int(nsteps) if nsteps else None
+            nstenergy = input_parameters.get('nstenergy', None)
+            sec_go.save_frequency = int(nstenergy) if nstenergy else None
+
+            force_maximum = input_parameters.get('emtol', None)
+            force_conversion = ureg.convert(1.0, ureg.kilojoule * ureg.avogadro_number / ureg.nanometer, ureg.newton)
+            sec_go.convergence_tolerance_force_maximum = float(force_maximum) * force_conversion if force_maximum else None
+            workflow.method.convergence_tolerance_force_maximum = float(force_maximum) * force_conversion if force_maximum else None
+
+            energies = []
+            steps = []
+            for calc in sec_calc:
+                val = calc.get('energy')
+                energy = val.get('potential') if val else None
+                if energy:
+                    energies.append(energy.value.magnitude)
+                    step = calc.get('step')
+                    steps.append(step)
+            sec_go.energies = energies
+            sec_go.steps = steps
+            sec_go.optimization_steps = len(energies) + 1
+
+            final_force_maximum = self.log_parser.get('maximum_force')
+            final_force_maximum = float(re.split('=|\n', final_force_maximum)[1]) if final_force_maximum else None
+            sec_go.final_force_maximum = float(final_force_maximum) * force_conversion if final_force_maximum else None
+            workflow.results.final_force_maximum = float(final_force_maximum) * force_conversion if final_force_maximum else None
+        else:
+            sec_workflow.type = 'molecular_dynamics'
+            sec_md = sec_workflow.m_create(MolecularDynamics)
+            workflow = workflow2.MolecularDynamics(
+                method=workflow2.MolecularDynamicsMethod(
+                    thermostat_parameters=workflow2.ThermostatParameters(),
+                    barostat_parameters=workflow2.BarostatParameters()
+                ), results=workflow2.MolecularDynamicsResults()
+            )
+
+            sec_integration_parameters = sec_md.m_create(IntegrationParameters)
+            nsteps = input_parameters.get('nsteps', None)
+            sec_integration_parameters.n_steps = int(nsteps) if nsteps else None
+            workflow.method.n_steps = int(nsteps) if nsteps else None
+            nstxout = input_parameters.get('nstxout', None)
+            sec_integration_parameters.coordinate_save_frequency = int(nstxout) if nstxout else None
+            workflow.method.coordinate_save_frequency = int(nstxout) if nstxout else None
+            nstvout = input_parameters.get('nstvout', None)
+            sec_integration_parameters.velocity_save_frequency = int(nstvout) if nstvout else None
+            workflow.method.velocity_save_frequency = int(nstvout) if nstvout else None
+            nstfout = input_parameters.get('nstfout', None)
+            sec_integration_parameters.force_save_frequency = int(nstfout) if nstfout else None
+            workflow.method.force_save_frequency = int(nstfout) if nstfout else None
+            nstenergy = input_parameters.get('nstenergy', None)
+            sec_integration_parameters.thermodynamics_save_frequency = int(nstenergy) if nstenergy else None
+            workflow.method.thermodynamics_save_frequency = int(nstenergy) if nstenergy else None
+
+            integrator_map = {'md': 'leap_frog', 'md-vv': 'velocity_verlet', 'sd': 'langevin_goga',
+                              'bd': 'brownian'}
+            value = integrator_map.get(integrator, [val for key, val in integrator_map.items() if key in integrator])
+            value = value if not isinstance(value, list) else value[0] if len(value) != 0 else None
+            sec_integration_parameters.integrator_type = value
+            workflow.method.integrator_type = value
+            timestep = input_parameters.get('dt', None)
+            sec_integration_parameters.integration_timestep = float(timestep) * ureg.picosecond if timestep else None
+            workflow.method.integration_timestep = float(timestep) * ureg.picosecond if timestep else None
+            sec_thermostat_parameters = sec_integration_parameters.m_create(ThermostatParameters)
+            sec_barostat_parameters = sec_integration_parameters.m_create(BarostatParameters)
+
+            flag_thermostat = False
+            thermostat = input_parameters.get('tcoupl', 'no').lower()
+            thermostat_map = {'berendsen': 'berendsen', 'v-rescale': 'velocity_rescaling',
+                              'nose-hoover': 'nose_hoover', 'andersen': 'andersen'}
+            value = thermostat_map.get(thermostat, [val for key, val in thermostat_map.items() if key in thermostat])
+            value = value if not isinstance(value, list) else value[0] if len(value) != 0 else None
+            sec_thermostat_parameters.thermostat_type = value
+            workflow.method.thermostat_parameters.thermostat_type = value
+            if 'sd' in integrator:
+                sec_thermostat_parameters.thermostat_type = 'langevin_goga'
+                workflow.method.thermostat_parameters.thermostat_type = 'langevin_goga'
+            if sec_thermostat_parameters.thermostat_type:
+                flag_thermostat = True
+                reference_temperature = input_parameters.get('ref-t', None)
+                if isinstance(reference_temperature, str):
+                    reference_temperature = float(reference_temperature.split()[0])
+                reference_temperature *= ureg.kelvin if reference_temperature else None
+                sec_thermostat_parameters.reference_temperature = reference_temperature
+                workflow.method.thermostat_parameters.reference_temperature = reference_temperature
+                coupling_constant = input_parameters.get('tau-t', None)
+                if isinstance(coupling_constant, str):
+                    coupling_constant = float(coupling_constant.split()[0])
+                coupling_constant *= ureg.picosecond if coupling_constant else None
+                sec_thermostat_parameters.coupling_constant = coupling_constant
+                workflow.method.thermostat_parameters.coupling_constant = coupling_constant
+
+            flag_barostat = False
+            barostat = input_parameters.get('pcoupl', 'no').lower()
+            barostat_map = {'berendsen': 'berendsen', 'parrinello-rahman': 'parrinello_rahman',
+                            'mttk': 'martyna_tuckerman_tobias_klein', 'c-rescale': 'stochastic_cell_rescaling'}
+            value = barostat_map.get(barostat, [val for key, val in barostat_map.items() if key in barostat])
+            value = value if not isinstance(value, list) else value[0] if len(value) != 0 else None
+            sec_barostat_parameters.barostat_type = value
+            workflow.method.barostat_parameters.barostat_type = value
+            if sec_barostat_parameters.barostat_type:
+                flag_barostat = True
+                couplingtype = input_parameters.get('pcoupltype', None).lower()
+                couplingtype_map = {'isotropic': 'isotropic', 'semiisotropic': 'semi_isotropic',
+                                    'anisotropic': 'anisotropic'}
+                value = couplingtype_map.get(couplingtype, [val for key, val in couplingtype_map.items() if key in couplingtype])
+                sec_barostat_parameters.coupling_type = value[0] if isinstance(value, list) else value
+                workflow.method.barostat_parameters.coupling_type = value[0] if isinstance(value, list) else value
+                taup = input_parameters.get('tau-p', None)
+                sec_barostat_parameters.coupling_constant = np.ones(shape=(3, 3)) * float(taup) * ureg.picosecond if taup else None
+                workflow.method.barostat_parameters.coupling_constant = np.ones(shape=(3, 3)) * float(taup) * ureg.picosecond if taup else None
+                refp = input_parameters.get('ref-p', None)
+                sec_barostat_parameters.reference_pressure = refp * ureg.bar if refp is not None else None
+                workflow.method.barostat_parameters.reference_pressure = refp * ureg.bar if refp is not None else None
+                compressibility = input_parameters.get('compressibility', None)
+                sec_barostat_parameters.compressibility = compressibility * (1. / ureg.bar) if compressibility is not None else None
+                workflow.method.barostat_parameters.compressibility = compressibility * (1. / ureg.bar) if compressibility is not None else None
+
+            if flag_thermostat:
+                sec_md.thermodynamic_ensemble = 'NPT' if flag_barostat else 'NVT'
+                workflow.method.thermodynamic_ensemble = 'NPT' if flag_barostat else 'NVT'
+            elif flag_barostat:
+                sec_md.thermodynamic_ensemble = 'NPH'
+                workflow.method.thermodynamic_ensemble = 'NPH'
+            else:
+                sec_md.thermodynamic_ensemble = 'NVE'
+                workflow.method.thermodynamic_ensemble = 'NVE'
+
+            # calculate molecular radial distribution functions
+            sec_molecular_dynamics = self.archive.workflow[-1].molecular_dynamics
+            sec_results = sec_molecular_dynamics.m_create(MolecularDynamicsResults)
+            n_traj_split = 10  # number of intervals to split trajectory into for averaging
+            interval_indices = []  # 2D array specifying the groups of the n_traj_split intervals to be averaged
+            # first 20% of trajectory
+            interval_indices.append(np.arange(int(n_traj_split * 0.20)))
+            # last 80% of trajectory
+            interval_indices.append(np.arange(n_traj_split)[len(interval_indices[0]):])
+            # last 60% of trajectory
+            interval_indices.append(np.arange(n_traj_split)[len(interval_indices[0]) * 2:])
+            # last 40% of trajectory
+            interval_indices.append(np.arange(n_traj_split)[len(interval_indices[0]) * 3:])
+
+            rdf_results = self.traj_parser.calc_molecular_rdf(n_traj_split=n_traj_split, n_prune=self._frame_rate, interval_indices=interval_indices)
+            if rdf_results is not None:
+                sec_rdfs = sec_results.m_create(RadialDistributionFunction)
+                sec_rdfs.type = 'molecular'
+                sec_rdfs.n_smooth = rdf_results.get('n_smooth')
+                sec_rdfs.n_prune = self._frame_rate
+                sec_rdfs.n_variables = 1
+                sec_rdfs.variables_name = np.array(['distance'])
+                for i_pair, pair_type in enumerate(rdf_results.get('types', [])):
+                    sec_rdf_values = sec_rdfs.m_create(RadialDistributionFunctionValues)
+                    sec_rdf_values.label = str(pair_type)
+                    sec_rdf_values.n_bins = len(rdf_results.get('bins', [[]] * i_pair)[i_pair])
+                    sec_rdf_values.bins = rdf_results['bins'][i_pair] if rdf_results.get(
+                        'bins') is not None else []
+                    sec_rdf_values.value = rdf_results['value'][i_pair] if rdf_results.get(
+                        'value') is not None else []
+                    sec_rdf_values.frame_start = rdf_results['frame_start'][i_pair] if rdf_results.get(
+                        'frame_start') is not None else []
+                    sec_rdf_values.frame_end = rdf_results['frame_end'][i_pair] if rdf_results.get(
+                        'frame_end') is not None else []
+
+            # calculate the molecular mean squared displacements
+            msd_results = self.traj_parser.calc_molecular_mean_squared_displacements()
+            if msd_results is not None:
+                sec_msds = sec_results.m_create(MeanSquaredDisplacement)
+                sec_msds.type = 'molecular'
+                sec_msds.direction = 'xyz'
+                for i_type, moltype in enumerate(msd_results.get('types', [])):
+                    sec_msd_values = sec_msds.m_create(MeanSquaredDisplacementValues)
+                    sec_msd_values.label = str(moltype)
+                    sec_msd_values.n_times = len(msd_results.get('times', [[]] * i_type)[i_type])
+                    sec_msd_values.times = msd_results['times'][i_type] if msd_results.get(
+                        'times') is not None else []
+                    sec_msd_values.value = msd_results['value'][i_type] if msd_results.get(
+                        'value') is not None else []
+                    sec_diffusion = sec_msd_values.m_create(DiffusionConstantValues)
+                    sec_diffusion.value = msd_results['diffusion_constant'][i_type] if msd_results.get(
+                        'diffusion_constant') is not None else []
+                    sec_diffusion.error_type = 'Pearson correlation coefficient'
+                    sec_diffusion.errors = msd_results['error_diffusion_constant'][i_type] if msd_results.get(
+                        'error_diffusion_constant') is not None else []
+
+        self.archive.workflow2 = workflow
 
     def parse_input(self):
         sec_run = self.archive.run[-1]
@@ -943,7 +1118,7 @@ class GromacsParser:
 
         header = self.log_parser.get('header', {})
         sec_run.program = Program(
-            name='GROMACS', version=header.get('version', 'unknown').lstrip('VERSION '))
+            name='GROMACS', version=str(header.get('version', 'unknown')).lstrip('VERSION '))
 
         sec_time_run = sec_run.m_create(TimeRun)
         for key in ['start', 'end']:
@@ -961,14 +1136,22 @@ class GromacsParser:
 
         topology_file = self.get_gromacs_file('tpr')
         # I have no idea if output trajectory file can be specified in input
-        trajectory_file = self.get_gromacs_file('trr')
+        trr_file = self.get_gromacs_file('trr')
+        trr_file_nopath = trr_file.rsplit('.', 1)[0]
+        trr_file_nopath = trr_file_nopath.rsplit('/')[-1]
+        if not trr_file_nopath.startswith(self._basename):
+            xtc_file = self.get_gromacs_file('xtc')
+            xtc_file_nopath = xtc_file.rsplit('.', 1)[0]
+            xtc_file_nopath = xtc_file_nopath.rsplit('/')[-1]
+            trajectory_file = xtc_file if xtc_file_nopath.startswith(self._basename) else trr_file
+        else:
+            trajectory_file = trr_file
+
         self.traj_parser.mainfile = topology_file
         self.traj_parser.auxilliary_files = [trajectory_file]
 
         self.parse_method()
 
-        self.parse_workflow()
-
         self.parse_system()
 
         # TODO read also from ene
@@ -978,3 +1161,5 @@ class GromacsParser:
         self.parse_thermodynamic_data()
 
         self.parse_input()
+
+        self.parse_workflow()
