@@ -40,6 +40,10 @@ from nomad.datamodel.metainfo.simulation.calculation import (
 from nomad.datamodel.metainfo.workflow import (
     Workflow, GeometryOptimization, MolecularDynamics
 )
+from nomad.datamodel.metainfo.simulation.workflow import (
+    GeometryOptimization as GeometryOptimization2, GeometryOptimizationMethod,
+    MolecularDynamics as MolecularDynamics2, MolecularDynamicsMethod
+)
 
 from .metainfo.castep import x_castep_section_phonons, x_castep_section_scf_parameters,\
     x_castep_section_density_mixing_parameters, x_castep_section_population_analysis_parameters,\
@@ -569,7 +573,7 @@ class CastepParser:
 
         self._xc_functional_map = {
             'Perdew Burke Ernzerhof': ['GGA_X_PBE', 'GGA_C_PBE'],
-            'Local Density Approximation': ['LDA_C_PZ_LDA_X_PZ'],
+            'Local Density Approximation': ['LDA_C_PZ', 'LDA_X_PZ'],
             'Perdew Wang (1991)': ['GGA_X_PW91', 'GGA_C_PW91'],
             'revised Perdew Burke Ernzerhof': ['GGA_X_RPBE'],
             'PBE for solids (2008)': ['GGA_X_PBE_SOL'],
@@ -822,26 +826,35 @@ class CastepParser:
 
     def parse_workflow(self):
         sec_workflow = self.archive.m_create(Workflow)
+        workflow = None
 
         title = self.out_parser.get('title', {})
 
         method = title.get('general parameters', {}).get('type of calculation', 'single point')
         method = self._sampling_method_map.get(method)
         if method is None:
-            self.logger.warn('Sampling method cannot be resolved.')
+            self.logger.warning('Sampling method cannot be resolved.')
             method = 'single_point'
         sec_workflow.type = method
 
         if method == 'geometry_optimization':
             sec_geometry_opt = sec_workflow.m_create(GeometryOptimization)
-            for key, val in title.get('geometry optimization parameters', {}).items():
-                key = self._metainfo_map.get('geometry optimization parameters').get(key)
+            parameters = title.get('geometry optimization parameters', {})
+            key_map = self._metainfo_map.get('geometry optimization parameters')
+            for key, val in parameters.items():
+                key = key_map.get(key)
                 if key is None or val is None:
                     continue
                 setattr(sec_geometry_opt, key, val)
+            workflow = GeometryOptimization2(method=GeometryOptimizationMethod())
+            workflow.method.convergence_tolerance_displacement_maximum = parameters.get('max ionic |displacement| tolerance')
+            workflow.method.convergence_tolerance_energy_difference = parameters.get('total energy convergence tolerance')
+            workflow.method.convergence_tolerance_force_maximum = parameters.get('max ionic |force| tolerance')
         elif method == 'molecular_dynamics':
             sec_md = sec_workflow.m_create(MolecularDynamics)
-            for key, val in title.get('molecular dynamics parameters', {}).items():
+            parameters = title.get('molecular dynamics parameters', {})
+            workflow = MolecularDynamics2(method=MolecularDynamicsMethod())
+            for key, val in parameters.items():
                 key = self._metainfo_map.get('molecular dynamics parameters').get(key)
                 if key is None or val is None:
                     continue
@@ -850,6 +863,9 @@ class CastepParser:
                         setattr(sec_md, key_i, val[i])
                 else:
                     setattr(sec_md, key, val)
+            workflow.method.thermodynamic_ensemble = parameters.get('ensemble')
+
+        self.archive.workflow2 = workflow
 
     def parse_configurations(self):
         calculation = self.out_parser.get('calculation')
