@@ -29,6 +29,7 @@ from nomad.datamodel.metainfo.simulation.method import Method
 from nomad.datamodel.metainfo.simulation.system import System, Atoms
 from nomad.datamodel.metainfo.simulation.calculation import Calculation
 from nomad.datamodel.metainfo.workflow import Workflow, Elastic, StrainDiagrams
+from nomad.datamodel.metainfo.simulation import workflow as workflow2
 
 from .metainfo.elastic import x_elastic_section_fitting_parameters
 
@@ -510,6 +511,11 @@ class ElasticParser:
             sec_strain_diagram.n_eta = len(strain[0])
             sec_strain_diagram.eta = strain
             sec_strain_diagram.value = energy
+            sec_strain_diagram2 = self.archive.workflow2.results.m_create(workflow2.StrainDiagrams)
+            sec_strain_diagram2.type = 'energy'
+            sec_strain_diagram2.n_eta = len(strain[0])
+            sec_strain_diagram2.eta = strain
+            sec_strain_diagram2.value = energy
 
             energy_fit = self.get_energy_fit()
             if not energy_fit:
@@ -524,6 +530,12 @@ class ElasticParser:
                     sec_strain_diagram.n_eta = poly_fit.get(fit_order, None)
                     sec_strain_diagram.eta = energy_fit[diagram_type][0][fit_order]
                     sec_strain_diagram.value = energy_fit[diagram_type][1][fit_order]
+                    sec_strain_diagram2 = self.archive.workflow2.results.m_create(workflow2.StrainDiagrams)
+                    sec_strain_diagram2.type = diagram_type
+                    sec_strain_diagram2.polynomial_fit_order = int(fit_order[:-2])
+                    sec_strain_diagram2.n_eta = poly_fit.get(fit_order, None)
+                    sec_strain_diagram2.eta = energy_fit[diagram_type][0][fit_order]
+                    sec_strain_diagram2.value = energy_fit[diagram_type][1][fit_order]
 
         elif method == 'stress':
             strain, stress = self.get_strain_stress()
@@ -540,6 +552,12 @@ class ElasticParser:
                     sec_strain_diagram.n_eta = len(strain_i[0])
                     sec_strain_diagram.eta = strain_i
                     sec_strain_diagram.value = stress_i[si]
+                    sec_strain_diagram2 = self.archive.workflow2.results.m_create(workflow2.StrainDiagrams)
+                    sec_strain_diagram2.type = diagram_type
+                    sec_strain_diagram2.stress_voigt_component = si + 1
+                    sec_strain_diagram2.n_eta = len(strain_i[0])
+                    sec_strain_diagram2.eta = strain_i
+                    sec_strain_diagram2.value = stress_i[si]
 
             stress_fit = self.get_stress_fit()
             for diagram_type in ['cross-validation', 'dtn']:
@@ -554,9 +572,17 @@ class ElasticParser:
                         sec_strain_diagram.n_eta = poly_fit.get(fit_order, None)
                         sec_strain_diagram.eta = stress_fit[diagram_type][si][0][fit_order]
                         sec_strain_diagram.value = np.array(stress_fit[diagram_type][si][1][fit_order])
+                        sec_strain_diagram2 = self.archive.workflow2.results.m_create(workflow2.StrainDiagrams)
+                        sec_strain_diagram2.type = diagram_type
+                        sec_strain_diagram2.stress_voigt_component = si + 1
+                        sec_strain_diagram2.polynomial_fit_order = int(fit_order[:-2])
+                        sec_strain_diagram2.n_eta = poly_fit.get(fit_order, None)
+                        sec_strain_diagram2.eta = stress_fit[diagram_type][si][0][fit_order]
+                        sec_strain_diagram2.value = np.array(stress_fit[diagram_type][si][1][fit_order])
 
     def parse_elastic_constant(self):
         sec_elastic = self.archive.workflow[0].elastic
+        sec_results = self.archive.workflow2.results
 
         order = self.info['order']
 
@@ -584,10 +610,33 @@ class ElasticParser:
 
             sec_elastic.eigenvalues_elastic = eigenvalues
 
+            sec_results.elastic_constants_notation_matrix_second_order = matrices.get('voigt')
+            sec_results.elastic_constants_matrix_second_order = matrices.get('elastic_constant')
+            sec_results.compliance_matrix_second_order = matrices.get('compliance')
+
+            sec_results.bulk_modulus_voigt = moduli.get('B_V', moduli.get('K_V'))
+            sec_results.shear_modulus_voigt = moduli.get('G_V')
+
+            sec_results.bulk_modulus_reuss = moduli.get('B_R', moduli.get('K_R'))
+            sec_results.shear_modulus_reuss = moduli.get('G_R')
+
+            sec_results.bulk_modulus_hill = moduli.get('B_H', moduli.get('K_H'))
+            sec_results.shear_modulus_hill = moduli.get('G_H')
+
+            sec_results.young_modulus_voigt = moduli.get('E_V')
+            sec_results.poisson_ratio_voigt = moduli.get('nu_V')
+            sec_results.young_modulus_reuss = moduli.get('E_R')
+            sec_results.poisson_ratio_reuss = moduli.get('nu_R')
+            sec_results.young_modulus_hill = moduli.get('E_H')
+            sec_results.poisson_ratio_hill = moduli.get('nu_H')
+
+            sec_results.eigenvalues_elastic = eigenvalues
+
         elif order == 3:
             elastic_constant = self.get_elastic_constants_order3()
             if elastic_constant is not None:
                 sec_elastic.elastic_constants_matrix_third_order = elastic_constant * ureg.GPa
+                sec_results.elastic_constants_matrix_third_order = elastic_constant * ureg.GPa
 
     def init_parser(self):
         self._deform_dirs = None
@@ -661,5 +710,16 @@ class ElasticParser:
         sec_elastic.n_deformations = len(self.deformation_dirs)
         sec_elastic.deformation_types = deformation_types
 
+        workflow = workflow2.Elastic(
+            method=workflow2.ElasticMethod(), results=workflow2.ElasticResults())
+        workflow.method.energy_stress_calculator = self.info['code_name']
+        workflow.method.calculation_method = self.info['calculation_method'].lower()
+        workflow.method.elastic_constants_order = self.info['order']
+        workflow.method.strain_maximum = self.info['max_strain']
+        workflow.results.n_strains = self.info['n_strains']
+        workflow.results.n_deformations = len(self.deformation_dirs)
+        workflow.results.deformation_types = deformation_types
+        self.archive.workflow2 = workflow
+
         self.parse_strain()
         self.parse_elastic_constant()
