@@ -45,7 +45,7 @@ from nomad.metainfo.elasticsearch_extension import DocumentType, material_entry_
 from .utils import parameter_dependency_from_model, update_url_query_arguments
 
 
-User = datamodel.User.m_def.a_pydantic.model
+User: Any = datamodel.User.m_def.a_pydantic.model
 # It is important that datetime.datetime comes last. Otherwise, number valued strings
 # are interpreted as epoch dates by pydantic
 Value = Union[StrictInt, StrictFloat, StrictBool, str, datetime.datetime]
@@ -300,6 +300,17 @@ on the optimade filter language:
 ''')
 
 
+def restrict_query_to_upload(query: Query, upload_id: str):
+    ''' Utility for restricting an arbitrary Query to a single upload. '''
+    if isinstance(query, Mapping) and query.keys() == {'entry_id'} and isinstance(query['entry_id'], All):
+        # Special case. We want to allow this type of queries, but for some reason, ES will
+        # not find anything if we just "and" the query with an upload_id criteria as usual.
+        # Instead, for it to work, the "All" must be changed to an "Any" operation.
+        values = query['entry_id'].op
+        return And(**{'and': [{'upload_id': upload_id}, {'entry_id': Any_(any=values)}]})
+    return And(**{'and': [{'upload_id': upload_id}, query]})
+
+
 class WithQuery(BaseModel):
     owner: Optional[Owner] = Body('public')
     query: Optional[Query] = Body(
@@ -871,14 +882,71 @@ class TermsAggregation(BucketAggregation):
         '''))
 
 
+class Bounds(BaseModel):
+    min: float = Field(
+        description=strip('''
+        Start value for the histogram.
+        '''))
+    max: float = Field(
+        description=strip('''
+        Ending value for the histogram.
+        '''))
+
+    @root_validator
+    def check_order(cls, values):  # pylint: disable=no-self-argument
+        min = values.get('min')
+        max = values.get('max')
+        if min > max:
+            raise ValueError('The maximum value should be greater than the minimum value.')
+        return values
+
+
 class HistogramAggregation(BucketAggregation):
-    interval: pydantic.confloat(gt=0)  # type: ignore
+    interval: float = Field(
+        None,
+        gt=0,
+        description=strip('''
+            The histogram bucketing interval. Provide either this or the number
+            of buckets.
+        '''))
+    buckets: int = Field(
+        None,
+        gt=0,
+        description=strip('''
+            The number of buckets to use. Provide either this or the interval.
+            The interval for the bucketing is automatically chosen to achieve
+            the target number of buckets. Notice that for integer data types
+            (int, long, date), the returned number of buckets may be smaller
+            than the requested amount if the histogram range is smaller than
+            the number of buckets. This setting will override the interval.
+        '''))
+    offset: float = Field(
+        None,
+        gte=0)
+    extended_bounds: Optional[Bounds]
+
+    @root_validator
+    def check_bucketing(cls, values):  # pylint: disable=no-self-argument
+        interval = values.get('interval')
+        buckets = values.get('buckets')
+        if interval is None and buckets is None:
+            raise ValueError('Provide either interval or the number of buckets.')
+        return values
 
 
 class DateHistogramAggregation(BucketAggregation):
     interval: str = Field('1M')  # type: ignore
 
 
+class AutoDateHistogramAggregation(BucketAggregation):
+    buckets: int = Field(
+        10, description=strip('''
+        The target number of buckets. The interval of the buckets is
+        automatically chosen to best achieve that target. The number of buckets
+        returned will always be less than or equal to this target number.
+        '''))
+
+
 class MinMaxAggregation(QuantityAggregation):
     pass
 
@@ -965,7 +1033,7 @@ class Aggregation(BaseModel):
             {
                 "aggregations": {
                     "upload_create_times": {
-                        "histogram": {
+                        "date_histogram": {
                             "quantity": "upload_create_time",
                             "interval": "1M"
                         }
@@ -978,6 +1046,32 @@ class Aggregation(BaseModel):
             that determine a time period. The default is one month, `1M`.
         '''))
 
+    auto_date_histogram: Optional[AutoDateHistogramAggregation] = Body(
+        None,
+        description=strip('''
+            A `auto_date_histogram` aggregation is like a normal
+            `date_histogram` but instead of providing an explicit time step,
+            you can provide a target number of buckets.:
+
+            ```json
+            {
+                "aggregations": {
+                    "upload_create_times": {
+                        "auto_date_histogram": {
+                            "quantity": "upload_create_time",
+                            "buckets": "10"
+                        }
+                    }
+                }
+            }
+            ```
+
+            The used quantity must be a datetime typed quantity. Buckets are
+            integer numbers that determine the targeted amount of buckets.
+            Notice that number of buckets returned will always be less than or
+            equal to this target number. The default is to target 10 buckets.
+        '''))
+
     min_max: Optional[MinMaxAggregation] = Body(
         None,
         description=strip('''
@@ -1223,6 +1317,11 @@ class DateHistogramAggregationResponse(BucketAggregationResponse, DateHistogramA
     pass
 
 
+class AutoDateHistogramAggregationResponse(BucketAggregationResponse, AutoDateHistogramAggregation):
+    interval: str = Field(
+        None, description=strip('''The interval that was used for the bucketing.'''))
+
+
 class MinMaxAggregationResponse(MinMaxAggregation):
     data: List[Union[float, None]]
 
@@ -1235,6 +1334,7 @@ class AggregationResponse(Aggregation):
     terms: Optional[TermsAggregationResponse]
     histogram: Optional[HistogramAggregationResponse]
     date_histogram: Optional[DateHistogramAggregationResponse]
+    auto_date_histogram: Optional[AutoDateHistogramAggregationResponse]
     min_max: Optional[MinMaxAggregationResponse]
     statistics: Optional[StatisticsAggregationResponse]
 
