@@ -19,10 +19,11 @@
 import hmac
 import hashlib
 import uuid
+import requests
 from typing import Callable, cast
 from inspect import Parameter, signature
 from functools import wraps
-from fastapi import APIRouter, Depends, Query as FastApiQuery, HTTPException, status
+from fastapi import APIRouter, Depends, Query as FastApiQuery, Request, HTTPException, status
 from fastapi.security import OAuth2PasswordBearer, OAuth2PasswordRequestForm
 from pydantic import BaseModel
 import jwt
@@ -73,7 +74,7 @@ def create_user_dependency(
         if not user and upload_token_auth_allowed:
             user = _get_user_upload_token_auth(kwargs.get('token'))
         if not user and signature_token_auth_allowed:
-            user = _get_user_signature_token_auth(kwargs.get('signature_token'))
+            user = _get_user_signature_token_auth(kwargs.get('signature_token'), kwargs.get('request'))
 
         if required and not user:
             raise HTTPException(
@@ -92,6 +93,18 @@ def create_user_dependency(
                     status_code=status.HTTP_401_UNAUTHORIZED,
                     detail='You are not authorized to access this Oasis',
                     headers={'WWW-Authenticate': 'Bearer'})
+
+        if user:
+            try:
+                assert datamodel.User.get(user.user_id) is not None
+            except Exception as e:
+                logger = utils.get_logger(__name__)
+                logger.error('Api usage by unknown user. Possible missconfiguration', exc_info=e)
+                raise HTTPException(
+                    status_code=status.HTTP_401_UNAUTHORIZED,
+                    detail='You are logged in with an unknown user',
+                    headers={'WWW-Authenticate': 'Bearer'})
+
         return user
 
     # Create the desired function signature (as it depends on which auth options are allowed)
@@ -128,6 +141,11 @@ def create_user_dependency(
                     None,
                     description='Signature token used to sign download urls.'),
                 kind=Parameter.KEYWORD_ONLY))
+        new_parameters.append(
+            Parameter(
+                name='request',
+                annotation=Request,
+                kind=Parameter.KEYWORD_ONLY))
 
     # Create a wrapper around user_dependency, and set the signature on it
     @wraps(user_dependency)
@@ -149,7 +167,7 @@ def _get_user_basic_auth(form_data: OAuth2PasswordRequestForm) -> User:
     if form_data and form_data.username and form_data.password:
         try:
             infrastructure.keycloak.basicauth(form_data.username, form_data.password)
-            user = cast(datamodel.User, infrastructure.keycloak.get_user(form_data.username))
+            user = cast(datamodel.User, infrastructure.user_management.get_user(form_data.username))
             return user
         except infrastructure.KeycloakError:
             raise HTTPException(
@@ -193,7 +211,7 @@ def _get_user_upload_token_auth(upload_token: str) -> User:
 
             if signature_bytes == compare.digest():
                 user_id = str(uuid.UUID(bytes=payload_bytes))
-                user = cast(datamodel.User, infrastructure.keycloak.get_user(user_id))
+                user = cast(datamodel.User, infrastructure.user_management.get_user(user_id))
                 return user
         except Exception:
             # Decode error, format error, user not found, etc.
@@ -203,7 +221,7 @@ def _get_user_upload_token_auth(upload_token: str) -> User:
     return None
 
 
-def _get_user_signature_token_auth(signature_token: str) -> User:
+def _get_user_signature_token_auth(signature_token: str, request: Request) -> User:
     '''
     Verifies the signature token (throwing exception if illegal value provided) and returns the
     corresponding user object, or None, if no upload_token provided.
@@ -224,6 +242,21 @@ def _get_user_signature_token_auth(signature_token: str) -> User:
             raise HTTPException(
                 status_code=status.HTTP_401_UNAUTHORIZED,
                 detail='Invalid token.')
+    elif request:
+        auth_cookie = request.cookies.get('Authorization')
+        if auth_cookie:
+            try:
+                auth_cookie = requests.utils.unquote(auth_cookie)  # type: ignore
+                if auth_cookie.startswith('Bearer '):
+                    cookie_bearer_token = auth_cookie[7:]
+                    user = cast(datamodel.User, infrastructure.keycloak.tokenauth(cookie_bearer_token))
+                    return user
+            except infrastructure.KeycloakError as e:
+                raise HTTPException(
+                    status_code=status.HTTP_401_UNAUTHORIZED,
+                    detail=str(e), headers={'WWW-Authenticate': 'Bearer'})
+            except Exception:
+                pass
 
     return None
 
