@@ -15,12 +15,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
+import math
 from datetime import datetime
-import functools
-from itertools import zip_longest
-import multiprocessing
-import operator
 
 from typing import Optional, Set, Union, Dict, Iterator, Any, List
 from fastapi import (
@@ -35,6 +31,7 @@ import json
 import orjson
 from pydantic.main import create_model
 from starlette.responses import Response
+from joblib import Parallel, delayed, parallel_backend
 
 from nomad import files, config, utils, metainfo, processing as proc
 from nomad import datamodel
@@ -48,7 +45,7 @@ from nomad.metainfo.elasticsearch_extension import entry_type
 
 from .auth import create_user_dependency
 from ..utils import (
-    create_download_stream_zipped, create_download_stream_raw_file,
+    create_download_stream_zipped, create_download_stream_raw_file, browser_download_headers,
     DownloadItem, create_responses)
 from ..models import (
     Aggregation, Pagination, PaginationResponse, MetadataPagination, TermsAggregation,
@@ -204,6 +201,7 @@ class EntryRawDir(BaseModel):
     entry_id: str = Field(None)
     upload_id: str = Field(None)
     mainfile: str = Field(None)
+    mainfile_key: Optional[str] = Field(None)
     files: List[EntryRawDirFile] = Field(None)
 
 
@@ -476,6 +474,7 @@ def _create_entry_rawdir(entry_metadata: Dict[str, Any], uploads: _Uploads):
     entry_id = entry_metadata['entry_id']
     upload_id = entry_metadata['upload_id']
     mainfile = entry_metadata['mainfile']
+    mainfile_key = entry_metadata.get('mainfile_key')
 
     upload_files = uploads.get_upload_files(upload_id)
     mainfile_dir = os.path.dirname(mainfile)
@@ -484,7 +483,8 @@ def _create_entry_rawdir(entry_metadata: Dict[str, Any], uploads: _Uploads):
     for path_info in upload_files.raw_directory_list(mainfile_dir, files_only=True):
         files.append(EntryRawDirFile(path=path_info.path, size=path_info.size))
 
-    return EntryRawDir(entry_id=entry_id, upload_id=upload_id, mainfile=mainfile, files=files)
+    return EntryRawDir(
+        entry_id=entry_id, upload_id=upload_id, mainfile=mainfile, mainfile_key=mainfile_key, files=files)
 
 
 def _answer_entries_rawdir_request(
@@ -526,11 +526,11 @@ def _answer_entries_raw_request(owner: Owner, query: Query, files: Files, user:
         required=MetadataRequired(include=[]),
         user_id=user.user_id if user is not None else None)
 
-    if response.pagination.total > config.max_entry_download:
+    if response.pagination.total > config.services.max_entry_download:
         raise HTTPException(
             status_code=status.HTTP_400_BAD_REQUEST,
             detail='The limit of maximum number of entries in a single download (%d) has been exeeded (%d).' % (
-                config.max_entry_download, response.pagination.total))
+                config.services.max_entry_download, response.pagination.total))
 
     files_params = Files() if files is None else files
     search_includes = ['entry_id', 'upload_id', 'mainfile']
@@ -558,7 +558,9 @@ def _answer_entries_raw_request(owner: Owner, query: Query, files: Files, user:
             recursive=False,
             create_manifest_file=True,
             compress=files_params.compress)
-        return StreamingResponse(content, media_type='application/zip')
+        return StreamingResponse(content, headers=browser_download_headers(
+            filename='raw_files.zip',
+            media_type='application/zip'))
     except Exception as e:
         logger.error('exception while streaming download', exc_info=e)
         raise
@@ -676,21 +678,21 @@ def _read_archive(entry_metadata, uploads, required_reader: RequiredReader):
             return {
                 'entry_id': entry_id,
                 'parser_name': entry_metadata['parser_name'],
-                'archive': required_reader.read(archive, entry_id)}
+                'archive': required_reader.read(archive, entry_id, upload_id)}
     except ArchiveQueryError as e:
         raise HTTPException(status.HTTP_400_BAD_REQUEST, detail=str(e))
 
 
-def _validate_required(required: ArchiveRequired) -> RequiredReader:
+def _validate_required(required: ArchiveRequired, user) -> RequiredReader:
     try:
-        return RequiredReader(required)
+        return RequiredReader(required, user=user)
     except RequiredValidationError as e:
         raise HTTPException(
             status.HTTP_422_UNPROCESSABLE_ENTITY,
             detail=[dict(msg=e.msg, loc=['required'] + e.loc)])
 
 
-def _read_entry_from_archive(entry, uploads, required_reader: RequiredReader):
+def _read_entry_from_archive(entry: dict, uploads, required_reader: RequiredReader):
     entry_id, upload_id = entry['entry_id'], entry['upload_id']
 
     # all other exceptions are handled by the caller `_read_entries_from_archive`
@@ -698,11 +700,8 @@ def _read_entry_from_archive(entry, uploads, required_reader: RequiredReader):
         upload_files = uploads.get_upload_files(upload_id)
 
         with upload_files.read_archive(entry_id, True) as archive:
-            return {
-                'entry_id': entry_id,
-                'upload_id': upload_id,
-                'parser_name': entry['parser_name'],
-                'archive': required_reader.read(archive, entry_id)}
+            entry['archive'] = required_reader.read(archive, entry_id, upload_id)
+            return entry
     except ArchiveQueryError as e:
         raise HTTPException(status.HTTP_400_BAD_REQUEST, detail=str(e))
     except KeyError as e:
@@ -711,19 +710,19 @@ def _read_entry_from_archive(entry, uploads, required_reader: RequiredReader):
         return None
 
 
-def _read_entries_from_archive(entries, required):
+def _read_entries_from_archive(entries: Union[list, dict], required: ArchiveRequired, user):
     '''
     Takes pickleable arguments so that it can be offloaded to worker processes.
 
     It is important to ensure the return values are also pickleable.
     '''
     with _Uploads() as uploads:
-        required_reader = _validate_required(required)
+        required_reader = _validate_required(required, user)
 
-        responses = [_read_entry_from_archive(
-            entry, uploads, required_reader) for entry in entries if entry is not None]
+        if isinstance(entries, dict):
+            return _read_entry_from_archive(entries, uploads, required_reader)
 
-        return list(filter(None, responses))
+        return [_read_entry_from_archive(entry, uploads, required_reader) for entry in entries]
 
 
 def _answer_entries_archive_request(
@@ -744,39 +743,29 @@ def _answer_entries_archive_request(
         required=MetadataRequired(include=['entry_id', 'upload_id', 'parser_name']),
         user_id=user.user_id if user is not None else None)
 
-    # fewer than 8 entries per process is not useful
+    entries: list = [{
+        'entry_id': entry['entry_id'], 'upload_id': entry['upload_id'],
+        'parser_name': entry['parser_name']} for entry in search_response.data]
+
+    # fewer than config.archive.min_entries_per_process entries per process is not useful
     # more than config.max_process_number processes is too much for the server
     number: int = min(
-        len(search_response.data) // config.archive.min_entires_per_process,
+        int(math.ceil(len(entries) / config.archive.min_entries_per_process)),
         config.archive.max_process_number)
 
     if number <= 1:
-        request_data: list = _read_entries_from_archive(search_response.data, required)
+        request_data: list = _read_entries_from_archive(entries, required, user)
     else:
-        entries_per_process = len(search_response.data) // number + 1
-
-        # use process pool
-        pool: multiprocessing.pool.Pool = multiprocessing.pool.Pool(processes=number)
-
-        try:
-            responses = pool.map(
-                functools.partial(_read_entries_from_archive, required=required),
-                zip_longest(*[iter(search_response.data)] * entries_per_process))
-        finally:
-            # gracefully shutdown the pool
-            pool.close()
-            pool.join()
-
-        # collect results from each process
-        # https://stackoverflow.com/a/45323085
-        request_data = functools.reduce(operator.iconcat, responses, [])
+        with parallel_backend('threading', n_jobs=number):
+            request_data = Parallel()(delayed(
+                _read_entries_from_archive)(i, required, user) for i in entries)
 
     return EntriesArchiveResponse(
         owner=search_response.owner,
         query=search_response.query,
         pagination=search_response.pagination,
         required=required,
-        data=request_data)
+        data=list(filter(None, request_data)))
 
 
 _entries_archive_docstring = strip('''
@@ -843,17 +832,17 @@ def _answer_entries_archive_download_request(
         required=MetadataRequired(include=[]),
         user_id=user.user_id if user is not None else None)
 
-    if response.pagination.total > config.max_entry_download:
+    if response.pagination.total > config.services.max_entry_download:
         raise HTTPException(
             status_code=status.HTTP_400_BAD_REQUEST,
             detail=(
                 'The limit of maximum number of entries in a single download (%d) has been '
-                'exeeded (%d).' % (config.max_entry_download, response.pagination.total)))
+                'exeeded (%d).' % (config.services.max_entry_download, response.pagination.total)))
 
     manifest = []
     search_includes = ['entry_id', 'upload_id', 'parser_name']
 
-    required_reader = RequiredReader('*')
+    required_reader = RequiredReader('*', user=user)
 
     # a generator of StreamedFile objects to create the zipstream from
     def streamed_files():
@@ -880,7 +869,9 @@ def _answer_entries_archive_download_request(
     with _Uploads() as uploads:
         # create the streaming response with zip file contents
         content = create_zipstream(streamed_files(), compress=files_params.compress)
-        return StreamingResponse(content, media_type='application/zip')
+        return StreamingResponse(content, headers=browser_download_headers(
+            filename='archives.zip',
+            media_type='application/zip'))
 
 
 _entries_archive_download_docstring = strip('''
@@ -1068,20 +1059,23 @@ async def get_entry_raw_file(
     return StreamingResponse(raw_file_content, media_type=mime_type)
 
 
-def answer_entry_archive_request(query: Dict[str, Any], required: ArchiveRequired, user: User):
-    required_reader = _validate_required(required)
+def answer_entry_archive_request(
+        query: Dict[str, Any], required: ArchiveRequired, user: User, entry_metadata=None):
+    required_reader = _validate_required(required, user)
 
-    response = perform_search(
-        owner=Owner.visible, query=query,
-        required=MetadataRequired(include=['entry_id', 'upload_id', 'parser_name']),
-        user_id=user.user_id if user is not None else None)
+    if not entry_metadata:
+        response = perform_search(
+            owner=Owner.visible, query=query,
+            required=MetadataRequired(include=['entry_id', 'upload_id', 'parser_name']),
+            user_id=user.user_id if user is not None else None)
 
-    if response.pagination.total == 0:
-        raise HTTPException(
-            status_code=status.HTTP_404_NOT_FOUND,
-            detail='The entry does not exist or is not visible to you.')
+        if response.pagination.total == 0:
+            raise HTTPException(
+                status_code=status.HTTP_404_NOT_FOUND,
+                detail='The entry does not exist or is not visible to you.')
+
+        entry_metadata = response.data[0]
 
-    entry_metadata = response.data[0]
     entry_id = entry_metadata['entry_id']
 
     with _Uploads() as uploads:
@@ -1126,12 +1120,21 @@ async def get_entry_archive(
     responses=create_responses(_bad_id_response, _archive_download_response))
 async def get_entry_archive_download(
         entry_id: str = Path(..., description='The unique entry id of the entry to retrieve archive data from.'),
+        ignore_mime_type: bool = QueryParameter(
+            False, description=strip('''
+                Sets the mime type specified in the response headers to `application/octet-stream`
+                instead of the actual mime type (i.e. `application/json`).''')),
         user: User = Depends(create_user_dependency(signature_token_auth_allowed=True))):
     '''
     Returns the full archive for the given `entry_id`.
     '''
     response = answer_entry_archive_request(dict(entry_id=entry_id), required='*', user=user)
-    return response['data']['archive']
+    archive = response['data']['archive']
+    return StreamingResponse(
+        io.BytesIO(json.dumps(archive, indent=2).encode()),
+        headers=browser_download_headers(
+            filename=f'{entry_id}.json',
+            media_type='application/octet-stream' if ignore_mime_type else 'application/json'))
 
 
 @router.post(
