@@ -17,48 +17,50 @@
 #
 
 import click
-import os
-import shutil
-import tabulate
-import elasticsearch_dsl
-
-from nomad import config as nomad_config, infrastructure, processing
-from nomad import search as nomad_search
 
 from .admin import admin
 
 
 @admin.command(help='Checks consistency of files and es vs mongo and deletes orphan entries.')
 @click.option('--dry', is_flag=True, help='Do not delete anything, just check.')
-@click.option('--skip-calcs', is_flag=True, help='Skip cleaning calcs with missing uploads.')
+@click.option('--skip-entries', is_flag=True, help='Skip cleaning entries with missing uploads.')
 @click.option('--skip-fs', is_flag=True, help='Skip cleaning the filesystem.')
 @click.option('--skip-es', is_flag=True, help='Skip cleaning the es index.')
 @click.option('--staging-too', is_flag=True, help='Also clean published entries in staging, make sure these files are not due to reprocessing')
 @click.option('--force', is_flag=True, help='Do not ask for confirmation.')
-def clean(dry, skip_calcs, skip_fs, skip_es, staging_too, force):
+def clean(dry, skip_entries, skip_fs, skip_es, staging_too, force):
+    import os
+    import shutil
+    import tabulate
+    import elasticsearch_dsl
+
+    from nomad import config as nomad_config, infrastructure, processing
+    from nomad.search import delete_by_query
+    from nomad.search import quantity_values
+
     mongo_client = infrastructure.setup_mongo()
     infrastructure.setup_elastic()
 
-    if not skip_calcs:
-        uploads_for_calcs = mongo_client[nomad_config.mongo.db_name]['calc'].distinct('upload_id')
+    if not skip_entries:
+        uploads_for_entries = mongo_client[nomad_config.mongo.db_name]['entry'].distinct('upload_id')
         uploads = {}
         for upload in mongo_client[nomad_config.mongo.db_name]['upload'].distinct('_id'):
             uploads[upload] = True
 
         missing_uploads = []
-        for upload_for_calc in uploads_for_calcs:
-            if upload_for_calc not in uploads:
-                missing_uploads.append(upload_for_calc)
+        for upload_for_entry in uploads_for_entries:
+            if upload_for_entry not in uploads:
+                missing_uploads.append(upload_for_entry)
 
         if not dry and len(missing_uploads) > 0:
             if not force:
-                input('Will delete calcs (mongo + es) for %d missing uploads. Press any key to continue ...' % len(missing_uploads))
+                input('Will delete entries (mongo + es) for %d missing uploads. Press any key to continue ...' % len(missing_uploads))
 
             for upload in missing_uploads:
-                mongo_client[nomad_config.mongo.db_name]['calc'].remove(dict(upload_id=upload))
-                elasticsearch_dsl.Search(index=nomad_config.elastic.index_name).query('term', upload_id=upload).delete()
+                mongo_client[nomad_config.mongo.db_name]['entry'].remove(dict(upload_id=upload))
+                elasticsearch_dsl.Search(index=nomad_config.elastic.entries_index).query('term', upload_id=upload).delete()
         else:
-            print('Found %s uploads that have calcs in mongo, but there is no upload entry.' % len(missing_uploads))
+            print('Found %s uploads that have entries in mongo, but there is no upload entry.' % len(missing_uploads))
             print('List first 10:')
             for upload in missing_uploads[:10]:
                 print(upload)
@@ -106,27 +108,25 @@ def clean(dry, skip_calcs, skip_fs, skip_es, staging_too, force):
                 print(path)
 
     if not skip_es:
-        search = nomad_search.Search(index=nomad_config.elastic.index_name)
-        search.aggs.bucket('uploads', elasticsearch_dsl.A('terms', field='upload_id', size=12000))
-        response = search.execute()
+        es_upload_buckets = quantity_values('upload_id', owner='all', return_buckets=True)
 
         to_delete = list(
-            (bucket.key, bucket.doc_count)
-            for bucket in response.aggregations.uploads.buckets
-            if processing.Upload.objects(upload_id=bucket.key).first() is None)
+            (bucket.value, bucket.count)
+            for bucket in es_upload_buckets
+            if processing.Upload.objects(upload_id=bucket.value).first() is None)
 
-        calcs = 0
-        for _, upload_calcs in to_delete:
-            calcs += upload_calcs
+        entries = 0
+        for _, upload_entries in to_delete:
+            entries += upload_entries
 
         if not dry and len(to_delete) > 0:
             if not force:
                 input(
-                    'Will delete %d calcs in %d uploads from ES. Press any key to continue ...' %
-                    (calcs, len(to_delete)))
-            for upload, _ in to_delete:
-                nomad_search.Search(index=nomad_config.elastic.index_name).query('term', upload_id=upload).delete()
+                    'Will delete %d entries in %d uploads from ES. Press any key to continue ...' %
+                    (entries, len(to_delete)))
+            for upload_id, _ in to_delete:
+                delete_by_query(owner='all', query=dict(upload_id=upload_id))
         else:
-            print('Found %d calcs in %d uploads from ES with no upload in mongo.' % (calcs, len(to_delete)))
+            print('Found %d entries in %d uploads from ES with no upload in mongo.' % (entries, len(to_delete)))
             print('List first 10:')
-            tabulate.tabulate(to_delete, headers=['id', '#calcs'])
+            tabulate.tabulate(to_delete, headers=['id', '#entries'])
