@@ -21,8 +21,9 @@ import typing                 # pylint: disable=unused-import
 from nomad.metainfo import (  # pylint: disable=unused-import
     MSection, MCategory, Category, Package, Quantity, Section, SubSection, SectionProxy,
     Reference, MEnum, derived)
-from nomad.datamodel.metainfo.simulation.system import System
-from nomad.datamodel.metainfo.simulation.method import Method
+from nomad.datamodel.metainfo.simulation.system import System, AtomsGroup
+from nomad.datamodel.metainfo.simulation.method import Method, HoppingMatrix
+from nomad.datamodel.data import ArchiveSection
 
 from ..common import FastAccess
 
@@ -178,6 +179,44 @@ class AtomicValues(MSection):
         ''')
 
 
+class AtomicGroup(MSection):
+    '''
+    Generic section containing the values and information reqarding a molecular or sub-molecular
+    quantity that is a function of an atomic group such as radius of gyration...
+    '''
+
+    m_def = Section(validate=False)
+
+    kind = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Kind of the quantity.
+        ''')
+
+
+class AtomicGroupValues(MSection):
+    '''
+    Generic section containing information regarding the values of a trajectory property.
+    '''
+
+    m_def = Section(validate=False)
+
+    label = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Describes the atoms or molecule types involved in determining the property.
+        ''')
+
+    atomsgroup_ref = Quantity(
+        type=Reference(AtomsGroup.m_def),
+        shape=[1],
+        description='''
+        References to the atoms_group section containing the molecule for which Rg was calculated.
+        ''')
+
+
 class EnergyEntry(Atomic):
     '''
     Section describing a type of energy or a contribution to the total energy.
@@ -502,6 +541,18 @@ class Energy(MSection):
         The lowest unoccupied energy.
         """)
 
+    kinetic = SubSection(
+        sub_section=EnergyEntry.m_def,
+        description='''
+        Contains the value and information regarding the kinetic energy.
+        ''')
+
+    potential = SubSection(
+        sub_section=EnergyEntry.m_def,
+        description='''
+        Contains the value and information regarding the potential energy.
+        ''')
+
 
 class ForcesEntry(Atomic):
     '''
@@ -986,7 +1037,7 @@ class DosFingerprint(MSection):
         ''')
 
     indices = Quantity(
-        type=np.dtype(np.int16),
+        type=np.dtype(np.int32),
         shape=[2],
         description='''
         Indices used to compare DOS fingerprints of different energy ranges.
@@ -1032,10 +1083,9 @@ class DosValues(AtomicValues):
     normalization_factor = Quantity(
         type=np.dtype(np.float64),
         shape=[],
-        unit='1 / (m ** 3)',
         description='''
-        Normalization factor for DOS values to get a cell-independent intensive DOS.
-        For total dos, this is given by 1 / (unit cell volume).
+        Normalization factor for DOS values to get a cell-independent intensive DOS,
+        defined as the DOS integral from the lowest energy state to the Fermi level for a neutrally charged system.
         ''')
 
     value = Quantity(
@@ -1051,7 +1101,7 @@ class DosValues(AtomicValues):
         type=np.dtype(np.float64),
         shape=['n_energies'],
         description='''
-        Integrated DOS starting from the mimunum energy up to given a energy.
+        A cumulative DOS starting from the mimunum energy available up to the energy level specified in `energies`.
         ''')
 
 
@@ -1453,6 +1503,78 @@ class ExcitedStates(MSection):
         ''')
 
 
+class GreensFunctions(MSection):
+    '''
+    Green's functions properties in different time/frequency basis.
+    '''
+
+    m_def = Section(validate=False)
+
+    matsubara_freq = Quantity(
+        type=np.float64,
+        shape=['2 * n_matsubara_freq'],
+        description='''
+        Matsubara frequencies (imaginary frequencies).
+        ''')
+
+    tau = Quantity(
+        type=np.float64,
+        shape=['n_tau'],
+        description='''
+        Imaginary times.
+        ''')
+
+    chemical_potential = Quantity(
+        type=np.float64,
+        unit='electron_volt',
+        description='''
+        Chemical potential.
+        ''')
+
+    self_energy_iw = Quantity(
+        type=np.complex128,
+        shape=['n_atoms_per_unit_cell', 2, 'n_correlated_orbitals', '2 * n_matsubara_freq'],
+        description='''
+        Self-energy matrix in Matsubara frequencies.
+        ''')
+
+    greens_function_iw = Quantity(
+        type=np.complex128,
+        shape=['n_atoms_per_unit_cell', 2, 'n_correlated_orbitals', '2 * n_matsubara_freq'],
+        description='''
+        Green's function matrix in Matsubara frequencies.
+        ''')
+
+    greens_function_freq = Quantity(
+        type=np.complex128,
+        description='''
+        Green's function matrix in real frequencies.
+        ''')
+
+    greens_function_tau = Quantity(
+        type=np.complex128,
+        shape=['n_atoms_per_unit_cell', 2, 'n_correlated_orbitals', 'n_tau'],
+        description='''
+        Green's function matrix in tau (imaginary time).
+        ''')
+
+    orbital_occupations = Quantity(
+        type=np.float64,
+        shape=['n_atoms_per_unit_cell', 2, 'n_correlated_orbitals'],
+        description='''
+        Orbital occupation per correlated atom in the unit cell and per spin.
+        ''')
+
+    quasiparticle_weights = Quantity(
+        type=np.float64,
+        shape=['n_atoms_per_unit_cell', 2, 'n_correlated_orbitals'],
+        description='''
+        Quasiparticle weights of each orbital per site and spin. Calculated from:
+            Z = inv(1.0 - d [Re Sigma] / dw at w=0)
+        it takes values ∈ [0.0, 1.0], being Z=1 non-correlated, and Z=0 in a Mott state.
+        ''')
+
+
 class VibrationalFrequenciesValues(MSection):
     '''
     Section describing a vibrational spectrum.
@@ -1526,7 +1648,35 @@ class VibrationalFrequencies(MSection):
     infrared = SubSection(sub_section=VibrationalFrequenciesValues.m_def, repeats=False)
 
 
-class BaseCalculation(MSection):
+class RadiusOfGyrationValues(AtomicGroupValues):
+    '''
+    Section containing information regarding the values of
+    radius of gyration (Rg).
+    '''
+
+    m_def = Section(validate=False)
+
+    value = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='m',
+        description='''
+        Value of Rg.
+        ''')
+
+
+class RadiusOfGyration(AtomicGroup):
+    '''
+    Section containing information about the calculation of
+    radius of gyration (Rg).
+    '''
+
+    m_def = Section(validate=False)
+
+    radius_of_gyration_values = SubSection(sub_section=RadiusOfGyrationValues.m_def, repeats=True)
+
+
+class BaseCalculation(ArchiveSection):
     '''
     Contains computed properties of a configuration as defined by the corresponding
     section system and with the simulation method defined by section method. The
@@ -1625,22 +1775,6 @@ class BaseCalculation(MSection):
         contamination in spin-unrestricted calculations.
         ''')
 
-    pressure = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='pascal',
-        description='''
-        Value of the pressure of the system.
-        ''')
-
-    temperature = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='kelvin',
-        description='''
-        Value of the temperature of the system at which the properties are calculated.
-        ''')
-
     time_physical = Quantity(
         type=np.dtype(np.int32),
         shape=[],
@@ -1649,13 +1783,6 @@ class BaseCalculation(MSection):
         The elapsed time with respect to the start of the simulation.
         ''')
 
-    time_step = Quantity(
-        type=np.dtype(np.int32),
-        shape=[],
-        description='''
-        The number of time steps with respect to the start of the simulation.
-        ''')
-
     energy = SubSection(sub_section=Energy.m_def, categories=[FastAccess])
 
     forces = SubSection(sub_section=Forces.m_def)
@@ -1674,8 +1801,12 @@ class BaseCalculation(MSection):
 
     thermodynamics = SubSection(sub_section=Thermodynamics.m_def, repeats=True)
 
+    hopping_matrix = SubSection(sub_section=HoppingMatrix.m_def, repeats=True)
+
     excited_states = SubSection(sub_section=ExcitedStates.m_def, repeats=True)
 
+    greens_functions = SubSection(sub_section=GreensFunctions.m_def, repeats=True)
+
     vibrational_frequencies = SubSection(sub_section=VibrationalFrequencies.m_def, repeats=True)
 
     potential = SubSection(sub_section=Potential.m_def, repeats=True)
@@ -1686,6 +1817,47 @@ class BaseCalculation(MSection):
 
     density_charge = SubSection(sub_section=Density.m_def, repeats=True)
 
+    radius_of_gyration = SubSection(sub_section=RadiusOfGyration.m_def, repeats=True)
+
+    volume = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='m ** 3',
+        description='''
+        Value of the volume of the system.
+        ''')
+
+    pressure = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='pascal',
+        description='''
+        Value of the pressure of the system.
+        ''')
+
+    temperature = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='kelvin',
+        description='''
+        Value of the temperature of the system.
+        ''')
+
+    step = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        The number of time steps with respect to the start of the simulation.
+        ''')
+
+    time = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='second',
+        description='''
+        The elapsed simulated physical time since the start of the simulation.
+        ''')
+
 
 class ScfIteration(BaseCalculation):
     '''
