@@ -20,6 +20,7 @@ import numpy as np            # pylint: disable=unused-import
 import typing
 
 from pint.util import SharedRegistryObject                 # pylint: disable=unused-import
+from nomad.datamodel.data import ArchiveSection
 from nomad.metainfo import (  # pylint: disable=unused-import
     MSection, MCategory, Category, Package, Quantity, Section, SubSection, SectionProxy,
     Reference, MEnum, derived)
@@ -31,7 +32,7 @@ m_package = Package()
 
 class KMesh(MSection):
     '''
-    Section containing the values for
+    Contains the settings for a uniformly spaced k-point grid (if employed).
     '''
 
     m_def = Section(validate=False)
@@ -43,6 +44,13 @@ class KMesh(MSection):
         Number of k points in the mesh (i.e. the k points used to evaluate energy_total).
         ''')
 
+    grid = Quantity(
+        type=np.dtype(np.int32),
+        shape=[3],
+        description='''
+        k-point grid used in notation [nx, ny, nz] in fractional coordinates.
+        ''')
+
     generation_method = Quantity(
         type=str,
         shape=[],
@@ -50,14 +58,6 @@ class KMesh(MSection):
         Method used to generate the k points.
         ''')
 
-    density = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='1 / meter ** 3',
-        description='''
-        Density of k points.
-        ''')
-
     points = Quantity(
         type=np.dtype(np.float64),
         shape=['n_points', 3],
@@ -120,6 +120,103 @@ class Scf(MSection):
         ''')
 
 
+class HubbardKanamoriModel(MSection):
+    '''
+    Setup of the local Hubbard model.
+    '''
+
+    m_def = Section(validate=False)
+
+    orbital = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Orbital label corresponding to the Hubbard model. The typical orbitals with strong
+        Hubbard interactions have partially filled '3d', '4d' and '4f' orbitals.
+        ''')
+
+    n_orbital = Quantity(
+        type=np.int32,
+        shape=[],
+        description='''
+        Number of non-degenerated orbitals of the same type (s, p, d, f, ...).
+        ''')
+
+    u = Quantity(
+        type=np.float64,
+        shape=[],
+        unit='joule',
+        description='''
+        Value of the (intraorbital) Hubbard interaction
+        ''')
+
+    jh = Quantity(
+        type=np.float64,
+        shape=[],
+        unit='joule',
+        description='''
+        Value of the (interorbital) Hund's coupling.
+        ''')
+
+    up = Quantity(
+        type=np.float64,
+        shape=[],
+        unit='joule',
+        description='''
+        Value of the (interorbital) Coulomb interaction. In rotational invariant
+        systems, up = u - 2 * jh.
+        ''')
+
+    j = Quantity(
+        type=np.float64,
+        shape=[],
+        unit='joule',
+        description='''
+        Value of the exchange interaction. In rotational invariant systems, j = jh.
+        ''')
+
+    u_effective = Quantity(
+        type=np.float64,
+        shape=[],
+        unit='joule',
+        description='''
+        Value of the effective U parameter (u - j).
+        ''')
+
+    slater_integrals = Quantity(
+        type=np.float64,
+        shape=[3],
+        unit='joule',
+        description='''
+        Value of the Slater integrals (F0, F2, F4) in spherical harmonics used to derive
+        the local Hubbard interactions:
+
+            u = ((2.0 / 7.0) ** 2) * (F0 + 5.0 * F2 + 9.0 * F4) / (4.0*np.pi)
+
+            up = ((2.0 / 7.0) ** 2) * (F0 - 5.0 * F2 + 3.0 * 0.5 * F4) / (4.0*np.pi)
+
+            jh = ((2.0 / 7.0) ** 2) * (5.0 * F2 + 15.0 * 0.25 * F4) / (4.0*np.pi)
+
+        Ref.: Elbio Dagotto, Nanoscale Phase Separation and Colossal Magnetoresistance,
+        Chapter 4, Springer Berlin (2003).
+        ''')
+
+    umn = Quantity(
+        type=np.float64,
+        shape=['n_orbital', 'n_orbital'],
+        unit='joule',
+        description='''
+        Value of the local Coulomb interaction matrix.
+        ''')
+
+    double_counting_correction = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Name of the double counting correction algorithm applied.
+        ''')
+
+
 class AtomParameters(MSection):
     '''
     Contains method-related information about a kind of atom identified by label. This
@@ -208,6 +305,8 @@ class AtomParameters(MSection):
         Values of the charge corresponding to each orbital.
         ''')
 
+    hubbard_kanamori_model = SubSection(sub_section=HubbardKanamoriModel.m_def, repeats=False)
+
 
 class MoleculeParameters(MSection):
     '''
@@ -575,64 +674,10 @@ class XCFunctional(Model):
 
     contributions = SubSection(sub_section=Functional.m_def, repeats=True)
 
-
-class DFTPlusU(MSection):
-    '''
-    Section for DFT+U-settings of a single orbital
-    '''
-
-    m_def = Section(validate=False)
-
-    atom = Quantity(
-        type=np.dtype(np.int32),
-        shape=[],
-        description='''
-        Index of the atom corresponding to the parameter setting
-        ''')
-
-    orbital = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Orbital label corresponding to the parameter setting following the notation:
-        '3d', '4f', ...
-        ''')
-
-    U_effective = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        description='''
-        Value of the effective U parameter (U-J).
-        ''')
-
-    U = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        description='''
-        Value of the on-site Coulomb interaction U
-        ''')
-
-    J = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        description='''
-        Value of the exchange interaction J
-        ''')
-
-    functional = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Type of DFT+U functional (such as DFT/DFT+U double-counting compensation).
-        ''')
-
-    projection_type = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        DFT+U: Type of orbitals used for projection in order to calculate occupation
-        numbers.
-        ''')
+    def normalize_hybrid(self):
+        for hyb in self.hybrid:
+            if 'exact_exchange_mixing_factor' in hyb.parameters.keys() and not hyb.name:
+                hyb.name += '+alpha'
 
 
 class DFT(MSection):
@@ -675,7 +720,151 @@ class DFT(MSection):
 
     xc_functional = SubSection(sub_section=XCFunctional.m_def)
 
-    dft_plus_u = SubSection(sub_section=DFTPlusU.m_def, repeats=True)
+
+class Wannier(MSection):
+    '''
+    Section containing the various parameters that define a Wannier tight-binding method.
+    '''
+
+    m_def = Section(validate=False)
+
+    k_mesh = SubSection(sub_section=KMesh.m_def, repeats=False)
+
+    n_projected_orbitals = Quantity(
+        type=np.int32,
+        description='''
+        Number of Wannier orbitals used to fit the DFT band structure
+        ''')
+
+    n_bands = Quantity(
+        type=np.int32,
+        description='''
+        Number of input Bloch bands to calculate the projection matrix.
+        ''')
+
+    is_maximally_localized = Quantity(
+        type=bool,
+        description='''
+        Are the projected orbitals maximally localized or just a single-shot projection?
+        ''')
+
+    convergence_tolerance_max_localization = Quantity(
+        type=np.float64,
+        description='''
+        Convergence tolerance for maximal localization of the projected orbitals.
+        ''')
+
+    energy_window_outer = Quantity(
+        type=np.float64,
+        unit='electron_volt',
+        shape=[2],
+        description='''
+        Bottom and top of the outer energy window used for the projection.
+        ''')
+
+    energy_window_inner = Quantity(
+        type=np.float64,
+        unit='electron_volt',
+        shape=[2],
+        description='''
+        Bottom and top of the inner energy window used for the projection.
+        ''')
+
+
+class Projection(MSection):
+    '''
+    Section containing the various parameters that define a Wannier90-like projection
+    '''
+
+    m_def = Section(validate=False)
+
+    wannier = SubSection(sub_section=Wannier.m_def, repeats=False)
+
+
+class HoppingMatrix(MSection):
+    '''
+    Section containing the hopping/overlap matrix elements between N projected
+    orbitals.
+    '''
+
+    m_def = Section(validate=False)
+
+    n_orbitals = Quantity(
+        type=np.int32,
+        description='''
+        Number of projected orbitals.
+        ''')
+
+    n_wigner_seitz_points = Quantity(
+        type=np.int32,
+        description='''
+        Number of Wigner-Seitz real points.
+        ''')
+
+    degeneracy_factors = Quantity(
+        type=np.int32,
+        shape=['n_wigner_seitz_points'],
+        description='''
+        Degeneracy of each Wigner-Seitz grid point.
+        ''')
+
+    value = Quantity(
+        type=np.float64,
+        shape=['n_wigner_seitz_points', 'n_orbitals * n_orbitals', 7],
+        description='''
+        Real space hopping matrix for each Wigner-Seitz grid point. The elements are
+        defined as follows:
+
+            n_x   n_y   n_z   orb_1   orb_2   real_part + j * imag_part
+
+        where (n_x, n_y, n_z) define the Wigner-Seitz cell vector in fractional coordinates,
+        (orb_1, orb_2) indicates the hopping amplitude between orb_1 and orb_2, and the
+        real and imaginary parts of the hopping in electron_volt.
+        ''')
+
+
+class LatticeModelHamiltonian(MSection):
+    '''
+    Section containing the parameters of the non-interacting parts of a lattice model Hamiltonian.
+    '''
+
+    m_def = Section(validate=False)
+
+    # I defined t_parameters apart from HoppingMatrix to simplify the parsing (writing
+    # everything in the HoppingMatrix basis might be tedious).
+    # TODO generalize parsers to write HoppingMatrix as default even for simple models.
+    lattice_name = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Name of the lattice to identify the model. E.g., 'Square', 'Honeycomb'.
+        ''')
+
+    n_neighbors = Quantity(
+        type=np.int32,
+        description='''
+        Number of direct neighbors considered for the hopping integrals.
+        ''')
+
+    t_parameters = Quantity(
+        type=np.complex128,
+        shape=['n_neighbors'],
+        description='''
+        Hopping parameters for simple models, with [t, t`, t``, etc].
+        ''')
+
+    # TODO determine standard dimensions for projection_matrix:
+    #   KMesh.n_points x n_atoms_per_unit_cell x n_orbitals? x n_bands?
+    projection_matrix = Quantity(
+        type=np.complex128,
+        shape=['*', '*', '*', '*'],
+        description='''
+        Projection matrices from Bloch bands to virtual projected orbitals.
+        ''')
+
+    hopping_matrix = SubSection(sub_section=HoppingMatrix.m_def, repeats=False)
+
+    hubbard_kanamori_model = SubSection(sub_section=HubbardKanamoriModel.m_def, repeats=True)
 
 
 class GW(MSection):
@@ -685,88 +874,30 @@ class GW(MSection):
 
     m_def = Section(validate=False)
 
-    bare_coulomb_cutofftype = Quantity(
-        type=str,
+    type = Quantity(
+        type=MEnum(["G0W0", "scGW", "scGW0", "scG0W", "ev-scGW", "qp-scGW"]),
         shape=[],
         description='''
-        Cutoff type for the calculation of the bare Coulomb potential: none, 0d, 1d, 2d.
-        See Rozzi et al., PRB 73, 205119 (2006)
+        GW Hedin's self-consistency cycle:
+
+        | Name      | Description                      | Reference             |
+
+        | --------- | -------------------------------- | --------------------- |
+
+        | `"G0W0"`  | single-shot                      | -                     |
+
+        | `"scGW"`  | self-consistent GW               | PRB 88, 075105 (2013) |
+
+        | `"scGW0"` | self-consistent G with fixed W0  | PRB 54, 8411 (1996)   |
+
+        | `"scG0W"` | self-consistent W with fixed G0  | -                     |
+
+        | `"ev-scGW"`  | eigenvalues self-consistent GW   | PRB 34, 5390 (1986)   |
+
+        | `"qp-scGW"`  | quasiparticle self-consistent GW | PRL 96, 226402 (2006) |
         ''')
 
-    bare_coulomb_gmax = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='1 / meter',
-        description='''
-        Maximum G for the pw basis for the Coulomb potential.
-        ''')
-
-    basis_set = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Auxillary basis set used for non-local operators: mixed - mixed basis set, Kotani
-        and Schilfgaarde, Solid State Comm. 121, 461 (2002).
-        ''')
-
-    core_treatment = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        It specifies whether the core states are treated in the GW calculation: all - All
-        electron calculation; val - Valence electron only calculation; vab - Core
-        electrons are excluded from the mixed product basis; xal - All electron treatment
-        of the exchange self-energy only
-        ''')
-
-    frequency_grid_type = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Frequency integration grid type for the correlational self energy: 'eqdis' -
-        equidistant frequencies from 0 to freqmax; 'gaulag' - Gauss-Laguerre quadrature
-        from 0 to infinity; 'gauleg' - Gauss-Legendre quadrature from 0 to freqmax;
-        'gaule2' (default) - double Gauss-Legendre quadrature from 0 to freqmax and from
-        freqmax to infinity.
-        ''')
-
-    max_frequency = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        description='''
-        Maximum frequency for the calculation of the self energy.
-        ''')
-
-    mixed_basis_gmax = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='1 / meter',
-        description='''
-        Cut-off parameter for the truncation of the expansion of the plane waves in the
-        interstitial region.
-        ''')
-
-    mixed_basis_lmax = Quantity(
-        type=np.dtype(np.int32),
-        shape=[],
-        description='''
-        Maximum l value used for the radial functions within the muffin-tin.
-        ''')
-
-    mixed_basis_tolerance = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        description='''
-        Eigenvalue threshold below which the egenvectors are discarded in the construction
-        of the radial basis set.
-        ''')
-
-    ngridq = Quantity(
-        type=np.dtype(np.int32),
-        shape=[3],
-        description='''
-        k/q-point grid size used in the GW calculation.
-        ''')
+    q_grid = SubSection(sub_section=KMesh.m_def, repeats=False)
 
     n_frequencies = Quantity(
         type=np.dtype(np.int32),
@@ -775,13 +906,6 @@ class GW(MSection):
         Number of frequency points used in the calculation of the self energy.
         ''')
 
-    frequency_number = Quantity(
-        type=np.dtype(np.int32),
-        shape=['n_frequencies'],
-        description='''
-        Number referring to the frequency used in the calculation of the self energy.
-        ''')
-
     frequency_values = Quantity(
         type=np.dtype(np.float64),
         shape=['n_frequencies'],
@@ -790,99 +914,178 @@ class GW(MSection):
         Values of the frequency used in the calculation of the self energy.
         ''')
 
-    frequency_weights = Quantity(
-        type=np.dtype(np.float64),
-        shape=['n_frequencies'],
+    core_treatment = Quantity(
+        type=str,
+        shape=[],
         description='''
-        Weights of the frequency used in the calculation of the self energy.
+        Specifies if core states are treated in the GW calculation:
+
+        | Name    | Description                                             |
+
+        | ------- | ------------------------------------------------------- |
+
+        | `"all"` | All electron calculation                                |
+
+        | `"val"` | Valence electron only calculation                       |
+
+        | `"fc"`  | Frozen-core approximation                               |
+
+        | `"xal"` | All electron treatment of the exchange self-energy only |
         ''')
 
-    polarizability_number_of_empty_states = Quantity(
-        type=int,
+    dielectric_function_treatment = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Model used to calculate the dinamically-screened dielectric function:
+
+        | Name        | Description                                    | Reference             |
+
+        | ----------- | ---------------------------------------------- | --------------------- |
+
+        | `"rpa"`     | Full frequency random-phase approximation      | -                     |
+
+        | `"ppm"`     | Godby-Needs plasmon-pole model                 | PRL 62, 1169 (1989)   |
+
+        | `"ppm_hl"`  | Hybertsen and Louie plasmon-pole model         | PRB 34, 5390 (1986)   |
+
+        | `"ppm_hl2"` | von der Linden and P. Horsh plasmon-pole model | PRB 37, 8351 (1988)   |
+
+        | `"ppm_fe"`  | Farid and Engel plasmon-pole model             | PRB 47, 15931 (1993)  |
+
+        | `"cdm"`     | Countour-deformation method                    | PRB 67, 155208 (2003) |
+        ''')
+
+    self_energy_analytical_continuation = Quantity(
+        type=MEnum(["pade", "multi-pole", "CD"]),
+        shape=[],
+        description='''
+        Analytical continuation approximations of the GW self-energy:
+
+        | Name           | Description         | Reference                        |
+
+        | -------------- | ------------------- | -------------------------------- |
+
+        | `"pade"`       | Pade's approximant  | J. Low Temp. Phys 29, 179 (1977) |
+
+        | `"multi-pole"` | Multi-pole fitting  | PRL 74, 1827 (1995)              |
+
+        | `"CD"`         | Contour deformation | PRB 67, 155208 (2003)            |
+        ''')
+
+    n_empty_states_polarizability = Quantity(
+        type=np.dtype(np.int32),
         shape=[],
         description='''
         Number of empty states used to compute the polarizability P
         ''')
 
-    qp_equation_treatment = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Methods to solve the quasi-particle equation: 'linearization', 'self-consistent'
-        ''')
-
-    screened_coulomb_volume_average = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Type of volume averaging for the dynamically screened Coulomb potential: isotropic
-        - Simple averaging along a specified direction using only diagonal components of
-        the dielectric tensor; anisotropic - Anisotropic screening by C. Freysoldt et al.,
-        CPC 176, 1-13 (2007)
-        ''')
-
-    screened_coulomb = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Model used to calculate the dinamically-screened Coulomb potential: 'rpa' - Full-
-        frequency random-phase approximation; 'ppm' - Godby-Needs plasmon-pole model Godby
-        and Needs, Phys. Rev. Lett. 62, 1169 (1989); 'ppm_hl' - Hybertsen and Louie, Phys.
-        Rev. B 34, 5390 (1986); 'ppm_lh' - von der Linden and P. Horsh, Phys. Rev. B 37,
-        8351 (1988); 'ppm_fe' - Farid and Engel, Phys. Rev. B 47,15931 (1993); 'cdm' -
-        Contour deformation method, Phys. Rev. B 67, 155208 (2003).)
-        ''')
-
-    self_energy_c_analytical_continuation = Quantity(
-        type=str,
-        shape=[],
-        description='''
-        Models for the correlation self-energy analytical continuation: 'pade' -  Pade's
-        approximant (by H. J. Vidberg and J. W. Serence, J. Low Temp. Phys. 29, 179
-        (1977)); 'mpf' -  Multi-Pole Fitting (by H. N Rojas, R. W. Godby and R. J. Needs,
-        Phys. Rev. Lett. 74, 1827 (1995)); 'cd' - contour deformation; 'ra' - real axis
-        ''')
-
-    self_energy_c_number_of_empty_states = Quantity(
+    n_empty_states_self_energy = Quantity(
         type=np.dtype(np.int32),
         shape=[],
         description='''
         Number of empty states to be used to calculate the correlation self energy.
         ''')
 
-    self_energy_c_number_of_poles = Quantity(
-        type=int,
+
+class DMFT(MSection):
+    '''
+    Section containing the various parameters that define a DMFT calculation
+    '''
+
+    m_def = Section(validate=False)
+
+    n_atoms_per_unit_cell = Quantity(
+        type=np.int32,
         shape=[],
         description='''
-        Number of poles used in the analytical continuation.
+        Number of atoms per unit cell.
         ''')
 
-    self_energy_singularity_treatment = Quantity(
-        type=str,
-        shape=[],
+    n_correlated_orbitals = Quantity(
+        type=np.int32,
+        shape=['n_atoms_per_unit_cell'],
         description='''
-        Treatment of the integrable singular terms in the calculation of the self energy.
-        Values: 'mpb' - Auxiliary function method by S. Massidda, M. Posternak, and A.
-        Baldereschi, PRB 48, 5058 (1993); 'crg' - Auxiliary function method by P. Carrier,
-        S. Rohra, and A. Goerling, PRB 75, 205126 (2007).
+        Number of correlated orbitals per atom in the unit cell.
         ''')
 
-    starting_point = Quantity(
-        type=str,
-        shape=[],
+    n_correlated_electrons = Quantity(
+        type=np.float64,
+        shape=['n_atoms_per_unit_cell'],
         description='''
-        Exchange-correlation functional of the ground-state calculation.
+        Number of correlated electrons per atom in the unit cell.
         ''')
 
-    type = Quantity(
-        type=MEnum(["G0W0", "scGW", "scGW0", "scG0W", "ev-scGW", "qp-scGW"]),
+    inverse_temperature = Quantity(
+        type=np.float64,
+        unit='1/joule',
         shape=[],
         description='''
-        GW methodology: G0W0; ev-scGW: (eigenvalues self-consistent GW) – Phys.Rev.B 34,
-        5390 (1986); qp-scGW: (quasi-particle self-consistent GW) – Phys. Rev. Lett. 96,
-        226402 (2006)  scGW0: (self-consistent G with fixed W0) – Phys.Rev.B 54, 8411
-        (1996); scG0W: (self-consistent W with fixed G0); scGW: (self-consistent GW) –
-        Phys. Rev. B 88, 075105 (2013)
+        Inverse temperature = 1/(kB*T).
+        ''')
+
+    magnetic_state = Quantity(
+        type=MEnum(["paramagnetic", "ferromagnetic", "antiferromagnetic"]),
+        shape=[],
+        description='''
+        Magnetic state in which the DMFT calculation is done:
+
+        | Name                  | State                   |
+
+        | --------------------- | ----------------------- |
+
+        | `"paramagnetic"`      | paramagnetic state      |
+
+        | `"ferromagnetic"`     | ferromagnetic state     |
+
+        | `"antiferromagnetic"` | antiferromagnetic state |
+        ''')
+
+    n_matsubara_freq = Quantity(
+        type=np.int32,
+        shape=[],
+        description='''
+        Number of Matsubara frequencies (imaginary frequencies).
+        ''')
+
+    n_tau = Quantity(
+        type=np.int32,
+        shape=[],
+        description='''
+        Number of tau (imaginary times).
+        ''')
+
+    impurity_solver = Quantity(
+        type=MEnum(["CT-INT", "CT-HYB", "CT-AUX", "ED", "NRG", "MPS", "IPT", "NCA", "OCA", "slave_bosons", "hubbard_I"]),
+        shape=[],
+        description='''
+        Impurity solver method used in the DMFT loop:
+
+        | Name              | Reference                            |
+
+        | ----------------- | ------------------------------------ |
+
+        | `"CT-INT"`        | Rubtsov et al., JEPT Lett 80 (2004)  |
+
+        | `"CT-HYB"`        | Werner et al., PRL 97 (2006)         |
+
+        | `"CT-AUX"`        | Gull et al., EPL 82 (2008)           |
+
+        | `"ED"`            | Caffarrel et al, PRL 72 (1994)       |
+
+        | `"NRG"`           | Bulla et al., RMP 80 (2008)          |
+
+        | `"MPS"`           | Ganahl et al., PRB 90 (2014)         |
+
+        | `"IPT"`           | Georges et al., PRB 45 (1992)        |
+
+        | `"NCA"`           | Pruschke et al., PRB 47 (1993)       |
+
+        | `"OCA"`           | Pruschke et al., PRB 47 (1993)       |
+
+        | `"slave_bosons"`  | Kotliar et al., PRL 57 (1986)        |
+
+        | `"hubbard_I"`     | -                                    |
         ''')
 
 
@@ -921,6 +1124,88 @@ class TB(MSection):
     model = SubSection(sub_section=TBModel.m_def, repeats=True)
 
 
+class NeighborSearching(MSection):
+    '''
+    Section containing the parameters for neighbor searching/lists during a molecular dynamics run.
+    '''
+
+    m_def = Section(validate=False)
+
+    neighbor_update_frequency = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Number of timesteps between updating the neighbor list.
+        ''')
+
+    neighbor_update_cutoff = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='m',
+        description='''
+        The distance cutoff for determining the neighbor list.
+        ''')
+
+
+class ForceCalculations(MSection):
+    '''
+    Section containing the parameters for force calculations according to the referenced force field
+    during a molecular dynamics run.
+    '''
+
+    m_def = Section(validate=False)
+
+    vdw_cutoff = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='m',
+        description='''
+        Cutoff for calculating VDW forces.
+        ''')
+
+    coulomb_type = Quantity(
+        type=MEnum('cutoff', 'ewald', 'multilevel_summation', 'particle_mesh_ewald',
+                   'particle_particle_particle_mesh', 'reaction_field'),
+        shape=[],
+        description='''
+        Method used for calculating long-ranged Coulomb forces.
+
+        Allowed values are:
+
+        | Barostat Name          | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `""`                   | No thermostat               |
+
+        | `"Cutoff"`          | Simple cutoff scheme. |
+
+        | `"Ewald"` | Standard Ewald summation as described in any solid-state physics text. |
+
+        | `"Multi-Level Summation"` |  D. Hardy, J.E. Stone, and K. Schulten,
+        [Parallel. Comput. **35**, 164](https://doi.org/10.1016/j.parco.2008.12.005)|
+
+        | `"Particle-Mesh-Ewald"`        | T. Darden, D. York, and L. Pedersen,
+        [J. Chem. Phys. **98**, 10089 (1993)](https://doi.org/10.1063/1.464397) |
+
+        | `"Particle-Particle Particle-Mesh"` | See e.g. Hockney and Eastwood, Computer Simulation Using Particles,
+        Adam Hilger, NY (1989). |
+
+        | `"Reaction-Field"` | J.A. Barker and R.O. Watts,
+        [Mol. Phys. **26**, 789 (1973)](https://doi.org/10.1080/00268977300102101)|
+        ''')
+
+    coulomb_cutoff = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='m',
+        description='''
+        Cutoff for calculating short-ranged Coulomb forces.
+        ''')
+
+    neighbor_searching = SubSection(sub_section=NeighborSearching.m_def, repeats=False)
+
+
 class ForceField(MSection):
     '''
     Section containing the parameters pertaining to a force field calculation.
@@ -930,6 +1215,8 @@ class ForceField(MSection):
 
     model = SubSection(sub_section=Model.m_def, repeats=True)
 
+    force_calculations = SubSection(sub_section=ForceCalculations.m_def, repeats=False)
+
 
 class Smearing(MSection):
     '''
@@ -1082,7 +1369,7 @@ class Electronic(MSection):
     smearing = SubSection(sub_section=Smearing.m_def)
 
 
-class Method(MSection):
+class Method(ArchiveSection):
     '''
     Section containing the various parameters that define the theory and the
     approximations (convergence, thresholds, etc.) behind the calculation.
@@ -1142,8 +1429,14 @@ class Method(MSection):
 
     dft = SubSection(sub_section=DFT.m_def)
 
+    projection = SubSection(sub_section=Projection.m_def)
+
+    lattice_model_hamiltonian = SubSection(sub_section=LatticeModelHamiltonian.m_def, repeats=True)
+
     gw = SubSection(sub_section=GW.m_def)
 
+    dmft = SubSection(sub_section=DMFT.m_def)
+
     tb = SubSection(sub_section=TB.m_def)
 
     force_field = SubSection(sub_section=ForceField.m_def)
