@@ -4,9 +4,9 @@ from nptyping import NDArray
 from nomad.metainfo import (  # pylint: disable=unused-import
     MSection, MEnum, Quantity, Section, SubSection, SectionProxy,
     Reference, derived)
-from nomad.datamodel.metainfo.simulation.calculation import Calculation
+from nomad.datamodel.metainfo.simulation.calculation import Calculation, Dos, BandStructure
 from nomad.datamodel.metainfo.simulation.run import Run
-from nomad.datamodel.metainfo.simulation.system import System, Atoms
+from nomad.datamodel.metainfo.simulation.system import System, Atoms, AtomsGroup
 from .common import FastAccess
 
 
@@ -533,17 +533,33 @@ class GeometryOptimization(MSection):
         validate=False)
 
     type = Quantity(
-        type=str,
+        type=MEnum('static', 'atomic', 'cell_shape', 'cell_volume'),
         shape=[],
         description='''
-        The type of geometry optimization can either be ionic, cell_shape, cell_volume.
+        The type of geometry optimization, which denotes what is being optimized.
+
+        Allowed values are:
+
+        | Type                   | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `"static"`             | no optimization |
+
+        | `"atomic"`             | the atomic coordinates alone are updated |
+
+        | `"cell_volume"`         | `"atomic"` + cell lattice paramters are updated isotropically |
+
+        | `"cell_shape"`        | `"cell_volume"` but without the isotropic constraint: all cell parameters are updated |
+
         ''')
 
     method = Quantity(
         type=str,
         shape=[],
         description='''
-        The method used for geometry optimization.
+        The method used for geometry optimization. Some known possible values are:
+        `"steepest_descent"`, `"conjugant_gradient"`, `"low_memory_broyden_fletcher_goldfarb_shanno"`.
         ''')
 
     convergence_tolerance_energy_difference = Quantity(
@@ -595,11 +611,18 @@ class GeometryOptimization(MSection):
         The maximum displacement in the last optimization step with respect to previous.
         ''')
 
+    optimization_steps_maximum = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Maximum number of optimization steps.
+        ''')
+
     optimization_steps = Quantity(
         type=int,
         shape=[],
         description='''
-        Number of optimization steps.
+        Number of saved optimization steps.
         ''')
 
     energies = Quantity(
@@ -611,6 +634,13 @@ class GeometryOptimization(MSection):
         calculations that are a part of the optimization trajectory.
         ''')
 
+    steps = Quantity(
+        type=np.dtype(np.int32),
+        shape=[],
+        description='''
+        The step index corresponding to each saved configuration.
+        ''')
+
     is_converged_geometry = Quantity(
         type=bool,
         shape=[],
@@ -820,7 +850,7 @@ class Elastic(MSection):
         ''')
 
     deformation_types = Quantity(
-        type=np.dtype('U'),
+        type=np.str_,
         shape=['n_deformations', 6],
         description='''
         deformation types
@@ -857,7 +887,7 @@ class Elastic(MSection):
         ''')
 
     elastic_constants_notation_matrix_second_order = Quantity(
-        type=np.dtype('U'),
+        type=np.str_,
         shape=[6, 6],
         description='''
         Symmetry of the second-order elastic constant matrix in Voigt notation
@@ -1207,7 +1237,8 @@ class Thermodynamics(MSection):
         mass_per_unit_cell = nomad.atomutils.get_summed_atomic_mass(atomic_numbers)
         heat_capacity = self.heat_capacity_c_v
         specific_heat_capacity = heat_capacity / mass_per_unit_cell
-        return specific_heat_capacity
+
+        return specific_heat_capacity.magnitude
 
     vibrational_free_energy_at_constant_volume = Quantity(
         type=np.dtype(np.float64),
@@ -1234,7 +1265,8 @@ class Thermodynamics(MSection):
         mass_per_unit_cell = nomad.atomutils.get_summed_atomic_mass(atomic_numbers)
         free_energy = self.vibrational_free_energy_at_constant_volume
         specific_free_energy = free_energy / mass_per_unit_cell
-        return specific_free_energy
+
+        return specific_free_energy.magnitude
 
     vibrational_free_energy = Quantity(
         type=np.dtype(np.float64),
@@ -1287,6 +1319,264 @@ class Thermodynamics(MSection):
     stability = SubSection(sub_section=Stability.m_def, repeats=False)
 
 
+class ThermostatParameters(MSection):
+    '''
+    Section containing the parameters pertaining to the thermostat for a molecular dynamics run.
+    '''
+
+    m_def = Section(validate=False)
+
+    thermostat_type = Quantity(
+        type=MEnum('andersen', 'berendsen', 'brownian', 'langevin_goga', 'langevin_schneider', 'nose_hoover', 'velocity_rescaling',
+                   'velocity_rescaling_langevin'),
+        shape=[],
+        description='''
+        The name of the thermostat used for temperature control. If skipped or an empty string is used, it
+        means no thermostat was applied.
+
+        Allowed values are:
+
+        | Thermostat Name        | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `""`                   | No thermostat               |
+
+        | `"andersen"`           | H.C. Andersen, [J. Chem. Phys.
+        **72**, 2384 (1980)](https://doi.org/10.1063/1.439486) |
+
+        | `"berendsen"`          | H. J. C. Berendsen, J. P. M. Postma,
+        W. F. van Gunsteren, A. DiNola, and J. R. Haak, [J. Chem. Phys.
+        **81**, 3684 (1984)](https://doi.org/10.1063/1.448118) |
+
+        | `"brownian"`           | Brownian Dynamics |
+
+        | `"langevin_goga"`           | N. Goga, A. J. Rzepiela, A. H. de Vries,
+        S. J. Marrink, and H. J. C. Berendsen, [J. Chem. Theory Comput. **8**, 3637 (2012)]
+        (https://doi.org/10.1021/ct3000876) |
+
+        | `"langevin_schneider"`           | T. Schneider and E. Stoll,
+        [Phys. Rev. B **17**, 1302](https://doi.org/10.1103/PhysRevB.17.1302) |
+
+        | `"nose_hoover"`        | S. Nosé, [Mol. Phys. **52**, 255 (1984)]
+        (https://doi.org/10.1080/00268978400101201); W.G. Hoover, [Phys. Rev. A
+        **31**, 1695 (1985) |
+
+        | `"velocity_rescaling"` | G. Bussi, D. Donadio, and M. Parrinello,
+        [J. Chem. Phys. **126**, 014101 (2007)](https://doi.org/10.1063/1.2408420) |
+
+        | `"velocity_rescaling_langevin"` | G. Bussi and M. Parrinello,
+        [Phys. Rev. E **75**, 056707 (2007)](https://doi.org/10.1103/PhysRevE.75.056707) |
+        ''')
+
+    reference_temperature = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='kelvin',
+        description='''
+        The target temperature for the simulation.
+        ''')
+
+    coupling_constant = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='s',
+        description='''
+        The time constant for temperature coupling. Need to describe what this means for the various
+        thermostat options...
+        ''')
+
+
+class BarostatParameters(MSection):
+    '''
+    Section containing the parameters pertaining to the barostat for a molecular dynamics run.
+    '''
+
+    m_def = Section(validate=False)
+
+    barostat_type = Quantity(
+        type=MEnum('berendsen', 'martyna_tuckerman_tobias_klein', 'nose_hoover', 'parrinello_rahman', 'stochastic_cell_rescaling'),
+        shape=[],
+        description='''
+        The name of the barostat used for temperature control. If skipped or an empty string is used, it
+        means no barostat was applied.
+
+        Allowed values are:
+
+        | Barostat Name          | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `""`                   | No thermostat               |
+
+        | `"berendsen"`          | H. J. C. Berendsen, J. P. M. Postma,
+        W. F. van Gunsteren, A. DiNola, and J. R. Haak, [J. Chem. Phys.
+        **81**, 3684 (1984)](https://doi.org/10.1063/1.448118) |
+
+        | `"martyna_tuckerman_tobias_klein"` | G.J. Martyna, M.E. Tuckerman, D.J. Tobias, and M.L. Klein,
+        [Mol. Phys. **87**, 1117 (1996)](https://doi.org/10.1080/00268979600100761);
+        M.E. Tuckerman, J. Alejandre, R. López-Rendón, A.L. Jochim, and G.J. Martyna,
+        [J. Phys. A. **59**, 5629 (2006)](https://doi.org/10.1088/0305-4470/39/19/S18)|
+
+        | `"nose_hoover"`        | S. Nosé, [Mol. Phys. **52**, 255 (1984)]
+        (https://doi.org/10.1080/00268978400101201); W.G. Hoover, [Phys. Rev. A
+        **31**, 1695 (1985) |
+
+        | `"parrinello_rahman"`        | M. Parrinello and A. Rahman,
+        [J. Appl. Phys. **52**, 7182 (1981)](https://doi.org/10.1063/1.328693);
+        S. Nosé and M.L. Klein, [Mol. Phys. **50**, 1055 (1983) |
+
+        | `"stochastic_cell_rescaling"` | M. Bernetti and G. Bussi,
+        [J. Chem. Phys. **153**, 114107 (2020)](https://doi.org/10.1063/1.2408420) |
+        ''')
+
+    coupling_type = Quantity(
+        type=MEnum('isotropic', 'semi_isotropic', 'anisotropic'),
+        shape=[],
+        description='''
+        Describes the symmetry of pressure coupling. Specifics can be inferred from the `coupling constant`
+
+        | Type          | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `isotropic`          | Identical coupling in all directions. |
+
+        | `semi_isotropic` | Identical coupling in 2 directions. |
+
+        | `anisotropic`        | General case. |
+        ''')
+
+    reference_pressure = Quantity(
+        type=np.dtype(np.float64),
+        shape=[3, 3],
+        unit='pascal',
+        description='''
+        The target pressure for the simulation, stored in a 3x3 matrix, indicating the values for individual directions
+        along the diagonal, and coupling between directions on the off-diagonal.
+        ''')
+
+    coupling_constant = Quantity(
+        type=np.dtype(np.float64),
+        shape=[3, 3],
+        unit='s',
+        description='''
+        The time constants for pressure coupling, stored in a 3x3 matrix, indicating the values for individual directions
+        along the diagonal, and coupling between directions on the off-diagonal. 0 values along the off-diagonal
+        indicate no-coupling between these directions.
+        ''')
+
+    compressibility = Quantity(
+        type=np.dtype(np.float64),
+        shape=[3, 3],
+        unit='1 / pascal',
+        description='''
+        An estimate of the system's compressibility, used for box rescaling, stored in a 3x3 matrix indicating the values for individual directions
+        along the diagonal, and coupling between directions on the off-diagonal. If None, it may indicate that these values
+        are incorporated into the coupling_constant, or simply that the software used uses a fixed value that is not available in
+        the input/output files.
+        ''')
+
+
+class IntegrationParameters(MSection):
+    '''
+    Section containing the parameters for the molecular dynamics integrator.
+    '''
+
+    m_def = Section(validate=False)
+
+    integrator_type = Quantity(
+        type=MEnum(
+            'brownian', 'conjugant_gradient', 'langevin_goga',
+            'langevin_schneider', 'leap_frog', 'rRESPA_multitimescale', 'velocity_verlet'
+        ),
+        shape=[],
+        description='''
+        Name of the integrator.
+
+        Allowed values are:
+
+        | Integrator Name          | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `"langevin_goga"`           | N. Goga, A. J. Rzepiela, A. H. de Vries,
+        S. J. Marrink, and H. J. C. Berendsen, [J. Chem. Theory Comput. **8**, 3637 (2012)]
+        (https://doi.org/10.1021/ct3000876) |
+
+        | `"langevin_schneider"`           | T. Schneider and E. Stoll,
+        [Phys. Rev. B **17**, 1302](https://doi.org/10.1103/PhysRevB.17.1302) |
+
+        | `"leap_frog"`          | R.W. Hockney, S.P. Goel, and J. Eastwood,
+        [J. Comp. Phys. **14**, 148 (1974)](https://doi.org/10.1016/0021-9991(74)90010-2) |
+
+        | `"velocity_verlet"` | W.C. Swope, H.C. Andersen, P.H. Berens, and K.R. Wilson,
+        [J. Chem. Phys. **76**, 637 (1982)](https://doi.org/10.1063/1.442716) |
+
+        | `"rRESPA_multitimescale"` | M. Tuckerman, B. J. Berne, and G. J. Martyna
+        [J. Chem. Phys. **97**, 1990 (1992)](https://doi.org/10.1063/1.463137) |
+        ''')
+
+    integration_timestep = Quantity(
+        type=np.dtype(np.float64),
+        shape=[],
+        unit='s',
+        description='''
+        The timestep at which the numerical integration is performed.
+        ''')
+
+    n_steps = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Number of timesteps performed.
+        ''')
+
+    coordinate_save_frequency = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        The number of timesteps between saving the coordinates.
+        ''')
+
+    velocity_save_frequency = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        The number of timesteps between saving the velocities.
+        ''')
+
+    force_save_frequency = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        The number of timesteps between saving the forces.
+        ''')
+
+    thermodynamics_save_frequency = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        The number of timesteps between saving the thermodynamic quantities.
+        ''')
+
+    thermostat_parameters = SubSection(sub_section=ThermostatParameters.m_def, repeats=False)
+
+    barostat_parameters = SubSection(sub_section=BarostatParameters.m_def, repeats=False)
+
+
+class MolecularDynamicsResults(MSection):
+    '''
+    Section containing the parameters for sampling via molecular dynamics using a force field model.
+    '''
+
+    m_def = Section(validate=False)
+
+    radial_distribution_functions = SubSection(sub_section=SectionProxy('RadialDistributionFunction'), repeats=True)
+
+    mean_squared_displacements = SubSection(sub_section=SectionProxy('MeanSquaredDisplacement'), repeats=True)
+
+
 class MolecularDynamics(MSection):
     '''
     Section containing results of molecular dynamics workflow.
@@ -1295,19 +1585,25 @@ class MolecularDynamics(MSection):
     m_def = Section(
         validate=False)
 
-    ensemble_type = Quantity(
-        type=str,
+    thermodynamic_ensemble = Quantity(
+        type=MEnum('NVE', 'NVT', 'NPT', 'NPH'),
         shape=[],
         description='''
-        Ensemble type used.
-        ''')
+        The type of thermodynamic ensemble that was simulated.
 
-    timestep = Quantity(
-        type=np.dtype(np.float64),
-        shape=[],
-        unit='s',
-        description='''
-        Time step in the simulation.
+        Allowed values are:
+
+        | Thermodynamic Ensemble          | Description                               |
+
+        | ---------------------- | ----------------------------------------- |
+
+        | `"NVE"`           | Constant number of particles, volume, and energy |
+
+        | `"NVT"`           | Constant number of particles, volume, and temperature |
+
+        | `"NPT"`           | Constant number of particles, pressure, and temperature |
+
+        | `"NPH"`           | Constant number of particles, pressure, and enthalpy |
         ''')
 
     finished_normally = Quantity(
@@ -1331,9 +1627,32 @@ class MolecularDynamics(MSection):
         Indicates if calculation contains thermodynamic data.
         ''')
 
-    radial_distribution_functions = SubSection(sub_section=SectionProxy('RadialDistributionFunction'), repeats=True)
+    integration_parameters = SubSection(sub_section=IntegrationParameters.m_def, repeats=False)
 
-    mean_squared_displacements = SubSection(sub_section=SectionProxy('MeanSquaredDisplacement'), repeats=True)
+    results = SubSection(sub_section=MolecularDynamicsResults.m_def, repeats=False)
+
+
+class TrajectoryProperty(MSection):
+    '''
+    Generic section containing information about a calculation of any observable
+    defined and stored at each individual frame of a trajectory.
+    '''
+
+    m_def = Section(validate=False)
+
+    type = Quantity(
+        type=MEnum('molecular', 'atomic'),
+        shape=[],
+        description='''
+        Describes if the observable is calculated at the molecular or atomic level.
+        ''')
+
+    error_type = Quantity(
+        type=str,
+        shape=[],
+        description='''
+        Describes the type of error reported for this observable.
+        ''')
 
 
 class EnsemblePropertyValues(MSection):
@@ -1347,7 +1666,35 @@ class EnsemblePropertyValues(MSection):
         type=str,
         shape=[],
         description='''
-        Describes the atoms or molecule types involved in determining the property.
+        Describes the group of atoms/molecule/groups of molecules involved in determining the property.
+        ''')
+
+    atomsgroup_ref = Quantity(
+        type=Reference(AtomsGroup.m_def),
+        shape=[],
+        description='''
+        References to the atoms_group section containing the group of atoms/molecule/groups of molecules for which the property was calculated.
+        ''')
+
+    n_bins = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Number of bins.
+        ''')
+
+    frame_start = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Trajectory frame number where the ensemble averaging starts.
+        ''')
+
+    frame_end = Quantity(
+        type=int,
+        shape=[],
+        description='''
+        Trajectory frame number where the ensemble averaging ends.
         ''')
 
 
@@ -1360,7 +1707,7 @@ class EnsembleProperty(MSection):
     m_def = Section(validate=False)
 
     type = Quantity(
-        type=MEnum('Molecular', 'Atomic'),
+        type=MEnum('molecular', 'atomic'),
         shape=[],
         description='''
         Describes if the observable is calculated at the molecular or atomic level.
@@ -1389,7 +1736,7 @@ class EnsembleProperty(MSection):
         ''')
 
     variables_name = Quantity(
-        type=np.dtype(str),
+        type=str,
         shape=['n_variables'],
         description='''
         Name/description of the independent variables along which the observable is defined.
@@ -1404,13 +1751,6 @@ class RadialDistributionFunctionValues(EnsemblePropertyValues):
 
     m_def = Section(validate=False)
 
-    n_bins = Quantity(
-        type=int,
-        shape=[],
-        description='''
-        Number of bins.
-        ''')
-
     bins = Quantity(
         type=np.dtype(np.float64),
         shape=['n_bins'],
@@ -1449,7 +1789,14 @@ class CorrelationFunctionValues(MSection):
         type=str,
         shape=[],
         description='''
-        Describes the atoms or molecule types involved in determining the property.
+        Describes the group of atoms/molecule/groups of molecules involved in determining the property.
+        ''')
+
+    atomsgroup_ref = Quantity(
+        type=Reference(AtomsGroup.m_def),
+        shape=[1],
+        description='''
+        References to the atoms_group section containing the group of atoms/molecule/groups of molecules for which the property was calculated.
         ''')
 
     n_times = Quantity(
@@ -1469,7 +1816,7 @@ class CorrelationFunction(MSection):
     m_def = Section(validate=False)
 
     type = Quantity(
-        type=MEnum('Molecular', 'Atomic'),
+        type=MEnum('molecular', 'atomic'),
         shape=[],
         description='''
         Describes if the correlation function is calculated at the molecular or atomic level.
@@ -1543,6 +1890,13 @@ class MeanSquaredDisplacementValues(CorrelationFunctionValues):
         Msd values.
         ''')
 
+    errors = Quantity(
+        type=np.dtype(np.float64),
+        shape=['*'],
+        description='''
+        Error associated with the determination of the msds.
+        ''')
+
     diffusion_constant = SubSection(sub_section=DiffusionConstantValues.m_def, repeats=False)
 
 
@@ -1556,6 +1910,38 @@ class MeanSquaredDisplacement(CorrelationFunction):
     mean_squared_displacement_values = SubSection(sub_section=MeanSquaredDisplacementValues.m_def, repeats=True)
 
 
+class GW(MSection):
+    '''
+    Section containing results of a GW workflow
+    '''
+
+    m_def = Section(validate=False)
+
+    dos_dft = Quantity(
+        type=Reference(Dos),
+        description='''
+        DFT density of states
+        ''')
+
+    dos_gw = Quantity(
+        type=Reference(Dos),
+        description='''
+        GW density of states
+        ''')
+
+    band_structure_dft = Quantity(
+        type=Reference(BandStructure),
+        description='''
+        DFT density of states
+        ''')
+
+    band_structure_gw = Quantity(
+        type=Reference(BandStructure),
+        description='''
+        DFT density of states
+        ''')
+
+
 class SinglePoint(MSection):
     '''
     Section containing results of a single point workflow.
@@ -1688,6 +2074,7 @@ class Workflow(MSection):
 
     type = Quantity(
         type=MEnum([
+            "GW",
             "single_point",
             "geometry_optimization",
             "phonon",
@@ -1776,8 +2163,11 @@ class Workflow(MSection):
 
     single_point = SubSection(
         sub_section=SinglePoint.m_def,
-        # TODO determine if there is a need for this to be a repeating section
-        # such as in the context of fhi-vibes single_point
+        repeats=False,
+        categories=[FastAccess])
+
+    gw = SubSection(
+        sub_section=GW.m_def,
         repeats=False,
         categories=[FastAccess])
 
