@@ -45,7 +45,7 @@ original mainfile, and vice versa.
 
 from abc import ABCMeta
 import sys
-from typing import IO, Dict, Iterable, Iterator, List, Tuple, Any, NamedTuple
+from typing import IO, Set, Dict, Iterable, Iterator, List, Tuple, Any, NamedTuple
 from functools import lru_cache
 from pydantic import BaseModel
 from datetime import datetime
@@ -71,8 +71,7 @@ if sys.version_info >= (3, 7):
 else:
     import zipfile37 as zipfile
 
-zip_file_extensions = ('.zip',)
-tar_file_extensions = ('.tgz', '.gz', '.tar.gz', '.tar.bz2', '.tar')
+decompress_file_extensions = ('.zip', '.tgz', '.gz', '.tar.gz', '.tar.bz2', '.tar')
 bundle_info_filename = 'bundle_info.json'
 
 # Used to check if zip-files/archive files are empty
@@ -81,6 +80,34 @@ empty_zip_file_size = 22
 empty_archive_file_size = 32
 
 
+def auto_decompress(path: str):
+    '''
+    Returns the decompression format ('zip', 'tar' or 'error') if `path` specifies a file
+    which should be automatically decompressed before adding it to an upload. If `path`
+    does *not* specify a file which we think should be decompressed, or if it specifies a
+    directory, we return None.
+
+    The value 'error' means that we think this file should be decompressed, but that we cannot
+    decompress it. This indicates that the file is corrupted, has a bad file format, or has
+    the wrong file extension.
+
+    Note, some files, like for example excel files, are actually zip files, and we don't want
+    to extract such files. Therefore, we only auto decompress if the file has an extension
+    we recognize as decompressable, like ".zip", ".tar" etc.
+    '''
+    if os.path.isdir(path):
+        return None
+    basename_lower = os.path.basename(path).lower()
+    for extension in decompress_file_extensions:
+        if basename_lower.endswith(extension):
+            if tarfile.is_tarfile(path):
+                return 'tar'
+            elif zipfile.is_zipfile(path):
+                return 'zip'
+            return 'error'
+    return None
+
+
 def copytree(src, dst):
     '''
     A close on ``shutils.copytree`` that does not try to copy the stats on all files.
@@ -228,7 +255,10 @@ class FileSource(metaclass=ABCMeta):
     The files in the source are associated with paths and have known sizes.
     '''
     def to_streamed_files(self) -> Iterable[StreamedFile]:
-        ''' Retrieves the files in the source as :class:`StreamedFile` objects. '''
+        '''
+        Retrieves the files in the source as :class:`StreamedFile` objects.
+        The caller should close the streams when consumed.
+        '''
         raise NotImplementedError()
 
     def to_zipstream(self) -> Iterator[bytes]:
@@ -265,8 +295,9 @@ class FileSource(metaclass=ABCMeta):
                 PathObject(os_path).delete()
             os.makedirs(dir_path, exist_ok=True)
             with open(os_path, 'wb') as output_file:
-                for chunk in streamed_file.f:
-                    output_file.write(chunk)
+                with streamed_file.f:
+                    for chunk in streamed_file.f:
+                        output_file.write(chunk)
 
     def close(self):
         ''' Perform "closing" of the source, if applicable. '''
@@ -427,22 +458,18 @@ class ZipFileSource(BrowsableFileSource):
 class CombinedFileSource(FileSource):
     '''
     Class for defining a :class:`FileSource` by combining multiple "subsources" into one.
-    New sources are added using :func:`add_file_source`.
     '''
-    def __init__(self):
-        self.sources = []
-
-    def add_file_source(self, file_source: FileSource):
-        assert isinstance(file_source, FileSource)
-        self.sources.append(file_source)
+    def __init__(self, file_sources=Iterable[FileSource]):
+        ''' file_sources: an Iterable for getting FileSources. '''
+        self.file_sources = file_sources
 
     def to_streamed_files(self) -> Iterable[StreamedFile]:
-        for file_source in self.sources:
+        for file_source in self.file_sources:
             for streamed_file in file_source.to_streamed_files():
                 yield streamed_file
 
     def to_disk(self, destination_dir: str, move_files: bool = False, overwrite: bool = False):
-        for file_source in self.sources:
+        for file_source in self.file_sources:
             file_source.to_disk(destination_dir, move_files, overwrite)
 
 
@@ -484,11 +511,12 @@ def create_zipstream_content(streamed_files: Iterable[StreamedFile]) -> Iterable
     for streamed_file in streamed_files:
 
         def content_generator():
-            while True:
-                data = streamed_file.f.read(1024 * 64)
-                if not data:
-                    break
-                yield data
+            with streamed_file.f as f:
+                while True:
+                    data = f.read(1024 * 64)
+                    if not data:
+                        break
+                    yield data
 
         yield dict(
             arcname=streamed_file.path,
@@ -529,6 +557,17 @@ class UploadFiles(DirectoryObject, metaclass=ABCMeta):
         '''
         raise NotImplementedError()
 
+    @property
+    def external_os_path(self):
+        '''
+        Full path to where the upload files of this class are stored on the server.
+        This is equal to `self.os_path` if no external path substitutes for staging
+        and public area are configured. This is helpful, when nomad is run in a container
+        and the mounted path used by nomad are different from the actual paths on the
+        host server.
+        '''
+        raise NotImplementedError()
+
     @classmethod
     def base_folder_for(cls, upload_id: str) -> str:
         '''
@@ -641,26 +680,22 @@ class UploadFiles(DirectoryObject, metaclass=ABCMeta):
                     utils.get_logger(__name__).error(
                         'could not remove empty prefix dir', directory=parent_directory, exc_info=e)
 
-    def files_to_bundle(
-            self, bundle_info: Dict[str, Any],
-            include_raw_files: bool,
-            include_archive_files: bool) -> FileSource:
+    def files_to_bundle(self, export_settings: config.BundleExportSettings) -> Iterable[FileSource]:
         '''
-        Returns a :class:`FileSource`, defining the files/folders to be included in an
-        upload bundle when *exporting*. Note, the bundle_info.json file is not included,
-        only the "regular" files, and only those specified by the arguments to the method.
+        A generator of :class:`FileSource` objects, defining the files/folders to be included in an
+        upload bundle when *exporting*. The arguments allows for further filtering of what to include.
+
+        Note, this only yields files to copy from the regular upload directory, not "special" files,
+        like the bundle_info.json file, which is created by the :class:`BundleExporter`.
         '''
         raise NotImplementedError()
 
     @classmethod
     def files_from_bundle(
-            cls, bundle_file_source: BrowsableFileSource,
-            include_raw_files: bool,
-            include_archive_files: bool,
-            include_bundle_info: bool) -> FileSource:
+            cls, bundle_file_source: BrowsableFileSource, import_settings: config.BundleImportSettings) -> Iterable[FileSource]:
         '''
-        Returns a :class:`FileSource`, defining the files/folders to be included in an
-        upload bundle when *importing*. Only the files specified by the arguments are included.
+        Returns an Iterable of :class:`FileSource`, defining the files/folders to be included in an
+        upload bundle when *importing*. Only the files specified by the import_settings are included.
         '''
         raise NotImplementedError()
 
@@ -679,6 +714,13 @@ class StagingUploadFiles(UploadFiles):
     def file_area(cls):
         return config.fs.staging
 
+    @property
+    def external_os_path(self):
+        if not config.fs.staging_external:
+            return self.os_path
+
+        return self.os_path.replace(config.fs.staging, config.fs.staging_external)
+
     def to_staging_upload_files(self, create: bool = False, include_archive: bool = False) -> 'StagingUploadFiles':
         return self
 
@@ -707,6 +749,10 @@ class StagingUploadFiles(UploadFiles):
             return False
         return os.path.isfile(os.path.join(self._raw_dir.os_path, path))
 
+    def raw_create_directory(self, path: str):
+        assert path and is_safe_relative_path(path), 'Bad path provided'
+        os.makedirs(os.path.join(self._raw_dir.os_path, path))
+
     def raw_directory_list(
             self, path: str = '', recursive=False, files_only=False, path_prefix=None) -> Iterable[RawPathInfo]:
         if not is_safe_relative_path(path):
@@ -771,10 +817,12 @@ class StagingUploadFiles(UploadFiles):
             raise KeyError(entry_id)
 
     def archive_file_object(self, entry_id: str) -> PathObject:
-        return self._archive_dir.join_file(f'{entry_id}.msg')
+        version_suffix = '-' + config.fs.archive_version_suffix if config.fs.archive_version_suffix else ''
+        return self._archive_dir.join_file(f'{entry_id}{version_suffix}.msg')
 
     def add_rawfiles(
-            self, path: str, target_dir: str = '', cleanup_source_file_and_dir: bool = False) -> None:
+            self, path: str, target_dir: str = '', cleanup_source_file_and_dir: bool = False,
+            updated_files: Set[str] = None) -> None:
         '''
         Adds the file or folder specified by `path` to this upload, in the raw directory
         specified by `target_dir`. If `path` denotes a zip or tar archive file, it will
@@ -794,72 +842,65 @@ class StagingUploadFiles(UploadFiles):
             target_dir: A raw path (i.e. path relative to the raw directory) defining
                 where the resource defined by `path` should be put. If `target_dir` is not
                 specified, it defaults to the empty string, i.e. the upload's raw dir.
-            cleanup_source_file_and_dir: If true, the source file (defined by `path`) and
-                its parent folder are included in the cleanup step - i.e. they are always
-                deleted. Use when the file is stored temporarily.
+            cleanup_source_file_and_dir: If true, the source file/folder (defined by `path`) is
+                deleted when done (regardless of success or failure). Additionally, the parent
+                folder is deleted if it's empty or if the operation failed. Use when the file/folder
+                to add is stored in a temporary directory.
+            updated_files: An optional set of paths. If provided with the call, the raw
+                path of all files added or updated by the operation will be added to this set.
         '''
         tmp_dir = None
         try:
             assert not self.is_frozen
             assert os.path.exists(path), f'{path} does not exist'
             assert is_safe_relative_path(target_dir)
-            ext = os.path.splitext(path)[1]
             self._size += os.stat(path).st_size
 
             is_dir = os.path.isdir(path)
-            if is_dir:
-                is_zipfile = is_tarfile = False
-            else:
-                is_zipfile = zipfile.is_zipfile(path) or ext in zip_file_extensions
-                is_tarfile = tarfile.is_tarfile(path) or ext in tar_file_extensions
-                if is_zipfile or is_tarfile:
-                    tmp_dir = create_tmp_dir(self.upload_id + '_unzip')
-                    if is_zipfile:
-                        with zipfile.ZipFile(path) as zf:
-                            zf.extractall(tmp_dir)
-                    elif is_tarfile:
-                        with tarfile.open(path) as tf:
-                            tf.extractall(tmp_dir)
+            decompress = auto_decompress(path)
+            if decompress:
+                tmp_dir = create_tmp_dir(self.upload_id + '_unzip')
+                if decompress == 'zip':
+                    with zipfile.ZipFile(path) as zf:
+                        zf.extractall(tmp_dir)
+                elif decompress == 'tar':
+                    with tarfile.open(path) as tf:
+                        tf.extractall(tmp_dir)
+                elif decompress == 'error':
+                    # Unknown / bad file format
+                    assert False, 'Cannot extract file. Bad file format or file extension?'
 
             # Determine what to merge
             elements_to_merge: Iterable[Tuple[str, List[str], List[str]]] = []
-            if is_dir or is_zipfile or is_tarfile:
+            if is_dir:
                 # Directory
-                source_dir = path if is_dir else tmp_dir
+                source_dir = path
+                elements_to_merge = os.walk(source_dir)
+            elif decompress:
+                # Zipped archive
+                source_dir = tmp_dir
                 elements_to_merge = os.walk(source_dir)
             else:
-                # Single file
+                # Single, non-compressed file
                 source_dir = os.path.dirname(path)
                 elements_to_merge = [(source_dir, [], [os.path.basename(path)])]
 
-            # Ensure target_dir exists and is a directory. If one of the elements in the
-            # directory chain is a file, it needs to be deleted (the regular os.makedirs
-            # doesn't do that).
-            target_dir_subpath = self._raw_dir.os_path
-            for dir_name in target_dir.split(os.path.sep):
-                target_dir_subpath = os.path.join(target_dir_subpath, dir_name)
-                if os.path.isfile(target_dir_subpath):
-                    os.remove(target_dir_subpath)
-                if not os.path.isdir(target_dir_subpath):
-                    os.makedirs(target_dir_subpath)
-
             # Do the merge
-            for root, dirs, files in elements_to_merge:
+            os_target_dir = os.path.join(self._raw_dir.os_path, target_dir)
+            if not os.path.isdir(os_target_dir):
+                os.makedirs(os_target_dir)
+            for source_root, dirs, files in elements_to_merge:
                 elements = dirs + files
-                os_target_dir = os.path.join(self._raw_dir.os_path, target_dir)
                 for element in elements:
-                    element_source_path = os.path.join(root, element)
+                    element_source_path = os.path.join(source_root, element)
                     element_relative_path = os.path.relpath(element_source_path, source_dir)
                     element_target_path = os.path.join(os_target_dir, element_relative_path)
                     if os.path.islink(element_source_path):
                         continue  # Skip links, could pose security risk
                     if os.path.exists(element_target_path):
-                        if not (os.path.isdir(element_source_path) and os.path.isdir(element_target_path)):
-                            # Target already exists and needs to be deleted
-                            if os.path.isdir(element_target_path):
-                                shutil.rmtree(element_target_path)
-                            else:
-                                os.remove(element_target_path)
+                        if os.path.isfile(element_target_path) != os.path.isfile(element_source_path):
+                            assert False, f'Cannot merge a file with a directory or vice versa: {element_relative_path}'
+
                     # Copy or move the element
                     if os.path.isdir(element_source_path):
                         # Directory - just create corresponding directory in the target if needed.
@@ -867,12 +908,20 @@ class StagingUploadFiles(UploadFiles):
                             os.makedirs(element_target_path)
                     else:
                         # File - copy or move it
-                        if cleanup_source_file_and_dir or is_zipfile or is_tarfile:
+                        if cleanup_source_file_and_dir or decompress:
                             # Move the file
                             shutil.move(element_source_path, element_target_path)
                         else:
                             # Copy the file
                             shutil.copyfile(element_source_path, element_target_path)
+                        if updated_files is not None:
+                            updated_files.add(os.path.join(target_dir, element_relative_path))
+        except Exception:
+            if cleanup_source_file_and_dir:
+                parent_dir = os.path.dirname(path)
+                if os.path.exists(parent_dir):
+                    shutil.rmtree(parent_dir)
+            raise
         finally:
             # Cleanup
             if tmp_dir and os.path.exists(tmp_dir):
@@ -884,21 +933,60 @@ class StagingUploadFiles(UploadFiles):
                     else:
                         os.remove(path)
                 parent_dir = os.path.dirname(path)
-                if os.path.exists(parent_dir):
+                if os.path.exists(parent_dir) and not os.listdir(parent_dir):
                     shutil.rmtree(parent_dir)
 
-    def delete_rawfiles(self, path):
+    def delete_rawfiles(self, path, updated_files: Set[str] = None):
         assert is_safe_relative_path(path)
-        os_path = os.path.join(self.os_path, 'raw', path)
-        assert os.path.exists(os_path)
+        raw_os_path = os.path.join(self.os_path, 'raw')
+        os_path = os.path.join(raw_os_path, path)
+        if not os.path.exists(os_path):
+            return
         if os.path.isfile(os_path):
+            # Deleting a file
+            if updated_files is not None:
+                updated_files.add(path)
             os.remove(os_path)
         else:
+            # Deleting a directory
+            if updated_files is not None:
+                for dirname, _, filenames in os.walk(os_path):
+                    for filename in filenames:
+                        file_os_path = os.path.join(dirname, filename)
+                        file_raw_path = os.path.relpath(file_os_path, raw_os_path)
+                        updated_files.add(file_raw_path)
             shutil.rmtree(os_path)
         if path == '':
             # Special case - deleting everything, i.e. the entire raw folder. Need to recreate.
             os.makedirs(os_path)
 
+    def copy_or_move_rawfile(self, path_to_existing_file, path_to_target_file, copy_or_move, updated_files: Set[str] = None):
+        assert is_safe_relative_path(path_to_existing_file)
+        assert is_safe_relative_path(path_to_target_file)
+        os_path_exisitng = os.path.join(self._raw_dir.os_path, path_to_existing_file)
+        os_path_target = os.path.join(self._raw_dir.os_path, path_to_target_file)
+        if not os.path.exists(os_path_exisitng):
+            return
+        if os.path.isfile(os_path_exisitng):
+            # copying or moving a file
+            if os.path.exists(os_path_target):
+                raise ValueError('A file with the same name already exists.')
+            if copy_or_move.lower() == 'copy':
+                shutil.copyfile(os_path_exisitng, os_path_target)
+            elif copy_or_move.lower() == 'move':
+                shutil.move(os_path_exisitng, os_path_target)
+
+            if updated_files is not None:
+                updated_files.add(path_to_target_file)
+                # if both the new and old name are the same then no new entry will be
+                # added to the set. but if different, we add the old one so that later on
+                # when self.matchall is called in data.py, the old filename is removed
+                # from mongo database
+                updated_files.add(path_to_existing_file)
+
+        elif os.path.isdir(os_path_target):
+            raise ValueError('Copying a directory is not possible.')
+
     @lru_cache()
     def metadata_file_cached(self, path_dir: str = ''):
         '''
@@ -1013,8 +1101,11 @@ class StagingUploadFiles(UploadFiles):
             with zipfile.ZipFile(raw_zip_file_object.os_path, mode='w') as raw_zip:
                 for path_info in self.raw_directory_list(recursive=True):
                     basename = os.path.basename(path_info.path)
-                    if basename.startswith('POTCAR') and not basename.endswith('.stripped'):
-                        continue  # Skip POTCAR files when publishing
+                    if basename.startswith('POTCAR'):
+                        if not basename.endswith('.stripped'):
+                            continue  # Skip the unstripped POTCAR files when publishing
+                        if basename.endswith('.stripped.stripped'):
+                            continue  # Skip redundantly stripped POTCAR files (created due to bug #979) when publishing
                     raw_zip.write(self._raw_dir.join_file(path_info.path).os_path, path_info.path)
             # Remove the zip file with the opposite access, if it exists
             other_raw_zip_file_object = PublicUploadFiles._create_raw_zip_file_object(target_dir, other_access)
@@ -1030,7 +1121,7 @@ class StagingUploadFiles(UploadFiles):
         nomad's logic about what is part of an entry and what not. The mainfile
         is the first element, the rest is sorted.
         Arguments:
-            mainfile: The mainfile relative to upload
+            mainfile: The mainfile path relative to upload
             with_mainfile: Do include the mainfile, default is True
         '''
         mainfile_object = self._raw_dir.join_file(mainfile)
@@ -1043,12 +1134,14 @@ class StagingUploadFiles(UploadFiles):
 
         file_count = 0
         aux_files: List[str] = []
-        for filename in os.listdir(entry_dir):
-            if filename != mainfile_basename and os.path.isfile(os.path.join(entry_dir, filename)):
-                aux_files.append(os.path.join(entry_relative_dir, filename))
+        dir_elements = os.listdir(entry_dir)
+        dir_elements.sort()
+        for dir_element in dir_elements:
+            if dir_element != mainfile_basename and os.path.isfile(os.path.join(entry_dir, dir_element)):
+                aux_files.append(os.path.join(entry_relative_dir, dir_element))
                 file_count += 1
 
-            if with_cutoff and file_count > config.auxfile_cutoff:
+            if with_cutoff and file_count > config.process.auxfile_cutoff:
                 # If there are too many of them, its probably just a directory with lots of
                 # mainfiles/entries. In this case it does not make any sense to provide thousands of
                 # aux files.
@@ -1061,11 +1154,13 @@ class StagingUploadFiles(UploadFiles):
         else:
             return aux_files
 
-    def entry_hash(self, mainfile: str) -> str:
+    def entry_hash(self, mainfile: str, mainfile_key: str) -> str:
         '''
         Calculates a hash for the given entry based on file contents and aux file contents.
         Arguments:
-            mainfile: The mainfile path relative to the upload that identifies the entry in the folder structure.
+            mainfile: The mainfile path relative to the upload that identifies the entry in
+                the folder structure.
+            mainfile_key: The mainfile_key of the entry (if any)
         Returns:
             The calculated hash
         Raises:
@@ -1076,36 +1171,27 @@ class StagingUploadFiles(UploadFiles):
             with open(self._raw_dir.join_file(filepath).os_path, 'rb') as f:
                 for data in iter(lambda: f.read(65536), b''):
                     hash.update(data)
-
+        if mainfile_key:
+            hash.update(mainfile_key.encode('utf8'))
         return utils.make_websave(hash)
 
-    def files_to_bundle(
-            self, bundle_info: Dict[str, Any],
-            include_raw_files: bool, include_archive_files: bool) -> FileSource:
-        # Files to export for staging uploads.
-        rv = CombinedFileSource()
-        rv.add_file_source(StreamedFileSource(json_to_streamed_file(bundle_info, bundle_info_filename)))
-        if include_raw_files:
-            rv.add_file_source(DiskFileSource(self.os_path, 'raw'))
-        if include_archive_files:
-            rv.add_file_source(DiskFileSource(self.os_path, 'archive'))
-        return rv
+    def files_to_bundle(self, export_settings: config.BundleExportSettings) -> Iterable[FileSource]:
+        # Defines files for upload bundles of staging uploads.
+        if export_settings.include_raw_files:
+            yield DiskFileSource(self.os_path, 'raw')
+        if export_settings.include_archive_files:
+            yield DiskFileSource(self.os_path, 'archive')
 
     @classmethod
     def files_from_bundle(
-            cls, bundle_file_source: BrowsableFileSource,
-            include_raw_files: bool,
-            include_archive_files: bool,
-            include_bundle_info: bool) -> FileSource:
+            cls, bundle_file_source: BrowsableFileSource, import_settings: config.BundleImportSettings) -> Iterable[FileSource]:
         # Files to import for a staging upload
-        rv = CombinedFileSource()
-        if include_raw_files:
-            rv.add_file_source(bundle_file_source.sub_source('raw'))
-        if include_archive_files:
-            rv.add_file_source(bundle_file_source.sub_source('archive'))
-        if include_bundle_info:
-            rv.add_file_source(bundle_file_source.sub_source(bundle_info_filename))
-        return rv
+        if import_settings.include_raw_files:
+            yield bundle_file_source.sub_source('raw')
+        if import_settings.include_archive_files:
+            yield bundle_file_source.sub_source('archive')
+        if import_settings.include_bundle_info:
+            yield bundle_file_source.sub_source(bundle_info_filename)
 
 
 class PublicUploadFiles(UploadFiles):
@@ -1118,11 +1204,19 @@ class PublicUploadFiles(UploadFiles):
         self._archive_msg_file_object: PathObject = None
         self._archive_msg_file: ArchiveReader = None
         self._access: str = None
+        self._missing_raw_files: bool = None
 
     @classmethod
     def file_area(cls):
         return config.fs.public
 
+    @property
+    def external_os_path(self):
+        if not config.fs.public_external:
+            return self.os_path
+
+        self.os_path.replace(config.fs.public, config.fs.public_external)
+
     def close(self):
         if self._raw_zip_file is not None:
             self._raw_zip_file.close()
@@ -1187,6 +1281,12 @@ class PublicUploadFiles(UploadFiles):
 
         return self._raw_zip_file
 
+    @property
+    def missing_raw_files(self):
+        if self._missing_raw_files is None:
+            self._missing_raw_files = not os.path.exists(self.raw_zip_file_object().os_path)
+        return self._missing_raw_files
+
     @staticmethod
     def _create_msg_file_object(target_dir: DirectoryObject, access: str) -> PathObject:
         version_suffix = '-' + config.fs.archive_version_suffix if config.fs.archive_version_suffix else ''
@@ -1291,6 +1391,8 @@ class PublicUploadFiles(UploadFiles):
     def raw_path_exists(self, path: str) -> bool:
         if not is_safe_relative_path(path):
             return False
+        if self.missing_raw_files:
+            return not path  # We consider the empty path (i.e. root) to always "exists".
         self._parse_content()
         explicit_directory_path = path.endswith(os.path.sep)
         path = path.rstrip(os.path.sep)
@@ -1308,7 +1410,7 @@ class PublicUploadFiles(UploadFiles):
         return False
 
     def raw_path_is_file(self, path: str) -> bool:
-        if not is_safe_relative_path(path):
+        if not is_safe_relative_path(path) or self.missing_raw_files:
             return False
         self._parse_content()
         base_name = os.path.basename(path)
@@ -1325,6 +1427,8 @@ class PublicUploadFiles(UploadFiles):
             self, path: str = '', recursive=False, files_only=False, path_prefix=None) -> Iterable[RawPathInfo]:
         if not is_safe_relative_path(path):
             return
+        if not path and self.missing_raw_files:
+            return
         self._parse_content()
         path = path.rstrip(os.path.sep)
         directory_content = self._directories.get(path)
@@ -1416,96 +1520,21 @@ class PublicUploadFiles(UploadFiles):
         self._raw_zip_file = self._raw_zip_file_object = None
         self._archive_msg_file = self._archive_msg_file_object = None
 
-    def files_to_bundle(
-            self, bundle_info: Dict[str, Any],
-            include_raw_files: bool, include_archive_files: bool) -> FileSource:
+    def files_to_bundle(self, export_settings: config.BundleExportSettings) -> Iterable[FileSource]:
         # Defines files for upload bundles of published uploads.
-        rv = CombinedFileSource()
-        rv.add_file_source(StreamedFileSource(json_to_streamed_file(bundle_info, bundle_info_filename)))
-        for filename in os.listdir(self.os_path):
-            if filename.startswith('raw-') and include_raw_files:
-                rv.add_file_source(DiskFileSource(self.os_path, filename))
-            if filename.startswith('archive-') and include_archive_files:
-                rv.add_file_source(DiskFileSource(self.os_path, filename))
-        return rv
+        for filename in sorted(os.listdir(self.os_path)):
+            if filename.startswith('raw-') and export_settings.include_raw_files:
+                yield DiskFileSource(self.os_path, filename)
+            if filename.startswith('archive-') and export_settings.include_archive_files:
+                yield DiskFileSource(self.os_path, filename)
 
     @classmethod
     def files_from_bundle(
-            cls, bundle_file_source: BrowsableFileSource,
-            include_raw_files: bool,
-            include_archive_files: bool,
-            include_bundle_info: bool) -> FileSource:
-        rv = CombinedFileSource()
+            cls, bundle_file_source: BrowsableFileSource, import_settings: config.BundleImportSettings) -> Iterable[FileSource]:
         for filename in bundle_file_source.directory_list(''):
-            if filename.startswith('raw-') and include_raw_files:
-                rv.add_file_source(bundle_file_source.sub_source(filename))
-            if filename.startswith('archive-') and include_archive_files:
-                rv.add_file_source(bundle_file_source.sub_source(filename))
-            if filename == bundle_info_filename and include_bundle_info:
-                rv.add_file_source(bundle_file_source.sub_source(filename))
-        return rv
-
-
-class UploadBundle:
-    '''
-    Class for handling file-related logic for an *upload bundle*. Upload bundles are used
-    to import and export uploads between different NOMAD installations.
-    '''
-    def __init__(self, path: str):
-        ''' Creates an UploadBundle instance. The `path` should denote a zipfile or a folder. '''
-        self.path = path
-        self.file_source: BrowsableFileSource = None
-        self._bundle_info: Dict[str, Any] = None
-        if os.path.isdir(path):
-            self.file_source = DiskFileSource(path)
-        else:
-            assert zipfile.is_zipfile(path), '`path` must define a folder or a zipfile.'
-            zip_file = zipfile.ZipFile(path, 'r')
-            self.file_source = ZipFileSource(zip_file)
-
-    @property
-    def bundle_info(self) -> Dict[str, Any]:
-        if self._bundle_info is None:
-            with self.file_source.open(bundle_info_filename, 'rt') as f:
-                self._bundle_info = json.load(f, cls=StandardJSONDecoder)
-        return self._bundle_info
-
-    def import_upload_files(
-            self, include_raw_files: bool, include_archive_files: bool, include_bundle_info: bool,
-            move_files: bool) -> UploadFiles:
-        '''
-        Creates an :class:`UploadFiles` object of the right type and imports the selected
-        files to it. The target folder must not already exist.
-        '''
-        try:
-            upload_files: UploadFiles = None
-            upload_id: str = self.bundle_info['upload_id']
-            published: bool = self.bundle_info.get('upload', {}).get('publish_time') is not None
-            cls = PublicUploadFiles if published else StagingUploadFiles
-            assert not os.path.exists(cls.base_folder_for(upload_id)), 'Upload folder already exists'
-            upload_files = cls(upload_id, create=True)
-            import_file_source = upload_files.files_from_bundle(
-                self.file_source, include_raw_files, include_archive_files, include_bundle_info)
-            import_file_source.to_disk(upload_files.os_path, move_files=move_files, overwrite=True)
-            return upload_files
-        except Exception:
-            # Some thing went wrong. Delete the files and re-raise the original exception
-            if upload_files:
-                upload_files.delete()
-            raise
-
-    def close(self):
-        self.file_source.close()
-
-    def delete(self, include_parent_folder: bool = False):
-        '''
-        Deletes the bundle files. If `include_parent_folder` is set, and the parent folder
-        is empty, it is also deleted.
-        '''
-        self.close()
-        if os.path.exists(self.path):
-            PathObject(self.path).delete()
-        if include_parent_folder:
-            parent_folder = os.path.dirname(self.path)
-            if not os.listdir(parent_folder):
-                PathObject(parent_folder).delete()
+            if filename.startswith('raw-') and import_settings.include_raw_files:
+                yield bundle_file_source.sub_source(filename)
+            if filename.startswith('archive-') and import_settings.include_archive_files:
+                yield bundle_file_source.sub_source(filename)
+            if filename == bundle_info_filename and import_settings.include_bundle_info:
+                yield bundle_file_source.sub_source(filename)
