@@ -40,14 +40,18 @@ import re
 import unidecode
 
 from nomad import config, utils
+from nomad.utils.structlogging import get_logger
 
 # The metainfo is defined and used during imports. This is problematic.
 # We import all parsers very early in the infrastructure setup. This will populate
 # the metainfo with parser specific definitions, before the metainfo might be used.
 from nomad.parsing import parsers  # pylint: disable=unused-import
 
+# TODO put somemore thought into warnings
+import warnings
+warnings.filterwarnings("ignore")
 
-logger = utils.get_logger(__name__)
+logger = get_logger(__name__)
 
 elastic_client = None
 ''' The elastic search client. '''
@@ -111,7 +115,6 @@ class Keycloak():
     '''
     def __init__(self):
         self.__oidc_client = None
-        self.__admin_client = None
         self.__public_keys = None
 
     @property
@@ -141,6 +144,9 @@ class Keycloak():
 
         return self.__public_keys
 
+    def refresh_token(self, access_token: str, refresh_token: str, **kwargs) -> str:
+        return self._oidc_client.refresh_token(refresh_token)
+
     def basicauth(self, username: str, password: str) -> str:
         '''
         Performs basic authentication and returns an access token.
@@ -158,16 +164,7 @@ class Keycloak():
 
         return token_info['access_token']
 
-    def tokenauth(self, access_token: str) -> object:
-        '''
-        Authenticates the given access_token
-
-        Returns:
-            The user
-
-        Raises:
-            KeycloakError
-        '''
+    def decode_access_token(self, access_token: str) -> dict:
         try:
             kid = jwt.get_unverified_header(access_token)['kid']
             key = keycloak._public_keys.get(kid)
@@ -180,9 +177,21 @@ class Keycloak():
 
             issuer = f'{config.keycloak.public_server_url.rstrip("/")}/realms/{config.keycloak.realm_name}'
             options = dict(verify_aud=False, verify_exp=True, verify_iss=True)
-            payload = jwt.decode(
+            return jwt.decode(
                 access_token, key=key, algorithms=['RS256'], options=options,
                 issuer=issuer)
+        except jwt.InvalidTokenError:
+            raise KeycloakError('Could not validate credentials. The given token is invalid.')
+
+    def tokenauth(self, access_token: str) -> object:
+        '''
+        Authenticates the given access_token
+
+        Returns:
+            The user
+        '''
+        try:
+            payload = self.decode_access_token(access_token)
 
             user_id: str = payload.get('sub')
             if user_id is None:
@@ -193,17 +202,91 @@ class Keycloak():
             from nomad import datamodel
             return datamodel.User(
                 user_id=user_id,
-                email=payload.get('email', None),
                 username=payload.get('preferred_username', None),
+                email=payload.get('email', None),
                 first_name=payload.get('given_name', None),
                 last_name=payload.get('family_name', None))
 
-        except jwt.InvalidTokenError:
-            raise KeycloakError('Could not validate credentials. The given token is invalid.')
         except Exception as e:
             logger.error('cannot perform tokenauth', exc_info=e)
             raise e
 
+
+keycloak = Keycloak()
+
+
+class UserManagement():
+    def add_user(self, user, bcrypt_password=None, invite=False):
+        '''
+        Adds the given :class:`nomad.datamodel.User` instance to the configured keycloak
+        realm using the keycloak admin API.
+        '''
+        raise NotImplementedError()
+
+    def search_user(self, query: str):
+        raise NotImplementedError()
+
+    def get_user(self, user_id: str = None, username: str = None, email: str = None):
+        '''
+        Retrives all available information about a user from the local keycloak admin
+        interface or the central NOMAD installation. This can be used to retrieve
+        complete user information, because the info solely gathered from tokens is generally
+        incomplete.
+        '''
+        raise NotImplementedError()
+
+
+class OasisUserManagement(UserManagement):
+    def __init__(self, users_api_url: str = None):
+        if users_api_url:
+            self._users_api_url = users_api_url
+        else:
+            self._users_api_url = f'{config.oasis.central_nomad_deployment_url}/v1/users'
+
+    def add_user(self, user, bcrypt_password=None, invite=False):
+        raise NotImplementedError(
+            'Adding a user is not possible for an Oasis using the central user management.')
+
+    def __user_from_api_user(self, api_user):
+        from nomad import datamodel
+        return datamodel.User.m_from_dict(api_user)
+
+    def search_user(self, query: str):
+        import requests
+        response = requests.get(self._users_api_url, params=dict(prefix=query))
+        if response.status_code != 200:
+            raise KeycloakError('Could not request central nomad\'s user management.')
+
+        return list(self.__user_from_api_user(user) for user in response.json()['data'])
+
+    def get_user(self, user_id: str = None, username: str = None, email: str = None):
+        import requests
+
+        kwargs = {}
+        if user_id:
+            kwargs['user_id'] = user_id
+        elif username:
+            kwargs['username'] = username
+        elif email:
+            kwargs['email'] = email
+        else:
+            return None
+
+        response = requests.get(self._users_api_url, params=kwargs)
+        if response.status_code != 200:
+            raise KeycloakError('Could not request central nomad\'s user management.')
+
+        data = response.json()
+        if len(data['data']) == 0:
+            return None
+
+        return self.__user_from_api_user(data['data'][0])
+
+
+class KeycloakUserManagement(UserManagement):
+    def __init__(self):
+        self.__admin_client = None
+
     def __create_username(self, user):
         if user.first_name is not None and user.last_name is not None:
             user.username = '%s%s' % (user.first_name[:1], user.last_name)
@@ -224,10 +307,6 @@ class Keycloak():
             pass
 
     def add_user(self, user, bcrypt_password=None, invite=False):
-        '''
-        Adds the given :class:`nomad.datamodel.User` instance to the configured keycloak
-        realm using the keycloak admin API.
-        '''
         from nomad import datamodel
         if not isinstance(user, datamodel.User):
             if 'user_id' not in user:
@@ -318,11 +397,12 @@ class Keycloak():
             created=datetime.fromtimestamp(keycloak_user['createdTimestamp'] / 1000),
             **kwargs)
 
-    def search_user(self, query: str = None, max=1000, **kwargs):
+    def search_user(self, query: str):
+        kwargs = {}
         if query is not None:
-            kwargs['query'] = dict(search=query, max=max)
+            kwargs['query'] = dict(search=query, max=1000)
         else:
-            kwargs['query'] = dict(max=max)
+            kwargs['query'] = dict(max=1000)
         try:
             keycloak_results = self._admin_client.get_users(**kwargs)
         except Exception as e:
@@ -333,16 +413,7 @@ class Keycloak():
             self.__user_from_keycloak_user(keycloak_user)
             for keycloak_user in keycloak_results]
 
-    def get_user(self, user_id: str = None, username: str = None, email: str = None, user=None):
-        '''
-        Retrives all available information about a user from the keycloak admin
-        interface. This must be used to retrieve complete user information, because
-        the info solely gathered from tokens is generally incomplete.
-        '''
-
-        if user is not None and user_id is None:
-            user_id = user.user_id
-
+    def get_user(self, user_id: str = None, username: str = None, email: str = None):
         if username is not None and user_id is None:
             with utils.lnr(logger, 'Could not use keycloak admin client'):
                 user_id = self._admin_client.get_user_id(username)
@@ -388,7 +459,11 @@ class Keycloak():
         return self.__admin_client
 
 
-keycloak = Keycloak()
+user_management: UserManagement
+if config.oasis.uses_central_user_management:
+    user_management = OasisUserManagement()
+else:
+    user_management = KeycloakUserManagement()
 
 
 def reset(remove: bool):
