@@ -166,7 +166,7 @@ from nomad import config, utils
 
 from .metainfo import (
     MSectionBound, Section, Quantity, MSection, MEnum, Datetime, Reference, DefinitionAnnotation,
-    Definition, QuantityReference)
+    Definition, QuantityReference, Unit)
 
 
 class DocumentType():
@@ -370,7 +370,7 @@ class DocumentType():
                 continue
 
             assert not isinstance(annotation, list), \
-                'sub sections can onyl have one elasticsearch annotation'
+                'sub sections can only have one elasticsearch annotation'
             continue_with_auto_include_subsections = auto_include_subsections or (
                 False if annotation is None else annotation.auto_include_subsections)
 
@@ -543,7 +543,7 @@ def get_tokenizer(regex):
     return tokenizer
 
 
-tokenizer_default = get_tokenizer(r'[_\s\.\/]+')
+tokenizer_default = get_tokenizer(r'[_\s\.\/-]+')
 
 
 class Elasticsearch(DefinitionAnnotation):
@@ -621,11 +621,13 @@ class Elasticsearch(DefinitionAnnotation):
             A callable that is used to transform the search input when
             targeting this field. Note that this does not affect the way the
             value is indexed.
+        es_query: The Elasticsearch query type that is used when querying for the annotated
+            quantity, e.g. match, term, match_phrase. Default is 'match'.
 
     Attributes:
         name:
             The name of the quantity (plus additional field if set).
-        search_quantity: The entry type SearchQuantity associated with this annoation.
+        search_quantity: The entry type SearchQuantity associated with this annotation.
     '''
     def __init__(
             self,
@@ -644,6 +646,7 @@ class Elasticsearch(DefinitionAnnotation):
             suggestion: Union[str, Callable[[MSectionBound], Any]] = None,
             variants: Union[Callable[[str], List[str]]] = None,
             normalizer: Callable[[Any], Any] = None,
+            es_query: str = 'match',
             _es_field: str = None):
 
         # TODO remove _es_field if it is not necessary anymore to enforce a specific mapping
@@ -678,6 +681,7 @@ class Elasticsearch(DefinitionAnnotation):
 
         self._custom_mapping = mapping
         self.field = field
+        self.es_query = es_query
         self._es_field = field if _es_field is None else _es_field
         self.doc_type = doc_type
         self.value = value
@@ -736,14 +740,16 @@ class Elasticsearch(DefinitionAnnotation):
                 return dict(type='double')
             elif quantity.type == np.float32:
                 return dict(type='float')
+            elif quantity.type in [np.int64]:
+                return dict(type='long')
             elif quantity.type in [int, np.int32]:
                 return dict(type='integer')
-            elif quantity.type == np.int64:
-                return dict(type='long')
             elif quantity.type == bool:
                 return dict(type='boolean')
             elif quantity.type == Datetime:
                 return dict(type='date')
+            elif quantity.type == Unit:
+                return dict(type='keyword')
             elif isinstance(quantity.type, QuantityReference):
                 return compute_mapping(quantity.type.target_quantity_def)
             elif isinstance(quantity.type, Reference):
@@ -834,19 +840,19 @@ class SearchQuantity():
         if prefix is not None:
             qualified_field = f'{prefix}.{qualified_field}'
 
-        if annotation.field is not None:
-            qualified_field = f'{qualified_field}.{annotation.field}'
-
         if annotation.suggestion:
             qualified_field = f'{qualified_field}__suggestion'
 
-        self.qualified_field = qualified_field
-        self.qualified_name = qualified_field
-
         self.search_field = qualified_field
         if not(annotation._es_field == '' or annotation._es_field is None):
             self.search_field = f'{qualified_field}.{annotation._es_field}'
 
+        if annotation.field is not None:
+            qualified_field = f'{qualified_field}.{annotation.field}'
+
+        self.qualified_field = qualified_field
+        self.qualified_name = qualified_field
+
     @property
     def definition(self):
         return self.annotation.definition
@@ -923,19 +929,22 @@ def index_entries_with_materials(entries: List, refresh: bool = False):
     update_materials(entries, refresh=refresh)
 
 
-def index_entries(entries: List, refresh: bool = False):
+def index_entries(entries: List, refresh: bool = False) -> Dict[str, str]:
     '''
     Upserts the given entries in the entry index. Optionally updates the materials index
-    as well.
+    as well. Returns a dictionary of the format {entry_id: error_message} for all entries
+    that failed to index.
     '''
+    rv = {}
     # split into reasonably sized problems
     if len(entries) > config.elastic.bulk_size:
         for entries_part in [entries[i:i + config.elastic.bulk_size] for i in range(0, len(entries), config.elastic.bulk_size)]:
-            index_entries(entries_part, refresh=refresh)
-        return
+            errors = index_entries(entries_part, refresh=refresh)
+            rv.update(errors)
+        return rv
 
     if len(entries) == 0:
-        return
+        return rv
 
     logger = utils.get_logger('nomad.search', n_entries=len(entries))
 
@@ -963,10 +972,16 @@ def index_entries(entries: List, refresh: bool = False):
         lnr_event='failed to bulk index entries',
         **timer_kwargs
     ):
-        entry_index.bulk(
+        indexing_result = entry_index.bulk(
             body=actions_and_docs, refresh=refresh,
             timeout=f'{config.elastic.bulk_timeout}s',
             request_timeout=config.elastic.bulk_timeout)
+        # Extract only the errors from the indexing_result
+        if indexing_result['errors']:
+            for item in indexing_result['items']:
+                if item['index']['status'] >= 400:
+                    rv[item['index']['_id']] = str(item['index']['error'])
+        return rv
 
 
 def update_materials(entries: List, refresh: bool = False):
