@@ -42,7 +42,7 @@ def resolve_difference(values):
 class TaskNormalizer(Normalizer):
     def __init__(self, entry_archive, workflow_index):
         super().__init__(entry_archive)
-        workflow_index = workflow_index if len(entry_archive.workflow) < workflow_index else -1
+        workflow_index = workflow_index if len(entry_archive.workflow) > workflow_index else -1
         self.workflow = entry_archive.workflow[workflow_index]
         run = self.workflow.run_ref
         self.run = run if run else entry_archive.run[-1]
@@ -117,6 +117,9 @@ class SinglePointNormalizer(TaskNormalizer):
 
         scc = self.run.calculation
         if not scc:
+            if self.section.method is None:
+                # remove single point section
+                self.workflow.m_remove_sub_section(Workflow.single_point, 0)
             return
 
         if not self.section.n_scf_steps:
@@ -195,7 +198,7 @@ class GeometryOptimizationNormalizer(TaskNormalizer):
             if (atom_pos_init.magnitude == atom_pos_final.magnitude).all():
                 return 'static'
 
-            return 'ionic'
+            return 'atomic'
 
     def normalize(self):
         super().normalize()
@@ -248,7 +251,8 @@ class GeometryOptimizationNormalizer(TaskNormalizer):
                     invalid = True
                     break
                 if energy is None:
-                    continue
+                    invalid = True
+                    break
                 energies.append(energy.magnitude)
             if invalid:
                 self.logger.warning("energy not reported for an scc that is part of a geometry optimization")
@@ -398,12 +402,6 @@ class ElasticNormalizer(TaskNormalizer):
 
 
 class MolecularDynamicsNormalizer(TaskNormalizer):
-    def _is_with_thermodynamics(self):
-        try:
-            return len(self.run.calculation[-1].thermodynamics[-1].values()) > 0
-        except Exception:
-            return False
-
     def _is_with_trajectory(self):
         try:
             return self.run.system[-1].atoms.positions is not None
@@ -417,9 +415,6 @@ class MolecularDynamicsNormalizer(TaskNormalizer):
         if not self.section:
             self.section = self.workflow.m_create(MolecularDynamics)
 
-        if self.section.with_thermodynamics is None:
-            self.section.with_thermodynamics = self._is_with_thermodynamics()
-
         if self.section.with_trajectory is None:
             self.section.with_trajectory = self._is_with_trajectory()
 
@@ -454,9 +449,6 @@ class ThermodynamicsNormalizer(TaskNormalizer):
         if self.section.temperature is None:
             set_thermo_property('temperature')
 
-        if self.section.pressure is None:
-            set_thermo_property('pressure')
-
         if self.section.helmholtz_free_energy is None:
             set_thermo_property('helmholtz_free_energy')
 
@@ -515,7 +507,9 @@ class WorkflowNormalizer(Normalizer):
         if not self.entry_archive.run:
             return
 
+        created = False
         if not self.entry_archive.workflow:
+            created = True
             self.entry_archive.m_create(Workflow)
 
         for n, sec_workflow in enumerate(self.entry_archive.workflow):
@@ -555,5 +549,6 @@ class WorkflowNormalizer(Normalizer):
             ThermodynamicsNormalizer(self.entry_archive, n).normalize()
 
             # remove the section workflow again, if the parser/normalizer could not produce a result
-            if sec_workflow.calculation_result_ref is None:
+            # we should not remove workflows that are created by parser
+            if sec_workflow.calculation_result_ref is None and created:
                 self.entry_archive.m_remove_sub_section(EntryArchive.workflow, n)
