@@ -17,15 +17,18 @@
 #
 
 import os.path
-from typing import Dict
+from typing import Tuple, List, Dict
+from collections.abc import Iterable
+import pkgutil
+from pathlib import Path
 
 from nomad import config
-from nomad.datamodel import results
+from nomad.datamodel import EntryArchive, EntryMetadata, results
+from nomad.datamodel.context import Context, ClientContext
 
 from .parser import MissingParser, BrokenParser, Parser, ArchiveParser, MatchingParserInterface
 from .artificial import EmptyParser, GenerateRandomParser, TemplateParser, ChaosParser
-
-from eelsdbparser import EELSDBParser
+from .tabular import TabularDataParser
 
 
 try:
@@ -48,7 +51,7 @@ except ImportError:
     pass
 
 
-def match_parser(mainfile_path: str, strict=True) -> Parser:
+def match_parser(mainfile_path: str, strict=True, parser_name: str = None) -> Tuple[Parser, List[str]]:
     '''
     Performs parser matching. This means it take the given mainfile and potentially
     opens it with the given callback and tries to identify a parser that can parse
@@ -60,18 +63,23 @@ def match_parser(mainfile_path: str, strict=True) -> Parser:
     Arguments:
         mainfile_path: Path to the mainfile
         strict: Only match strict parsers, e.g. no artificial parsers for missing or empty entries.
+        parser_name: Optional, to force the matching to test only a specific parser
 
-    Returns: The parser, or None if no parser could be matched.
+    Returns:
+        A tuple (`parser`, `mainfile_keys`). The `parser` is the matched parser, and
+        `mainfile_keys` defines the keys to use for child entries, if any. If there are
+        no child entries, `mainfile_keys` will be None. If no parser matches, we return
+        (None, None).
     '''
     mainfile = os.path.basename(mainfile_path)
     if mainfile.startswith('.') or mainfile.startswith('~'):
-        return None
+        return None, None
 
     with open(mainfile_path, 'rb') as f:
         compression, open_compressed = _compressions.get(f.read(3), (None, open))
 
     with open_compressed(mainfile_path, 'rb') as cf:  # type: ignore
-        buffer = cf.read(config.parser_matching_size)
+        buffer = cf.read(config.process.parser_matching_size)
 
     mime_type = magic.from_buffer(buffer, mime=True)
 
@@ -91,12 +99,27 @@ def match_parser(mainfile_path: str, strict=True) -> Parser:
                 decoded_buffer = buffer.decode(encoding)
             except Exception:
                 pass
-
-    for parser in parsers:
+    if parser_name:
+        parser = parser_dict.get(parser_name)
+        assert parser is not None, f'parser by the name `{parser_name}` does not exist'
+        parsers_to_check = [parser]
+    else:
+        parsers_to_check = parsers
+    for parser in parsers_to_check:
         if strict and isinstance(parser, (MissingParser, EmptyParser)):
             continue
 
-        if parser.is_mainfile(mainfile_path, mime_type, buffer, decoded_buffer, compression):
+        match_result = parser.is_mainfile(mainfile_path, mime_type, buffer, decoded_buffer, compression)
+        if match_result:
+            if isinstance(match_result, Iterable):
+                assert parser.creates_children, 'Illegal return value - parser does not specify `creates_children`'
+                for mainfile_key in match_result:  # type: ignore
+                    assert mainfile_key and type(mainfile_key) == str, (
+                        f'Child keys must be strings, got {type(mainfile_key)}')
+                mainfile_keys = sorted(match_result)  # type: ignore
+            else:
+                mainfile_keys = None
+
             # potentially convert the file
             if encoding in ['iso-8859-1']:
                 try:
@@ -109,35 +132,108 @@ def match_parser(mainfile_path: str, strict=True) -> Parser:
                         text_file.write(content)
 
             # TODO: deal with multiple possible parser specs
-            return parser
+            return parser, mainfile_keys
 
-    return None
+    return None, None
 
 
+class ParserContext(Context):
+    def __init__(self, mainfile_dir):
+        self._mainfile_dir = mainfile_dir
+
+    def raw_file(self, path, *args, **kwargs):
+        return open(os.path.join(self._mainfile_dir, path), *args, **kwargs)
+
+    def raw_path_exists(self, path: str) -> bool:
+        return os.path.exists(os.path.join(self._mainfile_dir, path))
+
+
+def run_parser(
+        mainfile_path: str, parser: Parser, mainfile_keys: List[str] = None,
+        logger=None, server_context: bool = False,
+        username: str = None, password: str = None) -> List[EntryArchive]:
+    '''
+    Parses a file, given the path, the parser, and mainfile_keys, as returned by
+    :func:`match_parser`, and returns the resulting EntryArchive objects. Parsers that have
+    `create_children == False` (the most common case) will generate a list with a single entry,
+    for parsers that create children the list will consist of the main entry followed by the
+    child entries. The returned archive objects will have minimal metadata.
+    '''
+    directory = os.path.dirname(mainfile_path)
+    if server_context:
+        # TODO this looks totally wrong. ParserContext is not a server context at all.
+        # There should be three different context. Client, Server and Parser. Currently,
+        # ClientContext seems to cover both the client and the local parser use-case.
+        entry_archive = EntryArchive(m_context=ParserContext(directory))
+    else:
+        entry_archive = EntryArchive(
+            m_context=ClientContext(local_dir=directory, username=username, password=password))
+
+    entry_archive.metadata = EntryMetadata()
+    entry_archive.metadata.mainfile = mainfile_path
+
+    entry_archives = [entry_archive]
+    if mainfile_keys:
+        child_archives = {}
+        for mainfile_key in mainfile_keys:
+            child_archive = EntryArchive()
+            child_metadata = child_archive.m_create(EntryMetadata)
+            child_metadata.mainfile = mainfile_path
+            child_metadata.mainfile_key = mainfile_key
+            child_archives[mainfile_key] = child_archive
+            entry_archives.append(child_archive)
+        kwargs = dict(child_archives=child_archives)
+    else:
+        kwargs = {}
+
+    cwd = os.getcwd()
+    try:
+        mainfile_path = os.path.abspath(mainfile_path)
+        os.chdir(os.path.abspath(os.path.dirname(mainfile_path)))
+        parser.parse(mainfile_path, entry_archive, logger=logger, **kwargs)
+    except Exception as e:
+        if logger:
+            logger.error('parsing was not successful', exc_info=e)
+        raise e
+    finally:
+        os.chdir(cwd)
+    for entry_archive in entry_archives:
+        if entry_archive.metadata.domain is None:
+            entry_archive.metadata.domain = parser.domain
+    return entry_archives
+
+
+# Here we resolve the path of the installation directories of various parsers.
+# Note that this should be done in a way that does not yet import the modules
+# themselves (this takes a while). The parser modules are imported lazily later.
+prefix_electronic = f'{Path(pkgutil.get_loader("electronicparsers").path).parent.absolute()}'  # type: ignore
+prefix_atomistic = f'{Path(pkgutil.get_loader("atomisticparsers").path).parent.absolute()}'  # type: ignore
+prefix_workflow = f'{Path(pkgutil.get_loader("workflowparsers").path).parent.absolute()}'  # type: ignore
+prefix_database = f'{Path(pkgutil.get_loader("databaseparsers").path).parent.absolute()}'  # type: ignore
+prefix_eels = f'{Path(pkgutil.get_loader("eelsdbparser").path).parent.absolute()}'  # type: ignore
+
 parsers = [
     GenerateRandomParser(),
     TemplateParser(),
     ChaosParser(),
     MatchingParserInterface(
         'electronicparsers.AbinitParser',
-        name='parsers/abinit', code_name='ABINIT', code_homepage='https://www.abinit.org/',
+        metadata_path=f'{prefix_electronic}/abinit/metadata.yaml',
         mainfile_contents_re=(r'^\n*\.Version\s*[0-9.]*\s*of ABINIT\s*')
     ),
+    MatchingParserInterface(
+        'electronicparsers.AMSParser',
+        metadata_path=f'{prefix_electronic}/ams/metadata.yaml',
+        mainfile_contents_re=r'\* +\| +A M S +\| +\*'
+    ),
     MatchingParserInterface(
         'electronicparsers.ATKParser',
-        name='parsers/atk', code_name='AtomistixToolKit',
-        code_homepage='https://www.synopsys.com/silicon/quantumatk.html',
+        metadata_path=f'{prefix_electronic}/atk/metadata.yaml',
         mainfile_name_re=r'^.*\.nc', mainfile_mime_re=r'application/octet-stream'
     ),
-    MatchingParserInterface(
-        'electronicparsers.BandParser',
-        name='parsers/band', code_name='BAND',
-        code_homepage='https://www.scm.com/product/band_periodicdft/',
-        mainfile_contents_re=r' +\* +Amsterdam Density Functional +\(ADF\)'
-    ),
     MatchingParserInterface(
         'electronicparsers.BigDFTParser',
-        name='parsers/bigdft', code_name='BigDFT', code_homepage='http://bigdft.org/',
+        metadata_path=f'{prefix_electronic}/bigdft/metadata.yaml',
         mainfile_contents_re=(
             # r'__________________________________ A fast and precise DFT wavelet code\s*'
             # r'\|     \|     \|     \|     \|     \|\s*'
@@ -168,18 +264,18 @@ parsers = [
     ),
     MatchingParserInterface(
         'electronicparsers.CastepParser',
-        name='parsers/castep', code_name='CASTEP', code_homepage='http://www.castep.org/',
+        metadata_path=f'{prefix_electronic}/castep/metadata.yaml',
         mainfile_contents_re=(r'\s\|\s*CCC\s*AA\s*SSS\s*TTTTT\s*EEEEE\s*PPPP\s*\|\s*')
     ),
     MatchingParserInterface(
         'electronicparsers.CharmmParser',
-        name='parsers/charmm', code_name='Charmm', domain='dft',
+        metadata_path=f'{prefix_electronic}/charmm/metadata.yaml',
         mainfile_contents_re=r'\s*Chemistry\s*at\s*HARvard\s*Macromolecular\s*Mechanics\s*',
         mainfile_mime_re=r'text/.*'
     ),
     MatchingParserInterface(
         'electronicparsers.CP2KParser',
-        name='parsers/cp2k', code_name='CP2K', code_homepage='https://www.cp2k.org/',
+        metadata_path=f'{prefix_electronic}/cp2k/metadata.yaml',
         mainfile_contents_re=(
             r'\*\*\*\* \*\*\*\* \*\*\*\*\*\*  \*\*  PROGRAM STARTED AT\s.*\n'
             r' \*\*\*\*\* \*\* \*\*\*  \*\*\* \*\*   PROGRAM STARTED ON\s*.*\n'
@@ -189,59 +285,55 @@ parsers = [
     ),
     MatchingParserInterface(
         'electronicparsers.CPMDParser',
-        name='parsers/cpmd', code_name='CPMD',
-        code_homepage='https://www.lcrc.anl.gov/for-users/software/available-software/cpmd/',
+        metadata_path=f'{prefix_electronic}/cpmd/metadata.yaml',
         mainfile_contents_re=(r'\*\*\*       \*\*   \*\*\*  \*\* \*\*\*\* \*\*  \*\*   \*\*\*')
     ),
     MatchingParserInterface(
         'electronicparsers.CrystalParser',
-        name='parsers/crystal',
-        code_name='Crystal',
-        code_homepage='https://www.crystal.unito.it/',
+        metadata_path=f'{prefix_electronic}/crystal/metadata.yaml',
         mainfile_contents_re=(
             fr'(\r?\n \*\s+CRYSTAL[\d]+\s+\*\r?\n \*\s*[a-zA-Z]+ : \d+[\.\d+]*)')
     ),
     MatchingParserInterface(
         'electronicparsers.Dmol3Parser',
-        name='parsers/dmol', code_name='DMol3',
-        code_homepage='http://dmol3.web.psi.ch/dmol3.html', domain='dft',
+        metadata_path=f'{prefix_electronic}/dmol3/metadata.yaml',
         mainfile_name_re=r'.*\.outmol',
         mainfile_contents_re=r'Materials Studio DMol\^3'
     ),
     MatchingParserInterface(
         'electronicparsers.ElkParser',
-        name='parsers/elk', code_name='elk', code_homepage='http://elk.sourceforge.net/',
+        metadata_path=f'{prefix_electronic}/elk/metadata.yaml',
         mainfile_contents_re=r'\| Elk version [0-9.a-zA-Z]+ started \|'
     ),
     MatchingParserInterface(
         'electronicparsers.ExcitingParser',
-        name='parsers/exciting', code_name='exciting', code_homepage='http://exciting-code.org/',
+        metadata_path=f'{prefix_electronic}/exciting/metadata.yaml',
         mainfile_name_re=r'^.*.OUT(\.[^/]*)?$', mainfile_contents_re=(r'EXCITING.*started')
     ),
     MatchingParserInterface(
         'electronicparsers.FHIAimsParser',
-        name='parsers/fhi-aims', code_name='FHI-aims',
-        code_homepage='https://aimsclub.fhi-berlin.mpg.de/',
+        metadata_path=f'{prefix_electronic}/fhiaims/metadata.yaml',
         mainfile_contents_re=(
             r'^(.*\n)*'
             r'?\s*Invoking FHI-aims \.\.\.')
     ),
     MatchingParserInterface(
         'electronicparsers.FleurParser',
-        name='parsers/fleur', code_name='fleur',
-        code_homepage='https://www.flapw.de/', domain='dft',
-        mainfile_contents_re=r'This output is generated by fleur.'
+        metadata_path=f'{prefix_electronic}/fleur/metadata.yaml',
+        mainfile_contents_re=r'This output is generated by fleur.|\<fleurOutput',
+        mainfile_name_re=r'.*[^/]*\.xml[^/]*',  # only the alternative mainfile name should match
+        mainfile_mime_re=r'(application/.*)|(text/.*)',
+        mainfile_alternative=True
     ),
     MatchingParserInterface(
         'electronicparsers.FploParser',
-        name='parsers/fplo', code_name='fplo', domain='dft',
+        metadata_path=f'{prefix_electronic}/fplo/metadata.yaml',
         mainfile_contents_re=r'\s*\|\s*FULL-POTENTIAL LOCAL-ORBITAL MINIMUM BASIS BANDSTRUCTURE CODE\s*\|\s*',
         mainfile_mime_re=r'text/.*'
     ),
     MatchingParserInterface(
         'electronicparsers.GamessParser',
-        name='parsers/gamess', code_name='GAMESS',
-        code_homepage='https://www.msg.chem.iastate.edu/gamess/versions.html',
+        metadata_path=f'{prefix_electronic}/gamess/metadata.yaml',
         mainfile_contents_re=(
             r'\s*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\**\s*'
             r'\s*\*\s*GAMESS VERSION =\s*(.*)\*\s*'
@@ -249,54 +341,55 @@ parsers = [
     ),
     MatchingParserInterface(
         'electronicparsers.GaussianParser',
-        name='parsers/gaussian', code_name='Gaussian', code_homepage='http://gaussian.com/',
-        mainfile_mime_re=r'.*', mainfile_contents_re=(
+        metadata_path=f'{prefix_electronic}/gaussian/metadata.yaml',
+        mainfile_mime_re=r'.*',
+        mainfile_contents_re=(
             r'\s*Cite this work as:'
             r'\s*Gaussian [0-9]+, Revision [A-Za-z0-9\.]*,')
     ),
     MatchingParserInterface(
         'electronicparsers.GPAWParser',
-        name='parsers/gpaw', code_name='GPAW', code_homepage='https://wiki.fysik.dtu.dk/gpaw/',
+        metadata_path=f'{prefix_electronic}/gpaw/metadata.yaml',
         mainfile_name_re=(r'^.*\.(gpw2|gpw)$'),
         mainfile_mime_re=r'application/(x-tar|octet-stream)'
     ),
     MatchingParserInterface(
         'electronicparsers.MolcasParser',
-        name='parsers/molcas', code_name='MOLCAS', code_homepage='http://www.molcas.org/',
-        domain='dft', mainfile_contents_re=r'M O L C A S'
+        metadata_path=f'{prefix_electronic}/molcas/metadata.yaml',
+        mainfile_contents_re=r'M O L C A S'
     ),
     MatchingParserInterface(
         'electronicparsers.MopacParser',
-        name='parsers/mopac', code_name='MOPAC', domain='dft',
+        metadata_path=f'{prefix_electronic}/mopac/metadata.yaml',
         mainfile_contents_re=r'\s*\*\*\s*MOPAC\s*([0-9a-zA-Z]*)\s*\*\*\s*',
         mainfile_mime_re=r'text/.*',
     ),
     MatchingParserInterface(
         'electronicparsers.NWChemParser',
-        name='parsers/nwchem', code_name='NWChem', code_homepage='http://www.nwchem-sw.org/',
+        metadata_path=f'{prefix_electronic}/nwchem/metadata.yaml',
         mainfile_contents_re=(
             r'Northwest Computational Chemistry Package \(NWChem\) (\d+\.)+\d+')
     ),
     MatchingParserInterface(
         'electronicparsers.OctopusParser',
-        name='parsers/octopus', code_name='Octopus', code_homepage='https://octopus-code.org/',
+        metadata_path=f'{prefix_electronic}/octopus/metadata.yaml',
         mainfile_contents_re=(r'\|0\) ~ \(0\) \|')
     ),
     MatchingParserInterface(
         'electronicparsers.OnetepParser',
-        name='parsers/onetep', code_name='ONETEP', code_homepage='https://www.onetep.org/',
-        domain='dft', mainfile_contents_re=r'####### #     # ####### ####### ####### ######'
+        metadata_path=f'{prefix_electronic}/onetep/metadata.yaml',
+        mainfile_contents_re=r'####### #     # ####### ####### ####### ######'
     ),
     MatchingParserInterface(
         'electronicparsers.OpenmxParser',
-        name='parsers/openmx', code_name='OpenMX', code_homepage='http://www.openmx-square.org/',
+        metadata_path=f'{prefix_electronic}/openmx/metadata.yaml',
         mainfile_mime_re=r'(text/.*)',
         mainfile_name_re=r'.*\.out$',
         mainfile_contents_re=(r'^\*{59}\s+\*{59}\s+This calculation was performed by OpenMX'),
     ),
     MatchingParserInterface(
         'electronicparsers.OrcaParser',
-        name='parsers/orca', code_name='ORCA', code_homepage='https://orcaforum.kofo.mpg.de/',
+        metadata_path=f'{prefix_electronic}/orca/metadata.yaml',
         mainfile_contents_re=(
             r'\s+\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\**\s*'
             r'\s+\* O   R   C   A \*\s*'
@@ -306,45 +399,41 @@ parsers = [
     ),
     MatchingParserInterface(
         'electronicparsers.Psi4Parser',
-        name='parsers/psi4', code_name='Psi4', code_homepage='https://psicode.org/',
+        metadata_path=f'{prefix_electronic}/psi4/metadata.yaml',
         mainfile_contents_re=(r'Psi4: An Open-Source Ab Initio Electronic Structure Package')
     ),
     MatchingParserInterface(
         'electronicparsers.QBallParser',
-        name='parsers/qball',
-        code_name='qball',
+        metadata_path=f'{prefix_electronic}/qball/metadata.yaml',
         mainfile_contents_re='qball',
         supported_compressions=["gz", "bz2", "xz"]
     ),
     MatchingParserInterface(
         'electronicparsers.QboxParser',
-        name='parsers/qbox', code_name='qbox', code_homepage='http://qboxcode.org/',
-        domain='dft', mainfile_mime_re=r'(application/xml)|(text/.*)',
+        metadata_path=f'{prefix_electronic}/qbox/metadata.yaml',
+        mainfile_mime_re=r'(application/xml)|(text/.*)',
         mainfile_contents_re=(r'http://qboxcode.org')
     ),
     MatchingParserInterface(
         'electronicparsers.QuantumEspressoParser',
-        name='parsers/quantumespresso', code_name='Quantum Espresso',
-        code_homepage='https://www.quantum-espresso.org/',
+        metadata_path=f'{prefix_electronic}/quantumespresso/metadata.yaml',
         mainfile_contents_re=(r'(Program PWSCF.*starts)|(Current dimensions of program PWSCF are)')
     ),
     MatchingParserInterface(
         'electronicparsers.SiestaParser',
-        name='parsers/siesta', code_name='Siesta',
-        code_homepage='https://departments.icmab.es/leem/siesta/',
+        metadata_path=f'{prefix_electronic}/siesta/metadata.yaml',
         mainfile_contents_re=(
             r'(Siesta Version: siesta-|SIESTA [0-9]\.[0-9]\.[0-9])|'
             r'(\*\s*WELCOME TO SIESTA\s*\*)')
     ),
     MatchingParserInterface(
         'electronicparsers.TurbomoleParser',
-        name='parsers/turbomole', code_name='turbomole',
-        code_homepage='https://www.turbomole.org/',
+        metadata_path=f'{prefix_electronic}/turbomole/metadata.yaml',
         mainfile_contents_re=(r'Copyright \(C\) [0-9]+ TURBOMOLE GmbH, Karlsruhe')
     ),
     MatchingParserInterface(
         'electronicparsers.VASPParser',
-        name='parsers/vasp', code_name='VASP', code_homepage='https://www.vasp.at/',
+        metadata_path=f'{prefix_electronic}/vasp/metadata.yaml',
         mainfile_mime_re=r'(application/.*)|(text/.*)',
         mainfile_name_re=r'.*[^/]*xml[^/]*',  # only the alternative mainfile name should match
         mainfile_contents_re=(
@@ -357,56 +446,55 @@ parsers = [
     ),
     MatchingParserInterface(
         'electronicparsers.Wien2kParser',
-        name='parsers/wien2k', code_name='WIEN2k', code_homepage='http://www.wien2k.at/',
+        metadata_path=f'{prefix_electronic}/wien2k/metadata.yaml',
         mainfile_name_re=r'.*\.scf$',
         mainfile_alternative=True,
         mainfile_contents_re=r'\s*---------\s*:ITE[0-9]+:\s*[0-9]+\.\s*ITERATION\s*---------'
     ),
     MatchingParserInterface(
         'electronicparsers.YamboParser',
-        name='parsers/yambo', code_name='YAMBO', code_homepage='https://yambo-code.org/',
+        metadata_path=f'{prefix_electronic}/yambo/metadata.yaml',
         mainfile_contents_re=(r'Build.+\s+http://www\.yambo-code\.org')
     ),
+    MatchingParserInterface(
+        'electronicparsers.ABACUSParser',
+        metadata_path=f'{prefix_electronic}/abacus/metadata.yaml',
+        mainfile_contents_re=(r'\s*\n\s*WELCOME TO ABACUS')
+    ),
     MatchingParserInterface(
         'atomisticparsers.AmberParser',
-        name='parsers/amber', code_name='Amber', domain='dft',
+        metadata_path=f'{prefix_atomistic}/amber/metadata.yaml',
         mainfile_contents_re=r'\s*Amber\s[0-9]+\s[A-Z]+\s*[0-9]+'
     ),
     MatchingParserInterface(
         'atomisticparsers.AsapParser',
-        name='parsers/asap', code_name='ASAP', domain='dft',
+        metadata_path=f'{prefix_atomistic}/asap/metadata.yaml',
         mainfile_name_re=r'.*.traj$', mainfile_mime_re=r'application/octet-stream'
     ),
-    MatchingParserInterface(
-        'atomisticparsers.BOPfoxParser',
-        name='parsers/bopfox', code_name='BOPfox',
-        mainfile_contents_re=r'\s+\-+\s+BOPfox'
-    ),
     MatchingParserInterface(
         'atomisticparsers.DFTBPlusParser',
-        name='parsers/dftbplus', code_name='DFTB+', domain='dft',
+        metadata_path=f'{prefix_atomistic}/dftbplus/metadata.yaml',
         mainfile_contents_re=r'\|  DFTB\+',
         mainfile_mime_re=r'text/.*'
     ),
     MatchingParserInterface(
         'atomisticparsers.DLPolyParser',
-        name='parsers/dl-poly', code_name='DL_POLY',
-        code_homepage='https://www.scd.stfc.ac.uk/Pages/DL_POLY.aspx',
-        mainfile_contents_re=(r'\*\* DL_POLY \*\*'),
+        metadata_path=f'{prefix_atomistic}/dlpoly/metadata.yaml',
+        mainfile_contents_re=(r'\*\*\s+DL_POLY.+\*\*'),
     ),
     MatchingParserInterface(
         'atomisticparsers.GromacsParser',
-        name='parsers/gromacs', code_name='Gromacs', code_homepage='http://www.gromacs.org/',
-        domain='dft', mainfile_contents_re=r'gmx mdrun, (VERSION|version)'
+        metadata_path=f'{prefix_atomistic}/gromacs/metadata.yaml',
+        mainfile_contents_re=r'gmx mdrun, (VERSION|version)[\s\S]*Input Parameters:'
     ),
     MatchingParserInterface(
         'atomisticparsers.GromosParser',
-        name='parsers/gromos', code_name='Gromos', domain='dft',
+        metadata_path=f'{prefix_atomistic}/gromos/metadata.yaml',
         mainfile_contents_re=r'Bugreports to http://www.gromos.net'
     ),
     MatchingParserInterface(
         'atomisticparsers.GulpParser',
-        name='parsers/gulp', code_name='gulp', code_homepage='http://gulp.curtin.edu.au/gulp/',
+        metadata_path=f'{prefix_atomistic}/gulp/metadata.yaml',
         mainfile_contents_re=(
             r'\s*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*'
             r'\*\*\*\*\*\*\*\*\*\*\*\*\*\s*'
@@ -414,45 +502,28 @@ parsers = [
     ),
     MatchingParserInterface(
         'atomisticparsers.LammpsParser',
-        name='parsers/lammps', code_name='LAMMPS', code_homepage='https://lammps.sandia.gov/',
-        domain='dft', mainfile_contents_re=r'^LAMMPS'
+        metadata_path=f'{prefix_atomistic}/lammps/metadata.yaml',
+        mainfile_contents_re=r'^LAMMPS\s+\(.+\)'
     ),
     MatchingParserInterface(
         'atomisticparsers.LibAtomsParser',
-        name='parsers/lib-atoms', code_name='libAtoms', code_homepage='https://libatoms.github.io/',
+        metadata_path=f'{prefix_atomistic}/libatoms/metadata.yaml',
         mainfile_contents_re=(r'\s*<GAP_params\s'),
     ),
     MatchingParserInterface(
         'atomisticparsers.NAMDParser',
-        name='parsers/namd', code_name='Namd', domain='dft',
+        metadata_path=f'{prefix_atomistic}/namd/metadata.yaml',
         mainfile_contents_re=r'\s*Info:\s*NAMD\s*[0-9.]+\s*for\s*',
         mainfile_mime_re=r'text/.*',
     ),
-    MatchingParserInterface(
-        'atomisticparsers.OpenKIMParser',
-        name='parsers/openkim', code_name='OpenKIM', domain='dft',
-        mainfile_mime_re=r'(application/json)|(text/.*)',
-        mainfile_contents_re=r'openkim|OPENKIM|OpenKIM'
-    ),
     MatchingParserInterface(
         'atomisticparsers.TinkerParser',
-        name='parsers/tinker', code_name='TINKER', domain='dft',
+        metadata_path=f'{prefix_atomistic}/tinker/metadata.yaml',
         mainfile_contents_re=r'TINKER  ---  Software Tools for Molecular Design'
     ),
-    MatchingParserInterface(
-        'atomisticparsers.XTBParser',
-        name='parsers/xtb', code_name='XTB',
-        code_homepage='https://www.chemie.uni-bonn.de/pctc/mulliken-center/software/xtb/xtb',
-        mainfile_contents_re=(
-            r'\s*-----------------------------------------------------------\s*'
-            r'\s*\|                   =====================                   \|\s*'
-            r'\s*\|                           x T B                           \|\s*'
-            r'\s*\|                   =====================                   \|\s*'
-        )
-    ),
     MatchingParserInterface(
         'workflowparsers.AFLOWParser',
-        name='parsers/aflow', code_name='AFlow', code_homepage='http://www.aflowlib.org/',
+        metadata_path=f'{prefix_workflow}/aflow/metadata.yaml',
         mainfile_mime_re=r'(application/json)|(text/.*)',
         mainfile_name_re=r'.*aflowlib\.json.*',  # only the alternative mainfile name should match
         mainfile_contents_re=(
@@ -470,73 +541,127 @@ parsers = [
     ),
     MatchingParserInterface(
         'workflowparsers.ASRParser',
-        name='parsers/asr', code_name='ASR',
-        code_homepage='https://asr.readthedocs.io/en/latest/index.html',
+        metadata_path=f'{prefix_workflow}/asr/metadata.yaml',
         mainfile_mime_re=r'(application/json)|(text/.*)',
         mainfile_name_re=r'.*archive_.*\.json',
         mainfile_contents_re=(r'"name": "ASR"')
     ),
     MatchingParserInterface(
         'workflowparsers.ElasticParser',
-        name='parsers/elastic', code_name='elastic', code_homepage='http://exciting-code.org/elastic',
+        metadata_path=f'{prefix_workflow}/elastic/metadata.yaml',
         mainfile_contents_re=r'\s*Order of elastic constants\s*=\s*[0-9]+\s*',
         mainfile_name_re=(r'.*/INFO_ElaStic')
     ),
     MatchingParserInterface(
         'workflowparsers.FHIVibesParser',
-        name='parsers/fhi-vibes', code_name='FHI-vibes',
-        code_homepage='https://vibes.fhi-berlin.mpg.de/',
-        mainfile_name_re=(r'^.*\.(nc)$'), mainfile_mime_re=r'(application/x-hdf)',
-        mainfile_binary_header_re=br'^\x89HDF'
+        metadata_path=f'{prefix_workflow}/fhivibes/metadata.yaml',
+        mainfile_name_re=(r'^.*\.(nc)$'),
+        mainfile_mime_re=r'(application/x-hdf)',
+        mainfile_binary_header_re=br'^\x89HDF',
+        mainfile_contents_dict={'__has_all_keys': ['I', 'a', 'b']}
     ),
     MatchingParserInterface(
         'workflowparsers.LobsterParser',
-        name='parsers/lobster', code_name='LOBSTER',
-        code_homepage='http://schmeling.ac.rwth-aachen.de/cohp/',
+        metadata_path=f'{prefix_workflow}/lobster/metadata.yaml',
         mainfile_name_re=r'.*lobsterout$',
         mainfile_contents_re=(r'^LOBSTER\s*v[\d\.]+.*'),
     ),
     MatchingParserInterface(
         'workflowparsers.AtomateParser',
-        name='parsers/atomate', code_name='Atomate',
-        code_homepage='https://atomate.org',
+        metadata_path=f'{prefix_workflow}/atomate/metadata.yaml',
         mainfile_mime_re=r'(application/json)|(text/.*)',
         mainfile_name_re=r'.*mp.+materials\.json',
         mainfile_contents_re=(r'"pymatgen_version":')
     ),
     MatchingParserInterface(
         'workflowparsers.PhonopyParser',
-        name='parsers/phonopy', code_name='Phonopy', code_homepage='https://phonopy.github.io/phonopy/',
+        metadata_path=f'{prefix_workflow}/phonopy/metadata.yaml',
         mainfile_name_re=(r'(.*/phonopy-FHI-aims-displacement-0*1/control.in$)|(.*/phon.+yaml)')
     ),
-    EELSDBParser(),
+    MatchingParserInterface(
+        'eelsdbparser.EELSDBParser',
+        metadata_path=f'{prefix_eels}/metadata.yaml',
+        mainfile_mime_re=r'application/json',
+        mainfile_contents_re=(r'https://eelsdb.eu/spectra')
+    ),
+    MatchingParserInterface(
+        'workflowparsers.MOFStructuresParser',
+        metadata_path=f'{prefix_workflow}/mofstructures/metadata.yaml',
+        mainfile_mime_re=r'(application/json)|(text/.*)',
+        mainfile_name_re=r'.*mof_.*\.json',
+        mainfile_contents_re=r'MOF Structures'
+    ),
+    MatchingParserInterface(
+        'workflowparsers.QuantumEspressoPhononParser',
+        metadata_path=f'{prefix_workflow}/quantum_espresso_phonon/metadata.yaml',
+        mainfile_contents_re=(
+            r'Program PHONON.+\s*'
+            r'This program is part of the open-source Quantum ESPRESSO suite')
+    ),
+    MatchingParserInterface(
+        'workflowparsers.QuantumEspressoEPWParser',
+        metadata_path=f'{prefix_workflow}/quantum_espresso_epw/metadata.yaml',
+        mainfile_contents_re=(
+            r'Program EPW.+\s*'
+            r'This program is part of the open-source Quantum ESPRESSO suite')
+    ),
+    MatchingParserInterface(
+        'databaseparsers.OpenKIMParser',
+        metadata_path=f'{prefix_database}/openkim/metadata.yaml',
+        mainfile_mime_re=r'(application/json)|(text/.*)',
+        mainfile_contents_re=r'openkim|OPENKIM|OpenKIM'
+    ),
+    MatchingParserInterface(
+        'electronicparsers.Wannier90Parser',
+        metadata_path=f'{prefix_electronic}/wannier90/metadata.yaml',
+        mainfile_contents_re=r'\|\s*WANNIER90\s*\|'
+    ),
+    MatchingParserInterface(
+        'electronicparsers.W2DynamicsParser',
+        metadata_path=f'{prefix_electronic}/w2dynamics/metadata.yaml',
+        mainfile_name_re=(r'^.*\.(h5|hdf5)$'),
+        mainfile_mime_re=r'(application/x-hdf)',
+        mainfile_binary_header_re=br'^\x89HDF[\s\S]*general.nat'
+    ),
+    MatchingParserInterface(
+        'electronicparsers.SolidDMFTParser',
+        metadata_path=f'{prefix_electronic}/soliddmft/metadata.yaml',
+        mainfile_name_re=(r'^.*\.(h5|hdf5)$'),
+        mainfile_mime_re=r'(application/x-hdf)',
+        mainfile_binary_header_re=br'^\x89HDF',
+        mainfile_contents_dict={'__has_all_keys': ['dft_input', 'DMFT_input', 'DMFT_results']}
+    ),
+    MatchingParserInterface(
+        'nomad.parsing.nexus.NexusParser',
+        metadata_path=os.path.join(os.path.dirname(__file__), 'metadata.yaml'),
+        mainfile_mime_re=r'(application/.*)|(text/.*)',
+        mainfile_name_re=r'.*\.nxs',
+        supported_compressions=['gz', 'bz2', 'xz']),
+
+    TabularDataParser(),
     ArchiveParser()
 ]
 
 empty_parsers = [
     EmptyParser(
         name='missing/octopus', code_name='Octopus', code_homepage='https://octopus-code.org/',
-        domain='dft',
         mainfile_name_re=r'(inp)|(.*/inp)'
     ),
     EmptyParser(
-        name='missing/crystal', code_name='Crystal', code_homepage='https://www.crystal.unito.it/index.php',
-        domain='dft',
+        name='missing/crystal', code_name='CRYSTAL', code_homepage='https://www.crystal.unito.it/index.php',
         mainfile_name_re=r'.*\.cryst\.out'
     ),
     EmptyParser(
         name='missing/wien2k', code_name='WIEN2k', code_homepage='http://www.wien2k.at/',
-        domain='dft',
         mainfile_name_re=r'.*\.scf'
     ),
     EmptyParser(
         name='missing/fhi-aims', code_name='FHI-aims', code_homepage='https://aimsclub.fhi-berlin.mpg.de/',
-        domain='dft',
         mainfile_name_re=r'.*\.fhiaims'
     )
 ]
 
-if config.use_empty_parsers:
+if config.process.use_empty_parsers:
     # There are some entries with PIDs that have mainfiles which do not match what
     # the actual parsers expect. We use the EmptyParser to produce placeholder entries
     # to keep the PIDs. These parsers will not match for new, non migrated data.
@@ -559,15 +684,20 @@ parser_dict['parser/onetep'] = parser_dict['parsers/onetep']
 
 # register code names as possible statistic value to the dft datamodel
 code_names = []
+code_metadata = {}
 for parser in parsers:
+    code_name = getattr(parser, 'code_name', None)
     if parser.domain == 'dft' and \
-            getattr(parser, 'code_name', None) is not None and \
-            getattr(parser, 'code_name') != 'currupted mainfile' and \
-            getattr(parser, 'code_name') != 'Template':
-        code_names.append(getattr(parser, 'code_name'))
+            code_name is not None and \
+            code_name != 'currupted mainfile' and \
+            code_name != 'Template':
+        code_names.append(code_name)
+        if parser.metadata:
+            code_metadata[code_name] = parser.metadata.dict()
+        else:
+            code_metadata[code_name] = {}
 code_names = sorted(set(code_names), key=lambda code_name: code_name.lower())
-results.Simulation.program_name.a_elasticsearch[0].values = code_names + [
-    config.services.unavailable_value, config.services.not_processed_value]
+results.Simulation.program_name.a_elasticsearch[0].values = code_names + [config.services.unavailable_value]
 
 
 def import_all_parsers():
