@@ -16,7 +16,7 @@
 # limitations under the License.
 #
 
-from typing import Any, Tuple, List, Dict
+from typing import Any, Tuple, List, Dict, NamedTuple
 import logging
 import time
 import os
@@ -86,6 +86,8 @@ if config.celery.routing == config.CELERY_WORKER_ROUTING:
     app.conf.update(worker_direct=True)
 
 app.conf.task_queue_max_priority = 10
+app.conf.worker_redirect_stdouts = config.process.redirect_stdouts
+app.conf.worker_redirect_stdouts_level = "INFO"
 
 
 class ProcessStatus:
@@ -146,6 +148,14 @@ class ProcessFailure(Exception):
         self._kwargs = kwargs
 
 
+class ProcessFlags(NamedTuple):
+    ''' Flags defined by the @process and @process_local decorators. '''
+    is_blocking: bool
+    clear_queue_on_failure: bool
+    is_child: bool
+    is_local: bool
+
+
 class Proc(Document):
     '''
     Base class for objects that are subject to processing and need persistent processing
@@ -234,6 +244,12 @@ class Proc(Document):
         ''' Returns a mongoengine query dict (to be used in objects) to find running processes. '''
         return dict(process_status__in=ProcessStatus.STATUSES_PROCESSING)
 
+    @property
+    def current_process_flags(self) -> ProcessFlags:
+        if not self.current_process:
+            return None
+        return process_flags[self.__class__.__name__][self.current_process]
+
     @property
     def queue_blocked(self) -> bool:
         '''
@@ -242,11 +258,13 @@ class Proc(Document):
         mongo, not neccesarily the current state.
         '''
         if self.process_status in ProcessStatus.STATUSES_PROCESSING:
-            blocking_processes = all_blocking_processes[self.__class__.__name__]
-            if self.current_process in blocking_processes:
+            # Check current process
+            if self.current_process_flags.is_blocking:
                 return True
+            # Check queued processes
             for item in self.queue:
-                if item[0] in blocking_processes:
+                func_name = item[0]
+                if process_flags[self.__class__.__name__][func_name].is_blocking:
                     return True
         return False
 
@@ -358,6 +376,7 @@ class Proc(Document):
 
         failed_with_exception = False
 
+        # Log the error
         logger = self.get_logger(**kwargs)
         self.errors = []
         for error in errors:
@@ -370,12 +389,12 @@ class Proc(Document):
             else:
                 self.errors.append(str(error))
 
-        self.complete_time = datetime.utcnow()
-
         if not failed_with_exception:
             errors_str = "; ".join([str(error) for error in errors])
             Proc.log(logger, log_level, 'process failed', errors=errors_str)
 
+        self.complete_time = datetime.utcnow()
+
         try:
             self.on_fail()
         except Exception as e:
@@ -388,7 +407,7 @@ class Proc(Document):
 
         self.process_status = ProcessStatus.FAILURE
         if complete:
-            self._sync_complete_process()
+            self._sync_complete_process(force_clear_queue_on_failure=True)
 
     def warning(self, *warnings, log_level=logging.WARNING, **kwargs):
         ''' Allows to save warnings. Takes strings or exceptions as args. '''
@@ -530,7 +549,7 @@ class Proc(Document):
         is running or has been added to the queue (blocking processes prevent further requests
         to be queued until they have been completed).
 
-        This is the first of two *sync operations*. These should be atomic and occur in
+        This is the first of three *sync operations*. These should be atomic and occur in
         sequence, each call fully seeing the relevant changes of the previous operation.
         Because of propagation delays and possible race conditions, we use `sync_counter` to
         detect update collisions. Such collisions should be very unusual, but if they occur
@@ -572,7 +591,7 @@ class Proc(Document):
                     ]
                 }, mongo_update)
             try_counter += 1
-            if old_record and old_record['sync_counter'] == self.sync_counter:
+            if old_record and old_record.get('sync_counter', 0) == self.sync_counter:
                 # We have successfully scheduled the process!
                 self.reload()
                 return not prev_process_running
@@ -584,21 +603,74 @@ class Proc(Document):
             time.sleep(0.1)
             self.reload()
 
-    def _sync_complete_process(self) -> Tuple[str, List[Any], Dict[str, Any]]:
+    def _sync_start_local_process(self, func_name: str):
         '''
-        Used to complete a process (when done, successful or not). Should only be invoked
-        by a celery worker. Returns a triple containing information about the next process
-        to run (if any), of the form (func_name, args, kwargs).
+        Used to start a *local* process. If successful, the status transitions to RUNNING
+        atomically. The call will fail and raise a :class:`ProcessAlreadyRunning` if any
+        other process is currently running.
+
+        This is one of three *sync operations*. See :func:`_sync_schedule_process`
+        for more info.
+        '''
+        try_counter = 0
+        while True:
+            if self.process_running:
+                raise ProcessAlreadyRunning('Another process is running or waiting to run')
+            mongo_update = {
+                '$set': dict(
+                    sync_counter=self.sync_counter + 1,
+                    process_status=ProcessStatus.RUNNING,
+                    current_process=func_name,
+                    last_status_message='Started: ' + func_name,
+                    worker_hostname=None,
+                    celery_task_id=None,
+                    errors=[],
+                    warnings=[])}
+            # Try to update self atomically. Will fail if someone else has managed to write
+            # a sync op in between.
+            old_record = self._get_collection().find_one_and_update(
+                {
+                    '$and': [
+                        {'_id': self.id},
+                        {
+                            '$or': [
+                                {'sync_counter': self.sync_counter},
+                                {'sync_counter': {'$exists': False}}
+                            ]
+                        }
+                    ]
+                }, mongo_update)
+            try_counter += 1
+            if old_record and old_record.get('sync_counter') == self.sync_counter:
+                # We have successfully started the process!
+                self.reload()
+                return
+            # Someone else must have written a sync op (ticked up the sync_counter) in between
+            if try_counter >= 3:
+                # Three failed attempts - should be virtually impossible!
+                raise ProcessSyncFailure('Failed to start local process too many times - should not happen')
+            # Otherwise, sleep, reload, and try again
+            time.sleep(0.1)
+            self.reload()
+
+    def _sync_complete_process(self, force_clear_queue_on_failure=False) -> Tuple[str, List[Any], Dict[str, Any]]:
+        '''
+        Used to complete a process (when done, successful or not). Returns a triple
+        containing information about the next process to run (if any), of the
+        form (func_name, args, kwargs).
 
         There are 3 possibilities:
             1)  There is something in the queue, and the current process was successful
                 -> We set status to PENDING and return the next process.
             2)  There is something in the queue, and the current process FAILED:
-                -> We clear the queue, set status to FAILURE and return None.
+                -> Behaviour depends on the process decorator flag `clear_queue_on_failure`
+                   and the parameter `force_clear_queue_on_failure`:
+                        If either is True: We clear the queue, set status to FAILURE and return None.
+                        Otherwise: We set status to PENDING and return the next process.
             3)  There is nothing in the queue:
                 -> We set the status to the provided value and return None
 
-        This is one of two *sync operations*. See :func:`_sync_schedule_process`
+        This is one of three *sync operations*. See :func:`_sync_schedule_process`
         for more info.
         '''
         assert self.process_status in ProcessStatus.STATUSES_COMPLETED
@@ -608,13 +680,16 @@ class Proc(Document):
         self.process_status = ProcessStatus.RUNNING
         self.save()
         self.process_status = process_status
-
+        clear_queue_on_failure = (
+            force_clear_queue_on_failure or self.current_process_flags.clear_queue_on_failure)
         try_counter = 0
         while True:
             next_process = None
             mongo_update = {'$set': {'sync_counter': self.sync_counter + 1}}
             if self.queue:
-                if process_status == ProcessStatus.SUCCESS:
+                # Something in the queue
+                if not clear_queue_on_failure or process_status == ProcessStatus.SUCCESS:
+                    # Move on to the next process
                     next_process = self.queue[0]
                     next_func_name = next_process[0]
                     mongo_update['$pop'] = {'queue': -1}  # pops the first element
@@ -623,7 +698,7 @@ class Proc(Document):
                         last_status_message='Pending: ' + next_func_name,
                         current_process=next_func_name)
                 else:
-                    # Failed - clear the queue
+                    # Failed and clear_queue_on_failure is set to True - clear the queue
                     mongo_update['$set'].update(process_status=process_status, queue=[])
             else:
                 mongo_update['$set'].update(process_status=process_status)
@@ -632,7 +707,7 @@ class Proc(Document):
             old_record = self._get_collection().find_one_and_update(
                 {'_id': self.id, 'sync_counter': self.sync_counter}, mongo_update)
             try_counter += 1
-            if old_record and old_record['sync_counter'] == self.sync_counter:
+            if old_record and old_record.get('sync_counter') == self.sync_counter:
                 # We have successfully completed the process
                 return next_process
             # Someone else must have written a sync op (ticked up the sync_counter) in between
@@ -653,8 +728,8 @@ def all_subclasses(cls):
 all_proc_cls = {cls.__name__: cls for cls in all_subclasses(Proc)}
 ''' Name dictionary for all Proc classes. '''
 
-all_blocking_processes: Dict[str, List[str]] = defaultdict(list)
-''' { <proc class name>: <list of all blocking processes defined for this class> } '''
+process_flags: Dict[str, Dict[str, ProcessFlags]] = defaultdict(dict)
+''' { <Proc class name>: { <process func name>: ProcessFlags } } '''
 
 
 class NomadCeleryRequest(Request):
@@ -714,7 +789,7 @@ def unwarp_task(task, cls_name, self_id, *args, **kwargs):
         cls = all_proc_cls.get(cls_name, None)
 
     if cls is None:
-        logger.critical('document not a subcass of Proc')
+        logger.critical('document not a subclass of Proc')
         raise ProcNotRegistered('document %s not a subclass of Proc' % cls_name)
 
     # get the process instance
@@ -722,15 +797,10 @@ def unwarp_task(task, cls_name, self_id, *args, **kwargs):
         try:
             self = cls.get(self_id)
         except KeyError as e:
-            from nomad.app import flask
-            if flask.app.config['TESTING']:
-                # This only happens in tests, where it is not always avoidable that
-                # tasks from old test-cases bleed over.
-                raise ProcObjectDoesNotExist()
             logger.warning('called object is missing, retry')
             raise task.retry(exc=e, countdown=3)
     except KeyError:
-        logger.critical('called object is missing, retries exeeded', proc_id=self_id)
+        logger.critical('called object is missing, retries exceeded', proc_id=self_id)
         raise ProcObjectDoesNotExist()
 
     return self
@@ -768,7 +838,6 @@ def proc_task(task, cls_name, self_id, func_name, args, kwargs):
 
     # unwrap the process decorator
     unwrapped_func = getattr(func, '__process_unwrapped', None)
-    is_child = getattr(func, '__is_child', False)
     if unwrapped_func is None:  # "Should not happen"
         logger.error('called function was not decorated with @process')
         proc.fail('called function %s was not decorated with @process' % func_name)
@@ -776,6 +845,7 @@ def proc_task(task, cls_name, self_id, func_name, args, kwargs):
 
     # call the process function
     try:
+        is_child = process_flags[cls_name][func_name].is_child
         os.chdir(config.fs.working_directory)
         with utils.timer(logger, 'process executed on worker', log_memory=True):
             # Set state to RUNNING
@@ -830,9 +900,15 @@ def proc_task(task, cls_name, self_id, func_name, args, kwargs):
 
     # The proc is done running
     if is_child and proc.process_status in ProcessStatus.STATUSES_COMPLETED:
-        # Save and "switch" to the parent to try to join.
         try:
-            proc._sync_complete_process()  # Child processes are blocking, so their queues should be empty
+            next_process = proc._sync_complete_process()
+            if next_process:
+                # More jobs in the queue
+                func_name, args, kwargs = next_process
+                proc._send_to_worker(func_name, *args, **kwargs)
+                return
+            # Processing finished (successful or not)
+            # Switch to the parent to try to join.
             proc = proc.parent()
             logger = proc.get_logger()
             try_to_join = True
@@ -890,7 +966,7 @@ def proc_task(task, cls_name, self_id, func_name, args, kwargs):
             proc.fail(e)
 
 
-def process(is_blocking: bool = False, is_child: bool = False):
+def process(is_blocking: bool = False, clear_queue_on_failure: bool = True, is_child: bool = False):
     '''
     The decorator for process functions that will be queued up and executed async via celery.
     To transfer state, the instance will be saved to the database and loading on
@@ -903,27 +979,97 @@ def process(is_blocking: bool = False, is_child: bool = False):
             If True, this is a *blocking process*. After a blocking process has been scheduled,
             no other processes can be scheduled, until this process has finished. Attempts to
             invoke another process will in this case result in an exception.
+        clear_queue_on_failure:
+            If True and the process fails, we'll clear the queue, thus skipping any pending jobs.
         is_child:
-            If this is a child process, which should try to join with the parent process when done.
-            Child processes are implicitly blocking.
+            If this is a child process, which should try to join with the parent process when done
+            (= when the queue is empty).
     '''
-    if is_child:
-        is_blocking = True
-
     def process_decorator(func):
         # Determine canonical class name
         cls_name, func_name = func.__qualname__.split('.')
-        if is_blocking:
-            all_blocking_processes[cls_name].append(func_name)
+
+        process_flags[cls_name][func_name] = ProcessFlags(
+            is_blocking,
+            clear_queue_on_failure,
+            is_child,
+            is_local=False)
 
         @functools.wraps(func)
         def wrapper(self: Proc, *args, **kwargs):
             kwargs['_meta_label'] = config.meta.label
             send_to_worker = self._sync_schedule_process(func_name, *args, **kwargs)
             if send_to_worker:
-                self._send_to_worker(func_name, *args, **kwargs)
+                try:
+                    self._send_to_worker(func_name, *args, **kwargs)
+                except Exception as e:
+                    self.fail(e)
+                    raise
 
         setattr(wrapper, '__process_unwrapped', func)
-        setattr(wrapper, '__is_child', is_child)
         return wrapper
     return process_decorator
+
+
+def process_local(func):
+    '''
+    The decorator for functions that process locally. These work similarly to functions
+    marked with the `@process` decorator, but they are executed directly, in the current
+    thread, not via celery. Consequently, they can only be started if no other process is
+    running. They are also implicitly blocking, i.e. while running, no other process
+    (local or celery-based) can be started or scheduled on the same object. It can also not
+    spawn child processes and wait for them using the WAITING_FOR_RESULT mechanism, or itself
+    be a child process (as this means joining with a parent process when done).
+
+    If successful, a local process can return a value to the caller (unlike celery processes).
+    If unsuccessful, an exception will be raised (note that the usual process handling is
+    always applied, i.e. we set self.process_status, self.errors etc. accordingly). The Proc
+    object should not have any unsaved changes when a local process is invoked.
+    '''
+    # Determine canonical class name
+    cls_name, func_name = func.__qualname__.split('.')
+
+    process_flags[cls_name][func_name] = ProcessFlags(
+        is_blocking=True,
+        clear_queue_on_failure=False,  # Not relevant, since local processes are always blocking
+        is_child=False,
+        is_local=True)
+
+    def wrapper(self: Proc, *args, **kwargs):
+        logger = self.get_logger()
+        logger.debug('Executing local process')
+        self._sync_start_local_process(func_name)
+
+        try:
+            os.chdir(config.fs.working_directory)
+            with utils.timer(logger, 'process executed locally', log_memory=True):
+                # Actually call the process function
+                rv = func(self, *args, **kwargs)
+                if self.errors:
+                    # Should be impossible unless the process has tampered with self.errors, which
+                    # it should not do. We will treat it essentially as if it had raised an exception
+                    raise RuntimeError('completed with errors but no exception, should not happen')
+                # All looks good
+                self.on_success()
+                self.process_status = ProcessStatus.SUCCESS
+                self.complete_time = datetime.utcnow()
+                if self.warnings:
+                    self.last_status_message = f'Process {func_name} completed with warnings'
+                else:
+                    self.last_status_message = f'Process {func_name} completed successfully'
+                logger.info('completed process')
+                return rv
+        except SystemExit as e:
+            self.fail(e, complete=False)
+            raise
+        except ProcessFailure as e:
+            # Exception with details about how to call self.fail
+            self.fail(*e._errors, log_level=e._log_level, complete=False, **e._kwargs)
+            raise
+        except Exception as e:
+            self.fail(e, complete=False)
+            raise
+        finally:
+            self._sync_complete_process()  # Queue should be empty, so nothing more to do
+
+    return wrapper
