@@ -38,7 +38,7 @@ Depending on the configuration all logs will also be send to a central logstash.
 .. autofunc::nomad.utils.strip
 '''
 
-from typing import List, Iterable
+from typing import List, Iterable, Union, Any, Dict
 from collections import OrderedDict
 from functools import reduce
 import base64
@@ -53,8 +53,9 @@ import collections
 import logging
 import inspect
 import orjson
-import resource
 import os
+import unicodedata
+import re
 
 from nomad import config
 
@@ -93,7 +94,7 @@ except ImportError:
     def get_logger(name, **kwargs):
         return ClassicLogger(name, **kwargs)
 
-    def configure_logging(console_log_level=config.console_log_level):
+    def configure_logging(console_log_level=config.services.console_log_level):
         import logging
         logging.basicConfig(level=console_log_level)
 
@@ -150,6 +151,21 @@ def decode_handle_id(handle_str: str):
     return result
 
 
+def generate_entry_id(upload_id: str, mainfile: str, mainfile_key: str = None) -> str:
+    '''
+    Generates an id for an entry.
+    Arguments:
+        upload_id: The id of the upload
+        mainfile: The mainfile path (relative to the raw directory).
+        mainfile_key: Optional additional key for mainfiles that represent many entries.
+    Returns:
+        The generated entry id
+    '''
+    if mainfile_key:
+        return hash(upload_id, mainfile, mainfile_key)
+    return hash(upload_id, mainfile)
+
+
 def hash(*args, length: int = default_hash_len) -> str:
     ''' Creates a websafe hash of the given length based on the repr of the given arguments. '''
     hash = hashlib.sha512()
@@ -233,7 +249,10 @@ def timer(logger, event, method='info', lnr_event: str = None, log_memory: bool
         The method yields a dictionary that can be used to add further log data.
     '''
     def get_rss():
-        return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
+        if os.name != 'nt':
+            import resource
+            return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
+        return 0
 
     kwargs = dict(kwargs)
     start = time.time()
@@ -510,6 +529,63 @@ def flat(obj, prefix=None):
 def deep_get(dictionary, *keys):
     '''
     Helper that can be used to access nested dictionary-like containers using a
-    series of paths given as arguments.
+    series of paths given as arguments. The path can contain dictionary string
+    keys or list indices as integer numbers.
     '''
-    return reduce(lambda d, key: d.get(key) if d else None, keys, dictionary)
+    return reduce(lambda d, key: d[key] if isinstance(key, int) else d.get(key) if d else None, keys, dictionary)
+
+
+def slugify(value):
+    '''
+    Taken from https://github.com/django/django/blob/master/django/utils/text.py
+    Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
+    dashes to single dashes. Remove characters that aren't alphanumerics,
+    underscores, or hyphens. Convert to lowercase. Also strip leading and
+    trailing whitespace, dashes, and underscores.
+    '''
+
+    value = str(value)
+    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
+    value = re.sub(r'[^\w\s-]', '', value.lower())
+    return re.sub(r'[-\s]+', '-', value).strip('-_')
+
+
+def query_list_to_dict(path_list: List[Union[str, int]], value: Any) -> Dict[str, Any]:
+    '''Transforms a list of path fragments into a dictionary query. E.g. the list
+
+    ['run', 0, 'system', 2, 'atoms']
+
+    would be converted into
+
+    {
+        'run:0': {
+            'system:2': {
+                'atoms': <value>
+            }
+        }
+    }
+
+    Args:
+        path_list: List of path fragments
+        value: The value to place inside the final dictionary key
+
+    Returns:
+        A nested dictionary representing the query path.
+
+    '''
+    returned: Dict[str, Any] = {}
+    current = returned
+    n_items = len(path_list)
+    i = 0
+    while i < n_items:
+        name = path_list[i]
+        index = None if i == n_items - 1 else path_list[i + 1]
+        if isinstance(index, int):
+            i += 1
+        else:
+            index = None
+        key = f'{name}{f"[{index}]" if index is not None else ""}'
+        current[key] = value if i == n_items - 1 else {}
+        current = current[key]
+        i += 1
+    return returned
